@@ -18,10 +18,17 @@
 from __future__ import with_statement
 
 import contextlib
+import itertools
 import logging
 import os
 import shutil
-import itertools
+import sys
+
+from twisted.internet import defer
+from twisted.trial.unittest import TestCase as TwistedTestCase
+from ubuntuone.devtools.testcase import skipIfOS
+from zope.interface import implements
+from zope.interface.verify import verifyObject
 
 from ubuntuone.syncdaemon import (
     config,
@@ -35,12 +42,16 @@ from ubuntuone.syncdaemon import (
     tritcask,
 )
 from ubuntuone.syncdaemon import logger
-logger.init()
-from twisted.internet import defer
-from twisted.trial.unittest import TestCase as TwistedTestCase
-from zope.interface import implements
-from zope.interface.verify import verifyObject
+from ubuntuone.platform import (
+    can_write,
+    make_dir,
+    path_exists,
+    set_dir_readonly,
+    set_dir_readwrite,
+    stat_path,
+)
 
+logger.init()
 
 FAKED_CREDENTIALS = {'consumer_key': 'faked_consumer_key',
                      'consumer_secret': 'faked_consumer_secret',
@@ -81,7 +92,7 @@ class FakeHashQueue(object):
     def insert(self, path, mdid):
         """Fake insert."""
         self.eq.push('HQ_HASH_NEW', path=path, hash='',
-                     crc32='', size=0, stat=os.stat(path))
+                     crc32='', size=0, stat=stat_path(path))
 
 
 class FakeMark(object):
@@ -139,7 +150,7 @@ class FakeActionQueue(object):
         """Stub implementation."""
 
     def upload(self, share_id, node_id, previous_hash, hash, crc32,
-               size, path, fileobj_factory, tempfile_factory=None):
+               size, path, fileobj_factory):
         """Stub implementation."""
 
     # IMetaQueue
@@ -184,7 +195,7 @@ class FakeMain(main.Main):
     _sync_class = None
 
     # don't call Main.__init__ we take care of creating a fake main and
-    # all its attributes. pylint: disable-msg=W0231
+    # all its attributes. pylint: disable=W0231
     def __init__(self, root_dir, shares_dir, data_dir, partials_dir):
         """ create the instance. """
         self.logger = logging.getLogger('ubuntuone.SyncDaemon.FakeMain')
@@ -211,6 +222,8 @@ class FakeMain(main.Main):
         self.external = FakeExternalInterface()
         self.lr = local_rescan.LocalRescan(self.vm, self.fs,
                                            self.event_q, self.action_q)
+
+        self.eventlog_listener = None
         self.status_listener = FakeStatusListener()
 
     def _connect_aq(self, _):
@@ -253,65 +266,98 @@ class BaseTwistedTestCase(TwistedTestCase):
 
     This class provides:
         mktemp(name): helper to create temporary dirs
-        rmtree(path): support read-only shares
+        mmtree(path): support read-only shares
         makedirs(path): support read-only shares
     """
 
     def mktemp(self, name='temp'):
         """ Customized mktemp that accepts an optional name argument. """
         tempdir = os.path.join(self.tmpdir, name)
-        if os.path.exists(tempdir):
+        if path_exists(tempdir):
             self.rmtree(tempdir)
         self.makedirs(tempdir)
+        self.addCleanup(self.rmtree, tempdir)
+        assert isinstance(tempdir, str)
         return tempdir
 
     @property
     def tmpdir(self):
         """Default tmpdir: module/class/test_method."""
         # check if we already generated the root path
-        root_dir = getattr(self, '__root', None)
-        if root_dir:
-            return root_dir
-        MAX_FILENAME = 32 # some platforms limit lengths of filenames
-        base = os.path.join(self.__class__.__module__[:MAX_FILENAME],
-                            self.__class__.__name__[:MAX_FILENAME],
-                            self._testMethodName[:MAX_FILENAME])
-        # use _trial_temp dir, it should be os.gwtcwd()
-        # define the root temp dir of the testcase, pylint: disable-msg=W0201
-        self.__root = os.path.join(os.getcwd(), base)
+        if self.__root:
+            return self.__root
+        base = os.path.join(self.__class__.__module__,
+                            self.__class__.__name__,
+                            self._testMethodName)
+        # use _trial_temp dir, it should be TRIAL_TEMP_DIR or os.getcwd()
+        # define the root temp dir of the testcase, pylint: disable=W0201
+        root_tmp = os.environ.get('TRIAL_TEMP_DIR', os.getcwd())
+        self.__root = os.path.join(root_tmp, '_trial_temp', base)
+        self.addCleanup(self.rmtree, self.__root)
         return self.__root
 
     def rmtree(self, path):
         """Custom rmtree that handle ro parent(s) and childs."""
-        if not os.path.exists(path):
+        assert isinstance(path, str)
+        # on windows the paths cannot be removed because the process running
+        # them has the ownership and therefore are locked.
+        if not path_exists(path):
             return
         # change perms to rw, so we can delete the temp dir
-        if path != getattr(self, '__root', None):
-            os.chmod(os.path.dirname(path), 0755)
-        if not os.access(path, os.W_OK):
-            os.chmod(path, 0755)
-        # pylint: disable-msg=W0612
+        if path != self.__root:
+            set_dir_readwrite(os.path.dirname(path))
+        if not can_write(path):
+            set_dir_readwrite(path)
+
+        if sys.platform == 'win32':
+            # path is a byte sequence encoded with utf8. If we pass this to
+            # os.walk, in windows, we'll get results encoded with mbcs
+            path = path.decode('utf8')
+
         for dirpath, dirs, files in os.walk(path):
-            for dir in dirs:
-                if not os.access(os.path.join(dirpath, dir), os.W_OK):
-                    os.chmod(os.path.join(dirpath, dir), 0777)
-        shutil.rmtree(path)
+            for adir in dirs:
+                adir = os.path.join(dirpath, adir)
+                if sys.platform == 'win32':
+                    assert isinstance(adir, unicode)
+                    adir = adir.encode('utf8')
+                if not can_write(adir):
+                    set_dir_readwrite(adir)
+
+        if sys.platform == 'win32':
+            # in windows, we need to pass a unicode, literal path to
+            # shutil.rmtree, otherwise we can't remove "deep and wide" paths
+            path = u'\\\\?\\' + path.decode('utf8')
+
+        # Instead of ignoring the errors when removing trees, we are temporarly
+        # printing a message to stdout to caught everyone's attention.
+        # Once the tests are fixed in this regard, we're removing the
+        # try-except block and having shutil.rmtree failing if the path
+        # can not be removed.
+        try:
+            shutil.rmtree(path)
+        except Exception, e:
+            print 'ERROR!! could not recursively remove %r ' \
+                  '(error is %r).' % (path, e)
 
     def makedirs(self, path):
         """Custom makedirs that handle ro parent."""
         parent = os.path.dirname(path)
-        if os.path.exists(parent):
-            os.chmod(parent, 0755)
-        os.makedirs(path)
+        if path_exists(parent):
+            set_dir_readwrite(parent)
+        make_dir(path, recursive=True)
 
+    @defer.inlineCallbacks
     def setUp(self):
-        TwistedTestCase.setUp(self)
+        yield super(BaseTwistedTestCase, self).setUp()
+        self.__root = None
         # use the config from the branch
-        self.old_get_config_files = config.get_config_files
-        config.get_config_files = lambda: [os.path.join(os.environ['ROOTDIR'],
+        new_get_config_files = lambda: [os.path.join(os.environ['ROOTDIR'],
                                                    'data', 'syncdaemon.conf')]
+        self.patch(config, 'get_config_files', new_get_config_files)
+
         # fake a very basic config file with sane defaults for the tests
-        self.config_file = os.path.join(self.mktemp('config'), 'syncdaemon.conf')
+        config_dir = self.mktemp('config')
+        self.config_file = os.path.join(config_dir, 'syncdaemon.conf')
         with open(self.config_file, 'w') as fp:
             fp.write('[bandwidth_throttling]\n')
             fp.write('on = False\n')
@@ -321,28 +367,18 @@ class BaseTwistedTestCase(TwistedTestCase):
         config._user_config = None
         config.get_user_config(config_file=self.config_file)
 
-    def tearDown(self):
-        """ cleanup the temp dir. """
-        # invalidate the current config
-        config._user_config = None
-        # restore the old get_config_files
-        config.get_config_files = self.old_get_config_files
-        self.rmtree(os.path.dirname(self.config_file))
-        root_dir = getattr(self, '__root', None)
-        if root_dir:
-            self.rmtree(self.__root)
-        return TwistedTestCase.tearDown(self)
+        self.log = logging.getLogger("ubuntuone.SyncDaemon.TEST")
+        self.log.info("starting test %s.%s", self.__class__.__name__,
+                      self._testMethodName)
 
 
 class FakeMainTestCase(BaseTwistedTestCase):
     """A testcase that starts up a Main instance."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Setup the infrastructure for the test."""
-        BaseTwistedTestCase.setUp(self)
-        self.log = logging.getLogger("ubuntuone.SyncDaemon.TEST")
-        self.log.info("starting test %s.%s", self.__class__.__name__,
-                      self._testMethodName)
+        yield super(FakeMainTestCase, self).setUp()
         self.timeout = 2
         self.data_dir = self.mktemp('data_dir')
         self.partials_dir = self.mktemp('partials')
@@ -350,23 +386,13 @@ class FakeMainTestCase(BaseTwistedTestCase):
         self.shares_dir = self.mktemp('shares_dir')
         self.main = FakeMain(self.root_dir, self.shares_dir,
                              self.data_dir, self.partials_dir)
+        self.addCleanup(self.main.shutdown)
         self.vm = self.main.vm
         self.fs = self.main.fs
         self.event_q = self.main.event_q
         self.action_q = self.main.action_q
         self.event_q.push('SYS_INIT_DONE')
 
-    def tearDown(self):
-        """Shutdown this testcase."""
-        BaseTwistedTestCase.tearDown(self)
-        self.main.shutdown()
-        self.rmtree(self.shares_dir)
-        self.rmtree(self.root_dir)
-        self.rmtree(self.data_dir)
-        self.rmtree(self.partials_dir)
-        self.log.info("finished test %s.%s", self.__class__.__name__,
-                      self._testMethodName)
-
 
 class FakeVolumeManager(object):
     """ A volume manager that only knows one share, the root"""
@@ -382,11 +408,11 @@ class FakeVolumeManager(object):
         """Add share to the shares dict."""
         self.shares[share.id] = share
         # if the share don't exists, create it
-        if not os.path.exists(share.path):
-            os.mkdir(share.path)
+        if not path_exists(share.path):
+            make_dir(share.path)
         # if it's a ro share, change the perms
         if not share.can_write():
-            os.chmod(share.path, 0555)
+            set_dir_readonly(share.path)
 
     def add_udf(self, udf):
         """Add udf to the udfs dict."""
@@ -463,3 +489,17 @@ class DummyClass(object):
     def __getattr__(self, name):
         """Any attribute is a no-op."""
         return lambda *args, **kwargs: None
+
+
+skip_if_win32_and_uses_metadata_older_than_5 = \
+    skipIfOS('win32',
+             'In windows there is no need to migrate metadata older than v5.')
+
+
+skip_if_win32_and_uses_readonly = \
+    skipIfOS('win32', 'Can not test RO shares until bug #820350 is resolved.')
+
+
+skip_if_win32_missing_fs_event = \
+    skipIfOS('win32', 'Fails due to missing/out of order FS events, '
+                      'see bug #820598.')
