@@ -99,15 +99,14 @@ enable_setting_method (SyncdaemonConfigInterface *interface,
 		       const gchar *disable_method,
 		       gboolean enabled)
 {
-	gboolean result;
 	GError *error = NULL;
 
 	if (enabled) {
-		result = dbus_g_proxy_call (DBUS_G_PROXY (interface->priv->proxy), enable_method, &error,
+		dbus_g_proxy_call (DBUS_G_PROXY (interface->priv->proxy), enable_method, &error,
 					    G_TYPE_INVALID,
 					    G_TYPE_INVALID);
 	} else {
-		result = dbus_g_proxy_call (DBUS_G_PROXY (interface->priv->proxy), disable_method, &error,
+		dbus_g_proxy_call (DBUS_G_PROXY (interface->priv->proxy), disable_method, &error,
 					    G_TYPE_INVALID,
 					    G_TYPE_INVALID);
 	}
