@@ -21,9 +21,7 @@
 
 import logging
 import os
-import shutil
 import sys
-import unittest
 import uuid
 
 from os.path import basename
@@ -49,7 +47,6 @@ except ImportError:
 from contrib.testing.testcase import (
         FakeMain, FakeMainTestCase, BaseTwistedTestCase)
 from ubuntuone.devtools.handlers import MementoHandler
-from ubuntuone.platform.linux import get_udf_path
 from ubuntuone.storageprotocol import client, delta
 from ubuntuone.storageprotocol.request import ROOT
 from ubuntuone.storageprotocol.sharersp import NotifyShareHolder
@@ -70,10 +67,13 @@ from ubuntuone.eventlog.zg_listener import (
         INTERPRETATION_U1_CONTACT, URI_PROTOCOL_U1,
         STORAGE_DELETED, STORAGE_NETWORK, STORAGE_LOCAL)
 from ubuntuone.syncdaemon.sync import Sync
-from ubuntuone.syncdaemon.volume_manager import Share, Shared, UDF
+from ubuntuone.syncdaemon.volume_manager import (
+    get_udf_path, Share, Shared, UDF,
+)
 from tests.syncdaemon.test_action_queue import (
     ConnectedBaseTestCase,
     FakeSemaphore,
+    FakeTempFile,
     TestingProtocol,
 )
 
@@ -131,9 +131,10 @@ def listen_for(event_q, event, callback, count=1, collect=False):
 class ZeitgeistListenerTestCase(FakeMainTestCase):
     """Tests for ZeitgeistListener."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Initialize this instance."""
-        super(ZeitgeistListenerTestCase, self).setUp()
+        yield super(ZeitgeistListenerTestCase, self).setUp()
         self.patch(zglog, "ZeitgeistLogger", MockLogger)
         self.listener = ZeitgeistListener(self.fs, self.vm)
         self.event_q.subscribe(self.listener)
@@ -364,18 +365,19 @@ class ZeitgeistSharesTestCase(ZeitgeistListenerTestCase):
 class ZeitgeistUDFsTestCase(ZeitgeistListenerTestCase):
     """Tests for all UDFs-related zeitgeist events."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Initialize this test instance."""
-        super(ZeitgeistUDFsTestCase, self).setUp()
+        yield super(ZeitgeistUDFsTestCase, self).setUp()
         self.home_dir = self.mktemp('ubuntuonehacker')
         self._old_home = os.environ['HOME']
         os.environ['HOME'] = self.home_dir
 
+    @defer.inlineCallbacks
     def tearDown(self):
         """Finalize this test instance."""
-        self.rmtree(self.home_dir)
         os.environ['HOME'] = self._old_home
-        super(ZeitgeistUDFsTestCase, self).tearDown()
+        yield super(ZeitgeistUDFsTestCase, self).tearDown()
 
     def _create_udf(self, id, node_id, suggested_path, subscribed=True):
         """Create an UDF and returns it and the volume."""
@@ -561,9 +563,10 @@ class ZeitgeistUDFsTestCase(ZeitgeistListenerTestCase):
 class ZeitgeistRemoteFileSyncTestCase(ConnectedBaseTestCase):
     """File sync events are logged into Zeitgeist."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Initialize this test instance."""
-        ConnectedBaseTestCase.setUp(self)
+        yield super(ZeitgeistRemoteFileSyncTestCase, self).setUp()
         self.rq = request_queue = RequestQueue(action_queue=self.action_queue)
         self.rq.transfers_semaphore = FakeSemaphore()
 
@@ -574,9 +577,9 @@ class ZeitgeistRemoteFileSyncTestCase(ConnectedBaseTestCase):
         self.command = MyUpload(request_queue, share_id=self.share_id,
                                 node_id='a_node_id', previous_hash='prev_hash',
                                 hash='yadda', crc32=0, size=0, path='path',
-                                fileobj_factory=lambda: None,
-                                tempfile_factory=lambda: None)
+                                fileobj_factory=lambda: None)
         self.command.make_logger()
+        self.command.tempfile = FakeTempFile(self.mktemp('tmpdir'))
         self.fsm = self.action_queue.main.fs
         self.vm = self.action_queue.main.vm
         self.patch(zglog, "ZeitgeistLogger", MockLogger)
@@ -585,10 +588,6 @@ class ZeitgeistRemoteFileSyncTestCase(ConnectedBaseTestCase):
         self.root_id = "roootid"
         self.sync = Sync(main=self.main)
 
-    def tearDown(self):
-        """Finalize this test instance."""
-        ConnectedBaseTestCase.tearDown(self)
-
     def test_syncdaemon_creates_file_on_server_is_logged(self):
         """Files created by SyncDaemon on the server are logged."""
         filename = "filename.mp3"
@@ -803,9 +802,10 @@ class ZeitgeistLocalFileSyncTestCase(BaseTwistedTestCase):
     """Zeitgeist events coming from the server."""
     timeout = 5
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Initialize this instance."""
-        BaseTwistedTestCase.setUp(self)
+        yield super(ZeitgeistLocalFileSyncTestCase, self).setUp()
         self.root = self.mktemp('root')
         self.shares = self.mktemp('shares')
         self.data = self.mktemp('data')
@@ -837,19 +837,17 @@ class ZeitgeistLocalFileSyncTestCase(BaseTwistedTestCase):
         self.listener = ZeitgeistListener(self.main.fs, self.main.vm)
         self.main.event_q.subscribe(self.listener)
 
+    @defer.inlineCallbacks
     def tearDown(self):
         """Clean up this instance."""
         self._logger.removeHandler(self.handler)
         self.main.shutdown()
         FakeMain._sync_class = None
-        shutil.rmtree(self.root)
-        shutil.rmtree(self.shares)
-        shutil.rmtree(self.data)
         for record in self.handler.records:
             exc_info = getattr(record, 'exc_info', None)
             if exc_info is not None:
                 raise exc_info[0], exc_info[1], exc_info[2]
-        BaseTwistedTestCase.tearDown(self)
+        yield super(ZeitgeistLocalFileSyncTestCase, self).tearDown()
 
     @defer.inlineCallbacks
     def test_syncdaemon_creates_file_locally_is_logged(self):
@@ -1133,6 +1131,7 @@ class ZeitgeistLocalFileSyncTestCase(BaseTwistedTestCase):
         self.assertEqual(local_folder.mimetype, DIRECTORY_MIMETYPE)
         self.assertEqual(local_folder.storage, STORAGE_LOCAL)
 
+
 class ZeitgeistPublicFilesTestCase(ZeitgeistListenerTestCase):
     """Public files events are logged into Zeitgeist."""
 
@@ -1217,13 +1216,3 @@ class ZeitgeistPublicFilesTestCase(ZeitgeistListenerTestCase):
         self.assertEqual(public_file.text, public_url)
         self.assertEqual(public_file.mimetype, "audio/mpeg")
         self.assertEqual(public_file.storage, STORAGE_DELETED)
-
-
-def test_suite():
-    """Collect these tests only on linux."""
-    import sys
-    if sys.platform == 'linux2':
-        tests = unittest.TestLoader().loadTestsFromName(__name__)
-    else:
-        tests = []
-    return tests
