@@ -18,18 +18,34 @@
 
 """Tests for the Ubuntu One credentials management dbus service."""
 
+import logging
+
 from functools import wraps
 
 from twisted.internet.defer import Deferred, inlineCallbacks
-from ubuntuone.devtools.testcase import DBusTestCase as TestCase
+from ubuntuone.devtools.testcase import DBusTestCase
 from ubuntuone.devtools.handlers import MementoHandler
 
-from ubuntuone.platform.linux.credentials import (dbus, logger, logging,
-    CredentialsManagement, CredentialsManagementTool, CredentialsError,
+from ubuntuone.platform.credentials import (
+    CredentialsError,
+    CredentialsManagementTool,
+    logger,
+)
+from ubuntuone.platform.credentials.linux import (
+    APP_NAME,
+    CredentialsManagement,
+    dbus,
+    DBUS_BUS_NAME,
+    DBUS_CREDENTIALS_IFACE,
+    DBUS_CREDENTIALS_PATH,
+    DESCRIPTION,
+    HELP_TEXT_KEY,
+    PING_URL,
+    PING_URL_KEY,
+    TC_URL_KEY,
+    TC_URL,
+    TIMEOUT_INTERVAL,
     ubuntu_sso,
-    DBUS_BUS_NAME, DBUS_CREDENTIALS_PATH, DBUS_CREDENTIALS_IFACE,
-    APP_NAME, HELP_TEXT_KEY, DESCRIPTION, TC_URL_KEY, TC_URL,
-    PING_URL_KEY, PING_URL, TIMEOUT_INTERVAL,
 )
 
 FAKED_CREDENTIALS = {
@@ -160,15 +176,16 @@ class FakedSSOService(dbus.service.Object):
         self.register(app_name, args)
 
 
-class BaseTestCase(TestCase):
+class BaseTestCase(DBusTestCase):
     """Base test case."""
 
     timeout = 8
     app_name = APP_NAME
     error_dict = None
 
+    @inlineCallbacks
     def setUp(self):
-        super(BaseTestCase, self).setUp()
+        yield super(BaseTestCase, self).setUp()
         FakedSSOService.app_name = self.app_name
         FakedSSOService.error_dict = self.error_dict
 
@@ -176,7 +193,6 @@ class BaseTestCase(TestCase):
         self.memento.setLevel(logging.DEBUG)
         logger.addHandler(self.memento)
 
-        self.bus = dbus.SessionBus()
         self.sso_server = self.register_server(ubuntu_sso.DBUS_BUS_NAME,
                                 ubuntu_sso.DBUS_CREDENTIALS_PATH,
                                 FakedSSOService)  # faked SSO server
@@ -189,6 +205,7 @@ class BaseTestCase(TestCase):
                             'Service %s should not be running.' % bus_name)
         mock = service_class(object_path=object_path, conn=self.bus)
         self.addCleanup(mock.remove_from_connection)
+        self.addCleanup(self.bus.release_name, bus_name)
 
         return mock
 
@@ -210,8 +227,9 @@ class CredentialsManagementTestCase(BaseTestCase):
     signals = ('CredentialsFound', 'CredentialsNotFound', 'CredentialsCleared',
                'CredentialsStored', 'CredentialsError', 'AuthorizationDenied')
 
+    @inlineCallbacks
     def setUp(self):
-        super(CredentialsManagementTestCase, self).setUp()
+        yield super(CredentialsManagementTestCase, self).setUp()
         self.creds_server = self.register_server(DBUS_BUS_NAME,
                                 DBUS_CREDENTIALS_PATH,
                                 CredentialsManagement)  # real service
@@ -584,8 +602,9 @@ class OtherAppWithErrorTestCase(OtherAppNoErrorTestCase):
 class RefCountingTestCase(BaseTestCase):
     """Tests for the CredentialsManagement ref counting."""
 
+    @inlineCallbacks
     def setUp(self):
-        super(RefCountingTestCase, self).setUp()
+        yield super(RefCountingTestCase, self).setUp()
         self._called = False
         self.client = CredentialsManagement()
 
@@ -836,13 +855,13 @@ class RefCountingTestCase(BaseTestCase):
 class CredentialsTestCase(BaseTestCase):
     """Test suite for the Credentials class."""
 
+    @inlineCallbacks
     def setUp(self):
-        super(CredentialsTestCase, self).setUp()
+        yield super(CredentialsTestCase, self).setUp()
         FakedSSOService.error_dict = None
         self.creds_server = self.register_server(DBUS_BUS_NAME,
                                 DBUS_CREDENTIALS_PATH,
                                 CredentialsManagement)  # real service
-
         self.client = CredentialsManagementTool()
 
     @inlineCallbacks
@@ -866,12 +885,9 @@ class CredentialsTestCase(BaseTestCase):
         """Find credentials and error."""
         FakedSSOService.error_dict = {'failure': 'really'}
 
-        try:
-            yield self.client.find_credentials()
-        except CredentialsError, e:
-            self.assertEqual(e[0], FakedSSOService.error_dict)
-        else:
-            self.fail('Must not succeed!')
+        e = yield self.assertFailure(self.client.find_credentials(),
+                                     CredentialsError)
+        self.assertEqual(e[0], FakedSSOService.error_dict)
 
     @inlineCallbacks
     def test_clear_credentials(self):
@@ -888,12 +904,9 @@ class CredentialsTestCase(BaseTestCase):
         """Clear credentials and error."""
         FakedSSOService.error_dict = {'failure': 'really'}
 
-        try:
-            yield self.client.clear_credentials()
-        except CredentialsError, e:
-            self.assertEqual(e[0], FakedSSOService.error_dict)
-        else:
-            self.fail('Must not succeed!')
+        e = yield self.assertFailure(self.client.clear_credentials(),
+                                     CredentialsError)
+        self.assertEqual(e[0], FakedSSOService.error_dict)
 
     @inlineCallbacks
     def test_store_credentials(self):
@@ -910,12 +923,9 @@ class CredentialsTestCase(BaseTestCase):
         """Store credentials and error."""
         FakedSSOService.error_dict = {'failure': 'really'}
 
-        try:
-            yield self.client.store_credentials({'test': 'me'})
-        except CredentialsError, e:
-            self.assertEqual(e[0], FakedSSOService.error_dict)
-        else:
-            self.fail('Must not succeed!')
+        e = yield self.assertFailure(self.client.store_credentials({'0': '1'}),
+                                     CredentialsError)
+        self.assertEqual(e[0], FakedSSOService.error_dict)
 
     @inlineCallbacks
     def test_register(self):
@@ -937,12 +947,9 @@ class CredentialsTestCase(BaseTestCase):
         """Register and error."""
         FakedSSOService.error_dict = {'failure': 'really'}
 
-        try:
-            yield self.client.register()
-        except CredentialsError, e:
-            self.assertEqual(e[0], FakedSSOService.error_dict)
-        else:
-            self.fail('Must not succeed!')
+        e = yield self.assertFailure(self.client.register(),
+                                     CredentialsError)
+        self.assertEqual(e[0], FakedSSOService.error_dict)
 
     @inlineCallbacks
     def test_login(self):
@@ -964,10 +971,6 @@ class CredentialsTestCase(BaseTestCase):
         """Login and error."""
         FakedSSOService.error_dict = {'failure': 'really'}
 
-        try:
-            yield self.client.login()
-        except CredentialsError, e:
-            self.assertEqual(e[0], FakedSSOService.error_dict)
-        else:
-            self.fail('Must not succeed!')
-
+        e = yield self.assertFailure(self.client.login(),
+                                     CredentialsError)
+        self.assertEqual(e[0], FakedSSOService.error_dict)
