@@ -29,7 +29,7 @@ from ubuntuone.syncdaemon import (
 
 from ubuntuone.devtools.handlers import MementoHandler
 
-from ubuntuone.platform.linux import dbus_interface, tools, get_udf_path
+from ubuntuone.platform.linux import dbus_interface, tools
 from tests.platform.linux.test_dbus import (
     FakeCommand, DBusTwistedTestCase, AllEventsSender,
 )
@@ -40,20 +40,17 @@ from twisted.internet import defer, reactor
 class TestToolsBase(DBusTwistedTestCase):
     """ Base test case for SyncDaemonTool tests"""
 
+    @defer.inlineCallbacks
     def setUp(self):
-        DBusTwistedTestCase.setUp(self)
+        yield super(TestToolsBase, self).setUp()
         self.tool = tools.SyncDaemonTool(self.bus)
         self.home_dir = self.mktemp('ubuntuonehacker')
-        self._old_home = os.environ['HOME']
+        old_home = os.environ['HOME']
         os.environ['HOME'] = self.home_dir
+        self.addCleanup(os.environ.__setitem__, 'HOME', old_home)
         self.handler = MementoHandler()
         self.tool.log.addHandler(self.handler)
-        return self.main.wait_for_nirvana()
-
-    def tearDown(self):
-        os.environ['HOME'] = self._old_home
-        self.rmtree(self.home_dir)
-        return DBusTwistedTestCase.tearDown(self)
+        yield self.main.wait_for_nirvana()
 
     @defer.inlineCallbacks
     def create_file(self, path):
@@ -67,7 +64,7 @@ class TestToolsBase(DBusTwistedTestCase):
 
 
 class TestToolsBasic(TestToolsBase):
-    """ Baic test of SyncDaemonTool """
+    """Basic test of SyncDaemonTool."""
 
     def _create_share(self, accepted=True, subscribed=True):
         """Return a newly created Share."""
@@ -77,7 +74,7 @@ class TestToolsBasic(TestToolsBase):
 
     def _create_udf(self, volume_id, node_id, suggested_path, subscribed=True):
         """Create an UDF and returns it and the volume"""
-        path = get_udf_path(suggested_path)
+        path = volume_manager.get_udf_path(suggested_path)
         udf = volume_manager.UDF(str(volume_id), str(node_id), suggested_path,
                                  path, subscribed)
         return udf
@@ -88,7 +85,7 @@ class TestToolsBasic(TestToolsBase):
         d = self.tool.wait_connected()
         # test callback, pylint: disable-msg=C0111
         def connected(result):
-            self.assertEquals(True, result)
+            self.assertEqual(True, result)
         d.addBoth(connected)
         return d
 
@@ -116,7 +113,7 @@ class TestToolsBasic(TestToolsBase):
 
         # test callback, pylint: disable-msg=C0111
         def downloads(result):
-            self.assertEquals(True, result)
+            self.assertEqual(True, result)
         d.addBoth(downloads)
         return d
 
@@ -126,7 +123,7 @@ class TestToolsBasic(TestToolsBase):
 
         # test callback, pylint: disable-msg=C0111
         def uploads(result):
-            self.assertEquals(True, result)
+            self.assertEqual(True, result)
         d.addBoth(uploads)
         return d
 
@@ -143,7 +140,7 @@ class TestToolsBasic(TestToolsBase):
 
         # test callback, pylint: disable-msg=C0111
         def callback(result):
-            self.assertEquals(True, result)
+            self.assertEqual(True, result)
         d.addBoth(callback)
 
         # clear downloading
@@ -168,9 +165,9 @@ class TestToolsBasic(TestToolsBase):
         d = self.tool.get_metadata(path)
         # the callback, pylint: disable-msg=C0111
         def callback(result):
-            self.assertEquals(path, str(result['path']))
-            self.assertEquals('', str(result['share_id']))
-            self.assertEquals('node_id', result['node_id'])
+            self.assertEqual(path, str(result['path']))
+            self.assertEqual('', str(result['share_id']))
+            self.assertEqual('node_id', result['node_id'])
         d.addCallback(callback)
         return d
 
@@ -198,12 +195,12 @@ class TestToolsBasic(TestToolsBase):
         yield self.main.vm.add_share(volume_manager.Share(path=share_path,
                                      volume_id='share_id', access_level='Read',
                                      accepted=False, node_id="node_id"))
-        self.assertEquals(False, self.main.vm.shares['share_id'].accepted)
+        self.assertEqual(False, self.main.vm.shares['share_id'].accepted)
         result = yield self.tool.accept_share('share_id')
 
-        self.assertEquals('Yes', result['answer'])
-        self.assertEquals('share_id', result['volume_id'])
-        self.assertEquals(True, self.main.vm.shares['share_id'].accepted)
+        self.assertEqual('Yes', result['answer'])
+        self.assertEqual('share_id', result['volume_id'])
+        self.assertEqual(True, self.main.vm.shares['share_id'].accepted)
 
     @defer.inlineCallbacks
     def test_reject_share(self):
@@ -212,19 +209,19 @@ class TestToolsBasic(TestToolsBase):
         yield self.main.vm.add_share(volume_manager.Share(path=share_path,
                                      volume_id='share_id', access_level='Read',
                                      accepted=False))
-        self.assertEquals(False, self.main.vm.shares['share_id'].accepted)
+        self.assertEqual(False, self.main.vm.shares['share_id'].accepted)
         result = yield self.tool.reject_share('share_id')
 
-        self.assertEquals('No', result['answer'])
-        self.assertEquals('share_id', result['volume_id'])
-        self.assertEquals(False, self.main.vm.shares['share_id'].accepted)
+        self.assertEqual('No', result['answer'])
+        self.assertEqual('share_id', result['volume_id'])
+        self.assertEqual(False, self.main.vm.shares['share_id'].accepted)
 
     def test_wait_for_signal(self):
         """Test wait_for_signal method"""
         d = self.tool.wait_for_signal('Foo', lambda _: True)
         def check(result):
             """do the assert"""
-            self.assertEquals(True, result)
+            self.assertEqual(True, result)
         d.addCallback(check)
         client = tools.DBusClient(self.bus, '/events',
                                   dbus_interface.DBUS_IFACE_EVENTS_NAME)
@@ -239,8 +236,8 @@ class TestToolsBasic(TestToolsBase):
         d = self.tool.wait_for_signal('Foo', filter)
         def check(result):
             """do the assert"""
-            self.assertEquals('DIE!!!!', result.getErrorMessage())
-            self.assertEquals(ValueError, result.type)
+            self.assertEqual('DIE!!!!', result.getErrorMessage())
+            self.assertEqual(ValueError, result.type)
         d.addErrback(check)
         client = tools.DBusClient(self.bus, '/events',
                                   dbus_interface.DBUS_IFACE_EVENTS_NAME)
@@ -252,7 +249,7 @@ class TestToolsBasic(TestToolsBase):
         d = self.tool.wait_for_signals('FooOk','BarError')
         def check(result):
             """do the assert"""
-            self.assertEquals(True, result[0])
+            self.assertEqual(True, result[0])
         d.addCallback(check)
         client = tools.DBusClient(self.bus, '/folders',
                                   dbus_interface.DBUS_IFACE_FOLDERS_NAME)
@@ -265,7 +262,7 @@ class TestToolsBasic(TestToolsBase):
         def handle_error(error):
             """do the assert"""
             errorname, errorargs = error.value.args
-            self.assertEquals('BarError', errorname)
+            self.assertEqual('BarError', errorname)
         d.addCallbacks(self.fail, handle_error)
         client = tools.DBusClient(self.bus, '/folders',
                                   dbus_interface.DBUS_IFACE_FOLDERS_NAME)
@@ -274,7 +271,7 @@ class TestToolsBasic(TestToolsBase):
 
     def test_connect(self):
         """Test the connect method"""
-        self.assertEquals(self.main.state_manager.state,
+        self.assertEqual(self.main.state_manager.state,
                           states.StateManager.QUEUE_MANAGER)
         d = self.main.wait_for('SYS_USER_DISCONNECT')
         self.main.external.dbus_iface.disconnect()
@@ -288,7 +285,7 @@ class TestToolsBasic(TestToolsBase):
 
     def test_disconnect(self):
         """Test the disconnect method"""
-        self.assertEquals(self.main.state_manager.state,
+        self.assertEqual(self.main.state_manager.state,
                           states.StateManager.QUEUE_MANAGER)
         d = self.main.wait_for('SYS_USER_DISCONNECT')
         self.tool.disconnect()
@@ -300,14 +297,25 @@ class TestToolsBasic(TestToolsBase):
         d = self.tool.get_status()
         def handler(result):
             state = states.StateManager.QUEUE_MANAGER
-            self.assertEquals(state.name, result['name'])
-            self.assertEquals(state.description, result['description'])
-            self.assertEquals(state.is_error, bool(result['is_error']))
-            self.assertEquals(state.is_connected, bool(result['is_connected']))
-            self.assertEquals(state.is_online, bool(result['is_online']))
+            self.assertEqual(state.name, result['name'])
+            self.assertEqual(state.description, result['description'])
+            self.assertEqual(state.is_error, bool(result['is_error']))
+            self.assertEqual(state.is_connected, bool(result['is_connected']))
+            self.assertEqual(state.is_online, bool(result['is_online']))
         d.addCallbacks(handler, self.fail)
         return d
 
+    @defer.inlineCallbacks
+    def test_free_space(self):
+        """Test SyncDaemonTool.waiting."""
+        share_path = os.path.join(self.main.shares_dir, 'share')
+        share = volume_manager.Share(path=share_path, volume_id='vol_id')
+        yield self.main.vm.add_share(share)
+
+        self.main.vm.update_free_space('vol_id', 12345)
+        result = yield self.tool.free_space('vol_id')
+        self.assertEqual(result, 12345)
+
     @defer.inlineCallbacks
     def test_waiting_simple(self):
         """Test SyncDaemonTool.waiting."""
@@ -318,15 +326,15 @@ class TestToolsBasic(TestToolsBase):
         self.action_q.queue.waiting.extend([c1, c2])
         result = yield self.tool.waiting()
 
-        self.assertEquals(2, len(result))
+        self.assertEqual(2, len(result))
 
         pl = dict(share_id='node_a_foo', node_id='node_a_bar',
                   other='', path='path', running='True')
-        self.assertEqual(result[0], ('FakeCommand', pl))
+        self.assertEqual(result[0], ('FakeCommand', str(id(c1)), pl))
 
         pl = dict(share_id='node_b_foo', node_id='node_b_bar',
                   other='', running='')
-        self.assertEqual(result[1], ('FakeCommand', pl))
+        self.assertEqual(result[1], ('FakeCommand', str(id(c2)), pl))
 
     @defer.inlineCallbacks
     def test_waiting_metadata(self):
@@ -337,7 +345,7 @@ class TestToolsBasic(TestToolsBase):
                 FakeCommand("node_b_foo", "node_b_bar")])
         result = yield self.tool.waiting_metadata()
 
-        self.assertEquals(2, len(result))
+        self.assertEqual(2, len(result))
 
         pl = dict(share_id='node_a_foo', node_id='node_a_bar',
                   other='', path='path', running='True')
@@ -377,7 +385,7 @@ class TestToolsBasic(TestToolsBase):
         """Test start method"""
         # first quit the currently running client
         d = self.tool.start()
-        d.addCallbacks(lambda r: self.assertEquals(r, None), self.fail)
+        d.addCallbacks(lambda r: self.assertEqual(r, None), self.fail)
         return d
 
     def test_create_folder(self):
@@ -489,16 +497,16 @@ class TestToolsBasic(TestToolsBase):
     def test_get_throttling_limits(self):
         """Test for get_throttling_limits."""
         limits = yield self.tool.get_throttling_limits()
-        self.assertEquals(-1, limits[u'download'])
-        self.assertEquals(-1, limits[u'upload'])
+        self.assertEqual(-1, limits[u'download'])
+        self.assertEqual(-1, limits[u'upload'])
 
     @defer.inlineCallbacks
     def test_set_throttling_limits(self):
         """Test for set_throttling_limits."""
         yield self.tool.set_throttling_limits(10, 20)
         limits = yield self.tool.get_throttling_limits()
-        self.assertEquals(10, limits[u'download'])
-        self.assertEquals(20, limits[u'upload'])
+        self.assertEqual(10, limits[u'download'])
+        self.assertEqual(20, limits[u'upload'])
 
     @defer.inlineCallbacks
     def test_is_throttling_enabled(self):
@@ -583,6 +591,106 @@ class TestToolsBasic(TestToolsBase):
         config = tools.get_user_config()
         self.assertEqual(False, config.get_files_sync_enabled())
 
+    @defer.inlineCallbacks
+    def test_is_autoconnect_enabled(self):
+        """Test for is_autoconnect_enabled."""
+        yield self.tool.enable_autoconnect(False)
+        enabled = yield self.tool.is_autoconnect_enabled()
+        self.assertFalse(enabled)
+
+        yield self.tool.enable_autoconnect(True)
+        enabled = yield self.tool.is_autoconnect_enabled()
+        self.assertTrue(enabled)
+
+    @defer.inlineCallbacks
+    def test_enable_autoconnect(self):
+        """Test for enable_autoconnect."""
+        yield self.tool.enable_autoconnect(True)
+        enabled = yield self.tool.is_autoconnect_enabled()
+        self.assertTrue(enabled)
+
+    @defer.inlineCallbacks
+    def test_disable_autoconnect(self):
+        """Test for disable_autoconnect."""
+        yield self.tool.enable_autoconnect(False)
+        enabled = yield self.tool.is_autoconnect_enabled()
+        self.assertFalse(enabled)
+
+    @defer.inlineCallbacks
+    def test_is_show_all_notifications_enabled(self):
+        """Test for is_show_all_notifications_enabled."""
+        yield self.tool.enable_show_all_notifications(False)
+        enabled = yield self.tool.is_show_all_notifications_enabled()
+        self.assertFalse(enabled)
+
+        yield self.tool.enable_show_all_notifications(True)
+        enabled = yield self.tool.is_show_all_notifications_enabled()
+        self.assertTrue(enabled)
+
+    @defer.inlineCallbacks
+    def test_enable_show_all_notifications(self):
+        """Test for enable_show_all_notifications."""
+        yield self.tool.enable_show_all_notifications(True)
+        enabled = yield self.tool.is_show_all_notifications_enabled()
+        self.assertTrue(enabled)
+
+    @defer.inlineCallbacks
+    def test_disable_show_all_notifications(self):
+        """Test for disable_show_all_notifications."""
+        yield self.tool.enable_show_all_notifications(False)
+        enabled = yield self.tool.is_show_all_notifications_enabled()
+        self.assertFalse(enabled)
+
+    @defer.inlineCallbacks
+    def test_is_share_autosubscribe_enabled(self):
+        """Test for is_share_autosubscribe_enabled."""
+        yield self.tool.enable_share_autosubscribe(False)
+        enabled = yield self.tool.is_share_autosubscribe_enabled()
+        self.assertFalse(enabled)
+
+        yield self.tool.enable_share_autosubscribe(True)
+        enabled = yield self.tool.is_share_autosubscribe_enabled()
+        self.assertTrue(enabled)
+
+    @defer.inlineCallbacks
+    def test_enable_share_autosubscribe(self):
+        """Test for enable_share_autosubscribe."""
+        yield self.tool.enable_share_autosubscribe(True)
+        enabled = yield self.tool.is_share_autosubscribe_enabled()
+        self.assertTrue(enabled)
+
+    @defer.inlineCallbacks
+    def test_disable_share_autosubscribe(self):
+        """Test for disable_share_autosubscribe."""
+        yield self.tool.enable_share_autosubscribe(False)
+        enabled = yield self.tool.is_share_autosubscribe_enabled()
+        self.assertFalse(enabled)
+
+    @defer.inlineCallbacks
+    def test_is_udf_autosubscribe_enabled(self):
+        """Test for is_udf_autosubscribe_enabled."""
+        yield self.tool.enable_udf_autosubscribe(False)
+        enabled = yield self.tool.is_udf_autosubscribe_enabled()
+        self.assertFalse(enabled)
+
+        yield self.tool.enable_udf_autosubscribe(True)
+        enabled = yield self.tool.is_udf_autosubscribe_enabled()
+        self.assertTrue(enabled)
+
+    @defer.inlineCallbacks
+    def test_enable_udf_autosubscribe(self):
+        """Test for enable_udf_autosubscribe."""
+        yield self.tool.enable_udf_autosubscribe(True)
+        enabled = yield self.tool.is_udf_autosubscribe_enabled()
+        self.assertTrue(enabled)
+
+    @defer.inlineCallbacks
+    def test_disable_udf_autosubscribe(self):
+        """Test for disable_udf_autosubscribe."""
+        yield self.tool.enable_udf_autosubscribe(False)
+        enabled = yield self.tool.is_udf_autosubscribe_enabled()
+        self.assertFalse(enabled)
+
     @defer.inlineCallbacks
     def test_refresh_volumes(self):
         """Test for refresh_volumes method."""
@@ -608,7 +716,7 @@ class TestToolsBasic(TestToolsBase):
         self.main.action_q.rescan_from_scratch = rescan_from_scratch
         yield self.tool.rescan_from_scratch(udf.volume_id)
         vol_id = yield d
-        self.assertEquals(vol_id, udf.volume_id)
+        self.assertEqual(vol_id, udf.volume_id)
 
     @defer.inlineCallbacks
     def test_rescan_from_scratch_missing_volume(self):
@@ -648,5 +756,23 @@ class TestToolsBasic(TestToolsBase):
         self.assertNotIn(mdid1, dirty_mdids)
         self.assertNotIn(mdid3, dirty_mdids)
         # check that path de/encoding is done correctly
-        self.assertEquals(repr(self.main.fs.get_by_mdid(mdid2).path),
-                          repr(dirty_mdids[mdid2]['path'].encode('utf-8')))
+        self.assertEqual(repr(self.main.fs.get_by_mdid(mdid2).path),
+                         repr(dirty_mdids[mdid2]['path'].encode('utf-8')))
+
+    @defer.inlineCallbacks
+    def test_get_root_dir(self):
+        """Test the get_root_dir method"""
+        result = yield self.tool.get_root_dir()
+        self.assertEqual(self.main.root_dir, result)
+
+    @defer.inlineCallbacks
+    def test_get_shares_dir(self):
+        """Test the get_shares_dir method"""
+        result = yield self.tool.get_shares_dir()
+        self.assertEqual(self.main.shares_dir, result)
+
+    @defer.inlineCallbacks
+    def test_get_shares_dir_link(self):
+        """Test the get_shares_dir_link method"""
+        result = yield self.tool.get_shares_dir_link()
+        self.assertEqual(self.main.shares_dir_link, result)
