@@ -18,6 +18,7 @@
 
 """Tests for the liblauncher interface."""
 
+from twisted.internet import defer
 from twisted.trial.unittest import TestCase
 
 from ubuntuone.platform.linux import launcher
@@ -55,8 +56,10 @@ class LauncherTestCase(TestCase):
 
     skip = None if launcher.use_libunity else "libunity not installed."
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Initialize this test instance."""
+        yield super(LauncherTestCase, self).setUp()
         self.patch(launcher.Unity, "LauncherEntry", FakeLauncherEntry)
         self.launcher = launcher.UbuntuOneLauncher()
 
