@@ -19,6 +19,8 @@
 import os
 import uuid
 
+from twisted.internet import defer
+
 from contrib.testing.testcase import FakeMain, environ
 from tests.syncdaemon.test_vm import MetadataTestCase, BaseVolumeManagerTests
 from ubuntuone.storageprotocol import request
@@ -44,8 +46,9 @@ class VolumesTests(BaseVolumeManagerTests):
 class MetadataOldLayoutTests(MetadataTestCase):
     """Tests for 'old' layouts and metadata upgrade"""
 
+    @defer.inlineCallbacks
     def setUp(self):
-        MetadataTestCase.setUp(self)
+        yield super(MetadataOldLayoutTests, self).setUp()
         self.root_dir = os.path.join(self.u1_dir, 'My Files')
         self.shares_dir = os.path.join(self.u1_dir, 'Shared With Me')
         self.new_root_dir = self.u1_dir
@@ -451,8 +454,9 @@ class MetadataOldLayoutTests(MetadataTestCase):
 class MetadataNewLayoutTests(MetadataTestCase):
     """Test for 'new' layout and metadata upgrade."""
 
+    @defer.inlineCallbacks
     def setUp(self):
-        MetadataTestCase.setUp(self)
+        yield super(MetadataNewLayoutTests, self).setUp()
         self.share_md_dir = os.path.join(self.vm_data_dir, 'shares')
         self.shared_md_dir = os.path.join(self.vm_data_dir, 'shared')
         self.home_dir = os.path.join(self.tmpdir, 'home', 'ubuntuonehacker')
