@@ -18,15 +18,18 @@
 """Test that the interaction_interfaces are correctly called."""
 
 from mocker import MockerTestCase, MATCH
+from twisted.internet import defer
+
 from tests.platform import IPCTestCase
 
 
 class TestStatusIPC(MockerTestCase, IPCTestCase):
     """Ensure that calls are correctly fowarded."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Set up tests."""
-        super(TestStatusIPC, self).setUp()
+        yield super(TestStatusIPC, self).setUp()
         self.syncdaemon_status = self.mocker.mock()
         self.status.syncdaemon_status = self.syncdaemon_status
 
@@ -82,9 +85,10 @@ class TestStatusIPC(MockerTestCase, IPCTestCase):
 class TestEventsIPC(MockerTestCase, IPCTestCase):
     """Ensure that calls are correctly fowarded."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Set up tests."""
-        super(TestEventsIPC, self).setUp()
+        yield super(TestEventsIPC, self).setUp()
         self.events_mock = self.mocker.mock()
         self.events.events = self.events_mock
 
@@ -100,9 +104,10 @@ class TestEventsIPC(MockerTestCase, IPCTestCase):
 class TestSyncDaemonIPC(MockerTestCase, IPCTestCase):
     """Ensure that calls are correctly fowarded."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Set up tests."""
-        super(TestSyncDaemonIPC, self).setUp()
+        yield super(TestSyncDaemonIPC, self).setUp()
         self.service = self.mocker.mock()
         self.sync.service = self.service
 
@@ -174,9 +179,10 @@ class TestSyncDaemonIPC(MockerTestCase, IPCTestCase):
 class TestFileSystemIPC(MockerTestCase, IPCTestCase):
     """Ensure that calls are correctly fowarded."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Set up tests."""
-        super(TestFileSystemIPC, self).setUp()
+        yield super(TestFileSystemIPC, self).setUp()
         self.syncdaemon_filesystem = self.mocker.mock()
         self.fs.syncdaemon_filesystem = self.syncdaemon_filesystem
 
@@ -221,9 +227,10 @@ class TestFileSystemIPC(MockerTestCase, IPCTestCase):
 class TestSharesIPC(MockerTestCase, IPCTestCase):
     """Ensure that calls are correctly fowarded."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Set up tests."""
-        super(TestSharesIPC, self).setUp()
+        yield super(TestSharesIPC, self).setUp()
         self.syncdaemon_shares = self.mocker.mock()
         self.shares.syncdaemon_shares = self.syncdaemon_shares
 
@@ -316,9 +323,10 @@ class TestSharesIPC(MockerTestCase, IPCTestCase):
 class TestConfigIPC(MockerTestCase, IPCTestCase):
     """Ensure that calls are correctly fowarded."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Set up tests."""
-        super(TestConfigIPC, self).setUp()
+        yield super(TestConfigIPC, self).setUp()
         self.syncdaemon_config = self.mocker.mock()
         self.config.syncdaemon_config = self.syncdaemon_config
 
@@ -467,9 +475,10 @@ class TestConfigIPC(MockerTestCase, IPCTestCase):
 class TestFoldersIPC(MockerTestCase, IPCTestCase):
     """Ensure that calls are correctly fowarded."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Set up tests."""
-        super(TestFoldersIPC, self).setUp()
+        yield super(TestFoldersIPC, self).setUp()
         self.syncdaemon_folders = self.mocker.mock()
         self.folders.syncdaemon_folders = self.syncdaemon_folders
 
@@ -528,9 +537,10 @@ class TestFoldersIPC(MockerTestCase, IPCTestCase):
 class TestPublicFilesIPC(MockerTestCase, IPCTestCase):
     """Ensure that calls are correctly fowarded."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Set up tests."""
-        super(TestPublicFilesIPC, self).setUp()
+        yield super(TestPublicFilesIPC, self).setUp()
         self.syncdaemon_public_files = self.mocker.mock()
         self.public_files.syncdaemon_public_files =\
             self.syncdaemon_public_files
