@@ -7,6 +7,6 @@ class TestPlatform(unittest.TestCase):
 
     def test_import(self):
         """Make sure we can import the platform module."""
-        
+
         import ubuntuone.platform as uplt
-        self.assertEqual(uplt.platform, sys.platform)
+        self.assertTrue(sys.platform.startswith(uplt.platform))
