@@ -1,8 +1,5 @@
 """Windows related test code."""
 
-from ubuntuone.platform.windows.os_helper import LONG_PATH_PREFIX
-
-  
 def setup_action_queue_test(testcase):
     """Extra setup to be performed for the action queue tests."""
     # nothing to be done
@@ -19,8 +16,3 @@ def get_main_params(testcase, params):
     """Return the parameters used to create the main object for tests."""
     params['glib_loop'] = False
     return params
-
-def platform_mangled_path(path):
-    if not path.startswith(LONG_PATH_PREFIX):
-        return  LONG_PATH_PREFIX + path
-    return path
