@@ -50,8 +50,10 @@ class TimerTestCase(TestCase):
 
     TIMEOUT = 3.0
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Initialize this test instance."""
+        yield super(TimerTestCase, self).setUp()
         self.clock = PatchedClock()
         self.timer = aggregator.Timer(delay=3.0, clock=self.clock)
 
@@ -101,8 +103,10 @@ class DeadlineTimerTestCase(TimerTestCase):
 
     DELAY = 0.5
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Initialize this test instance."""
+        yield super(DeadlineTimerTestCase, self).setUp()
         self.clock = PatchedClock()
         self.timer = aggregator.DeadlineTimer(delay=0.5, timeout=3.0,
                                             clock=self.clock)
@@ -242,8 +246,10 @@ class FakeStatusAggregator(object):
 class ToggleableNotificationTestCase(TestCase):
     """Test the ToggleableNotification class."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Initialize this test instance."""
+        yield super(ToggleableNotificationTestCase, self).setUp()
         self.patch(aggregator, "Notification", FakeNotification)
         self.notification_switch = aggregator.NotificationSwitch()
         self.toggleable = self.notification_switch.get_notification()
@@ -282,8 +288,10 @@ class ToggleableNotificationTestCase(TestCase):
 class NotificationSwitchTestCase(TestCase):
     """Test the NotificationSwitch class."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Initialize this test instance."""
+        yield super(NotificationSwitchTestCase, self).setUp()
         self.notification_switch = aggregator.NotificationSwitch()
 
     def test_get_notification(self):
@@ -306,8 +314,10 @@ class NotificationSwitchTestCase(TestCase):
 class FileDiscoveryBubbleTestCase(TestCase):
     """Test the FileDiscoveryBubble class."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Initialize this test instance."""
+        yield super(FileDiscoveryBubbleTestCase, self).setUp()
         self.patch(aggregator, "ToggleableNotification",
                    FakeNotificationSingleton())
         self.clock = PatchedClock()
@@ -487,8 +497,10 @@ class FileDiscoveryBubbleTestCase(TestCase):
 class FinalBubbleTestCase(TestCase):
     """Test for the final status notification bubble."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Initialize this test instance."""
+        yield super(FinalBubbleTestCase, self).setUp()
         self.patch(aggregator, "ToggleableNotification",
                    FakeNotificationSingleton())
         self.clock = PatchedClock()
@@ -542,8 +554,10 @@ class FakeInhibitor(object):
 class ProgressBarTestCase(TestCase):
     """Tests for the progress bar."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Initialize this test instance."""
+        yield super(ProgressBarTestCase, self).setUp()
         self.patch(aggregator, "UbuntuOneLauncher", FakeLauncher)
         self.patch(aggregator.session, "Inhibitor", FakeInhibitor)
         self.clock = PatchedClock()
@@ -737,10 +751,10 @@ class FakeAggregator(object):
             self.files_uploading.remove(command)
         self.queued_commands.discard(command)
 
-    def progress_made(self, share_id, node_id, n_bytes_written, deflated_size):
+    def progress_made(self, share_id, node_id, n_bytes, deflated_size):
         """Progress made on up- or download."""
         self.progress_events.append(
-            (share_id, node_id, n_bytes_written, deflated_size))
+            (share_id, node_id, n_bytes, deflated_size))
 
     def connection_made(self):
         """The client made the connection to the server."""
@@ -1033,8 +1047,10 @@ class StatusEventTestCase(TestCase):
     CLASS_KWARGS = {}
     status = None
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Initialize this test instance."""
+        yield super(StatusEventTestCase, self).setUp()
         if type(self) == StatusEventTestCase:
             self.assertRaises(AssertionError, self.CLASS, **self.CLASS_KWARGS)
         else:
@@ -1177,8 +1193,10 @@ class FakeFinalBubble(object):
 class StatusAggregatorTestCase(TestCase):
     """Test the backend of the status aggregator."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Initialize this test instance."""
+        yield super(StatusAggregatorTestCase, self).setUp()
         self.patch(aggregator, "FileDiscoveryBubble",
                    FakeFileDiscoveryBubble)
         self.patch(aggregator, "FinalStatusBubble",
@@ -1307,6 +1325,26 @@ class StatusAggregatorTestCase(TestCase):
         result = self.aggregator.get_discovery_message()
         self.assertEqual(expected, result)
 
+    def test_get_discovery_message_clears_filenames(self):
+        """Test the message that's shown on the discovery bubble."""
+        uploading = 10
+        downloading = 8
+        filename = 'upfile0.ext'
+        filename2 = 'downfile0.ext'
+        self.aggregator.files_uploading.extend([
+            FakeCommand(path='upfile%d.ext' % n) for n in range(uploading)])
+        self.aggregator.uploading_filename = filename
+        self.aggregator.files_downloading.extend([
+            FakeCommand(path='downfile%d.ext' % n) for n in
+            range(downloading)])
+        self.aggregator.downloading_filename = 'STALE FILENAME'
+        self.aggregator.uploading_filename = 'STALE FILENAME'
+        expected = (
+            aggregator.files_being_uploaded(filename, uploading) + "\n" +
+            aggregator.files_being_downloaded(filename2, downloading))
+        result = self.aggregator.get_discovery_message()
+        self.assertEqual(expected, result)
+
     def test_get_final_status_message(self):
         """The final status message."""
         done = (5, 10)
