@@ -19,12 +19,15 @@ from __future__ import with_statement
 
 import logging
 import os
+
 from ConfigParser import ConfigParser
-from xdg.BaseDirectory import (
+from twisted.internet import defer
+
+from ubuntuone.platform import open_file, path_exists
+from ubuntu_sso.xdg_base_directory import (
     xdg_data_home,
     xdg_cache_home,
 )
-
 from ubuntuone.syncdaemon import config
 from contrib.testing.testcase import (
     BaseTwistedTestCase,
@@ -36,8 +39,9 @@ from contrib.testing.testcase import (
 class TestConfigBasic(BaseTwistedTestCase):
     """Basic _Config object tests"""
 
+    @defer.inlineCallbacks
     def setUp(self):
-        BaseTwistedTestCase.setUp(self)
+        yield super(TestConfigBasic, self).setUp()
         self.test_root = self.mktemp()
 
     def assertThrottlingSection(self, expected, current, on, read, write):
@@ -69,7 +73,7 @@ class TestConfigBasic(BaseTwistedTestCase):
         """test loading the config file containing only the throttling values"""
         conf_file = os.path.join(self.test_root, 'test_load_config.conf')
         # write some throttling values to the config file
-        with open(conf_file, 'w') as fp:
+        with open_file(conf_file, 'w') as fp:
             fp.write('[bandwidth_throttling]\n')
             fp.write('on = True\n')
             fp.write('read_limit = 1000\n')
@@ -83,7 +87,7 @@ class TestConfigBasic(BaseTwistedTestCase):
         """test loading the a config file with other sections too"""
         conf_file = os.path.join(self.test_root, 'test_load_extra_config.conf')
         # write some throttling values to the config file
-        with open(conf_file, 'w') as fp:
+        with open_file(conf_file, 'w') as fp:
             fp.write('[__main__]\n')
             fp.write('log_level = INFO\n')
             fp.write('disable_ssl_verify = True\n')
@@ -100,7 +104,7 @@ class TestConfigBasic(BaseTwistedTestCase):
     def test_write_new(self):
         """test writing the throttling section to a new config file"""
         conf_file = os.path.join(self.test_root, 'test_write_new_config.conf')
-        self.assertFalse(os.path.exists(conf_file))
+        self.assertFalse(path_exists(conf_file))
         conf = config._Config(conf_file)
         conf.set_throttling(True)
         conf.set_throttling_read_limit(1000)
@@ -116,12 +120,12 @@ class TestConfigBasic(BaseTwistedTestCase):
         conf_file = os.path.join(self.test_root,
                                  'test_write_existing_config.conf')
         # write some throttling values to the config file
-        with open(conf_file, 'w') as fp:
+        with open_file(conf_file, 'w') as fp:
             fp.write('[bandwidth_throttling]\n')
             fp.write('on = False\n')
             fp.write('read_limit = 1000\n')
             fp.write('write_limit = 100\n')
-        self.assertTrue(os.path.exists(conf_file))
+        self.assertTrue(path_exists(conf_file))
         conf = config._Config(conf_file)
         conf.set_throttling(True)
         conf.set_throttling_read_limit(2000)
@@ -138,7 +142,7 @@ class TestConfigBasic(BaseTwistedTestCase):
         """
         conf_file = os.path.join(self.test_root, 'test_write_extra_config.conf')
         # write some throttling values to the config file
-        with open(conf_file, 'w') as fp:
+        with open_file(conf_file, 'w') as fp:
             fp.write('[__main__]\n')
             fp.write('log_level = INFO\n')
             fp.write('disable_ssl_verify = True\n')
@@ -147,7 +151,7 @@ class TestConfigBasic(BaseTwistedTestCase):
             fp.write('on = False\n')
             fp.write('read_limit = 2000\n')
             fp.write('write_limit = 200\n')
-        self.assertTrue(os.path.exists(conf_file))
+        self.assertTrue(path_exists(conf_file))
         conf = config._Config(conf_file)
         conf.set_throttling(True)
         conf.set_throttling_read_limit(3000)
@@ -169,12 +173,12 @@ class TestConfigBasic(BaseTwistedTestCase):
         conf_file = os.path.join(self.test_root,
                                  'test_write_existing_config.conf')
         # write some throttling values to the config file
-        with open(conf_file, 'w') as fp:
+        with open_file(conf_file, 'w') as fp:
             fp.write('[bandwidth_throttling]\n')
             fp.write('on = True\n')
             fp.write('read_limit = 1000\n')
             fp.write('write_limit = 100\n')
-        self.assertTrue(os.path.exists(conf_file))
+        self.assertTrue(path_exists(conf_file))
         conf = config._Config(conf_file)
         conf.set_throttling(False)
         conf.save()
@@ -187,7 +191,7 @@ class TestConfigBasic(BaseTwistedTestCase):
         """test loading the config file with negative read/write limits"""
         conf_file = os.path.join(self.test_root, 'test_load_config.conf')
         # write some throttling values to the config file
-        with open(conf_file, 'w') as fp:
+        with open_file(conf_file, 'w') as fp:
             fp.write('[bandwidth_throttling]\n')
             fp.write('on = True\n')
             fp.write('read_limit = -1\n')
@@ -202,7 +206,7 @@ class TestConfigBasic(BaseTwistedTestCase):
         """test loading a partial config file and fallback to defaults"""
         conf_file = os.path.join(self.test_root, 'test_load_config.conf')
         # write some throttling values to the config file
-        with open(conf_file, 'w') as fp:
+        with open_file(conf_file, 'w') as fp:
             fp.write('[bandwidth_throttling]\n')
             fp.write('on = True\n')
             fp.write('read_limit = 1\n')
@@ -215,7 +219,7 @@ class TestConfigBasic(BaseTwistedTestCase):
         """test loading the config file containing only the throttling values"""
         conf_file = os.path.join(self.test_root, 'test_load_config.conf')
         # write some throttling values to the config file
-        with open(conf_file, 'w') as fp:
+        with open_file(conf_file, 'w') as fp:
             fp.write('[bandwidth_throttling]\n')
             fp.write('on = True\n')
             fp.write('read_limit = 1000\n')
@@ -239,7 +243,7 @@ class TestConfigBasic(BaseTwistedTestCase):
         conf_file = os.path.join(self.test_root,
                                  'test_udf_autosubscribe_config.conf')
         # write some throttling values to the config file
-        with open(conf_file, 'w') as fp:
+        with open_file(conf_file, 'w') as fp:
             fp.write('[__main__]\n')
             fp.write('log_level = INFO\n')
             fp.write('disable_ssl_verify = True\n')
@@ -288,7 +292,7 @@ class TestConfigBasic(BaseTwistedTestCase):
         conf_file = os.path.join(self.test_root,
                                  'test_share_autosubscribe_config.conf')
         # write some throttling values to the config file
-        with open(conf_file, 'w') as fp:
+        with open_file(conf_file, 'w') as fp:
             fp.write('[__main__]\n')
             fp.write('share_autosubscribe = True\n')
 
@@ -330,7 +334,7 @@ class TestConfigBasic(BaseTwistedTestCase):
         conf_file = os.path.join(self.test_root,
                                  'test_autoconnect_config.conf')
         # ensure that autoconnect is True
-        with open(conf_file, 'w') as fp:
+        with open_file(conf_file, 'w') as fp:
             fp.write('[__main__]\n')
             fp.write('autoconnect = True\n')
 
@@ -378,7 +382,7 @@ class TestConfigBasic(BaseTwistedTestCase):
         conf_file = os.path.join(self.test_root,
                                  'test_show_all_notifications_config.conf')
         # ensure that show_all_notifications is True
-        with open(conf_file, 'w') as fp:
+        with open_file(conf_file, 'w') as fp:
             fp.write('[notifications]\n')
             fp.write('show_all_notifications = True\n')
 
@@ -431,7 +435,7 @@ class TestConfigBasic(BaseTwistedTestCase):
     def test_get_simult_transfers(self):
         """Get simult transfers."""
         conf_file = os.path.join(self.test_root, 'test_load_config.conf')
-        with open(conf_file, 'w') as fh:
+        with open_file(conf_file, 'w') as fh:
             fh.write('[__main__]\n')
             fh.write('simult_transfers = 12345\n')
         conf = config._Config(conf_file)
@@ -440,7 +444,7 @@ class TestConfigBasic(BaseTwistedTestCase):
     def test_set_simult_transfers(self):
         """Set simult transfers."""
         conf_file = os.path.join(self.test_root, 'test_load_config.conf')
-        with open(conf_file, 'w') as fh:
+        with open_file(conf_file, 'w') as fh:
             fh.write('[__main__]\n')
             fh.write('simult_transfers = 12345\n')
         conf = config._Config(conf_file)
@@ -450,7 +454,7 @@ class TestConfigBasic(BaseTwistedTestCase):
     def test_get_max_payload_size(self):
         """Get the maximum payload size."""
         conf_file = os.path.join(self.test_root, 'test_load_config.conf')
-        with open(conf_file, 'w') as fh:
+        with open_file(conf_file, 'w') as fh:
             fh.write('[__main__]\n')
             fh.write('max_payload_size = 12345\n')
         conf = config._Config(conf_file)
@@ -459,7 +463,7 @@ class TestConfigBasic(BaseTwistedTestCase):
     def test_set_max_payload_size(self):
         """Set the maximum payload size."""
         conf_file = os.path.join(self.test_root, 'test_load_config.conf')
-        with open(conf_file, 'w') as fh:
+        with open_file(conf_file, 'w') as fh:
             fh.write('[__main__]\n')
             fh.write('max_payload_size = 12345\n')
         conf = config._Config(conf_file)
@@ -468,7 +472,7 @@ class TestConfigBasic(BaseTwistedTestCase):
 
 
 class ConfigglueParsersTests(BaseTwistedTestCase):
-    """Tests for our custom configglue parsers"""
+    """Tests for our custom configglue parsers."""
 
     def test_throttling_limit_parser(self):
         """Test throttling_limit_parser"""
@@ -494,45 +498,63 @@ class ConfigglueParsersTests(BaseTwistedTestCase):
         self.assertEqual(logging.DEBUG, parser(bad_value))
         self.assertEqual(logging.DEBUG, parser(invalid_value))
 
-    def test_home_dir_parser(self):
-        """Test home_dir_parser"""
-        good_value = '~/hola'
-        bad_value = 'hola'
-        invalid_value = None
-        parser = config.home_dir_parser
-        with environ('HOME', '/home/fake'):
-            self.assertEquals('/home/fake/hola', parser(good_value))
-        self.assertEquals('hola', parser(bad_value))
-        self.assertRaises(AttributeError, parser, invalid_value)
 
-    def test_xdg_cache_dir_parser(self):
-        """Test xdg_cache_dir_parser"""
-        good_value = 'hola'
-        bad_value = '/hola'
-        invalid_value = None
-        parser = config.xdg_cache_dir_parser
-        self.assertEquals(os.path.join(xdg_cache_home, 'hola'),
-                          parser(good_value))
-        self.assertEquals('/hola', parser(bad_value))
-        self.assertRaises(AttributeError, parser, invalid_value)
+class XdgHomeParsersTests(BaseTwistedTestCase):
+    """Tests for our custom xdg parsers."""
 
-    def test_xdg_data_dir_parser(self):
-        """Test xdg_data_dir_parser"""
-        good_value = 'hola'
+    good_value = '~/hola/mundo'
+    name = 'home'
+    xdg_dir = os.path.join('', 'home', 'fake')
+
+    @defer.inlineCallbacks
+    def setUp(self):
+        yield super(XdgHomeParsersTests, self).setUp()
+        self.parser = getattr(config, '%s_dir_parser' % self.name)
+
+    def test_good_value(self):
+        """Test the parser using a good value."""
+        homedir = os.path.join('', 'home', 'fake')
+        with environ('HOME', homedir):
+            expected = os.path.join(self.xdg_dir, 'hola', 'mundo')
+            actual = self.parser(self.good_value)
+            self.assertEqual(expected, actual)
+            self.assertIsInstance(actual, str)
+            self.assertNotIsInstance(actual, unicode)
+
+    def test_bad_value(self):
+        """Test the parser using a bad value."""
         bad_value = '/hola'
+        self.assertEqual(config.path_from_unix(bad_value),
+                         self.parser(bad_value))
+
+    def test_invalid_value(self):
+        """Test the parser using an invalid value."""
         invalid_value = None
-        parser = config.xdg_data_dir_parser
-        self.assertEquals(os.path.join(xdg_data_home, 'hola'),
-                          parser(good_value))
-        self.assertEquals('/hola', parser(bad_value))
-        self.assertRaises(AttributeError, parser, invalid_value)
+        self.assertRaises(AttributeError, self.parser, invalid_value)
+
+
+class XdgCacheParsersTests(XdgHomeParsersTests):
+    """Tests for our custom xdg parsers."""
+
+    good_value = 'hola/mundo'
+    name = 'xdg_cache'
+    xdg_dir = xdg_cache_home
+
+
+class XdgDataParsersTests(XdgCacheParsersTests):
+    """Tests for our custom xdg parsers."""
+
+    good_value = 'hola/mundo'
+    name = 'xdg_data'
+    xdg_dir = xdg_data_home
 
 
 class SyncDaemonConfigParserTests(BaseTwistedTestCase):
     """Tests for SyncDaemonConfigParser"""
 
+    @defer.inlineCallbacks
     def setUp(self):
-        BaseTwistedTestCase.setUp(self)
+        yield super(SyncDaemonConfigParserTests, self).setUp()
         self.test_root = self.mktemp()
         self.default_config = os.path.join(os.environ['ROOTDIR'], 'data',
                                            'syncdaemon.conf')
@@ -546,10 +568,10 @@ class SyncDaemonConfigParserTests(BaseTwistedTestCase):
         """Test log_level upgrade hook."""
         conf_file = os.path.join(self.test_root, 'test_old_config.conf')
         # write some throttling values to the config file
-        with open(conf_file, 'w') as fp:
+        with open_file(conf_file, 'w') as fp:
             fp.write('[__main__]\n')
             fp.write('log_level = DEBUG\n')
-        self.assertTrue(os.path.exists(conf_file))
+        self.assertTrue(path_exists(conf_file))
         self.cp.read([conf_file])
         self.cp.parse_all()
         self.assertEquals(self.cp.get('logging', 'level').value, 10)
@@ -558,10 +580,10 @@ class SyncDaemonConfigParserTests(BaseTwistedTestCase):
         """Test log_level upgrade hook with new config"""
         conf_file = os.path.join(self.test_root, 'test_new_config.conf')
         # write some throttling values to the config file
-        with open(conf_file, 'w') as fp:
+        with open_file(conf_file, 'w') as fp:
             fp.write('[logging]\n')
             fp.write('level = DEBUG\n')
-        self.assertTrue(os.path.exists(conf_file))
+        self.assertTrue(path_exists(conf_file))
         self.cp.read([conf_file])
         self.cp.parse_all()
         self.assertEquals(self.cp.get('logging', 'level').value, 10)
@@ -571,12 +593,12 @@ class SyncDaemonConfigParserTests(BaseTwistedTestCase):
         conf_file = os.path.join(self.test_root,
                                  'test_old_and_new_config.conf')
         # write some throttling values to the config file (not default ones)
-        with open(conf_file, 'w') as fp:
+        with open_file(conf_file, 'w') as fp:
             fp.write('[__main__]\n')
             fp.write('log_level = NOTE\n')
             fp.write('[logging]\n')
             fp.write('level = ERROR\n')
-        self.assertTrue(os.path.exists(conf_file))
+        self.assertTrue(path_exists(conf_file))
         self.cp.read([conf_file])
         self.cp.parse_all()
         self.assertEquals(self.cp.get('logging', 'level').value, logging.ERROR)
@@ -610,10 +632,10 @@ class SyncDaemonConfigParserTests(BaseTwistedTestCase):
     def test_ignore_one(self):
         """Test ignore files config, one regex."""
         conf_file = os.path.join(self.test_root, 'test_new_config.conf')
-        with open(conf_file, 'w') as fp:
+        with open_file(conf_file, 'w') as fp:
             fp.write('[__main__]\n')
             fp.write('ignore = .*\\.pyc\n')  # all .pyc files
-        self.assertTrue(os.path.exists(conf_file))
+        self.assertTrue(path_exists(conf_file))
         self.cp.read([conf_file])
         self.cp.parse_all()
         self.assertEquals(self.cp.get('__main__', 'ignore').value,
@@ -622,11 +644,11 @@ class SyncDaemonConfigParserTests(BaseTwistedTestCase):
     def test_ignore_two(self):
         """Test ignore files config, two regexes."""
         conf_file = os.path.join(self.test_root, 'test_new_config.conf')
-        with open(conf_file, 'w') as fp:
+        with open_file(conf_file, 'w') as fp:
             fp.write('[__main__]\n')
             fp.write('ignore = .*\\.pyc\n')  # all .pyc files
             fp.write('         .*\\.sw[opnx]\n')  # all gvim temp files
-        self.assertTrue(os.path.exists(conf_file))
+        self.assertTrue(path_exists(conf_file))
         self.cp.read([conf_file])
         self.cp.parse_all()
         self.assertEquals(self.cp.get('__main__', 'ignore').value,
