@@ -20,30 +20,33 @@
 """Tests for the Event Queue."""
 
 import logging
-import unittest
 
+from twisted.internet import defer
+from twisted.trial.unittest import TestCase
+
+from contrib.testing.testcase import BaseTwistedTestCase, FakeVolumeManager
 from ubuntuone.syncdaemon import (
     event_queue,
     filesystem_manager,
     tritcask,
 )
-from contrib.testing import testcase
-from twisted.internet import defer
 from ubuntuone.devtools.handlers import MementoHandler
 
 
-class BaseEQTestCase(testcase.BaseTwistedTestCase):
+class BaseEQTestCase(BaseTwistedTestCase):
     """ Setup an EQ for test. """
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Setup the test."""
-        testcase.BaseTwistedTestCase.setUp(self)
+        yield super(BaseEQTestCase, self).setUp()
         self.fsmdir = self.mktemp('fsmdir')
         self.partials_dir = self.mktemp('partials_dir')
         self.root_dir = self.mktemp('root_dir')
         self.home_dir = self.mktemp('home_dir')
-        self.vm = testcase.FakeVolumeManager(self.root_dir)
+        self.vm = FakeVolumeManager(self.root_dir)
         self.db = tritcask.Tritcask(self.mktemp('tritcask'))
+        self.addCleanup(self.db.shutdown)
         self.fs = filesystem_manager.FileSystemManager(self.fsmdir,
                                                        self.partials_dir,
                                                        self.vm, self.db)
@@ -52,6 +55,8 @@ class BaseEQTestCase(testcase.BaseTwistedTestCase):
         self.fs.set_by_path(path=self.root_dir,
                               local_hash=None, server_hash=None)
         self.eq = event_queue.EventQueue(self.fs)
+        self.eq.listener_map = {}
+        self.addCleanup(self.eq.shutdown)
         self.fs.register_eq(self.eq)
 
         # add a Memento handler to the logger
@@ -59,14 +64,6 @@ class BaseEQTestCase(testcase.BaseTwistedTestCase):
         self.log_handler.setLevel(logging.DEBUG)
         self.eq.log.addHandler(self.log_handler)
 
-    def tearDown(self):
-        """Clean up the tests."""
-        self.eq.listener_map = {}
-        self.eq.shutdown()
-        self.db.shutdown()
-        self.rmtree(self.tmpdir)
-        testcase.BaseTwistedTestCase.tearDown(self)
-
 
 class SubscriptionTests(BaseEQTestCase):
     """Test to subscribe and unsubscribe to the EQ."""
@@ -386,9 +383,30 @@ class PushTestsWithCallback(BaseEQTestCase):
         return d
 
 
-def test_suite():
-    # pylint: disable-msg=C0111
-    return unittest.TestLoader().loadTestsFromName(__name__)
+class FakeMonitor(object):
+    """A fake FilesystemMonitor."""
 
-if __name__ == "__main__":
-    unittest.main()
+    def __init__(self, *args):
+        """Initialize this fake."""
+        self.shutdown_d = defer.Deferred()
+
+    def shutdown(self):
+        """Get the shutdown deferred."""
+        return self.shutdown_d
+
+
+class EventQueueShutdownTestCase(TestCase):
+    """Test the shutdown method in EQ."""
+
+    timeout = 2
+
+    @defer.inlineCallbacks
+    def test_shutdown_defers(self):
+        """The shutdown method in eq defers on the shutdown of the monitor."""
+        self.patch(event_queue, "FilesystemMonitor", FakeMonitor)
+        eq = event_queue.EventQueue(None)
+        d = eq.shutdown()
+        self.assertFalse(d.called, "shutdown is fired after the monitor.")
+        eq.monitor.shutdown_d.callback(True)
+        self.assertTrue(d.called, "shutdown is fired after the monitor.")
+        yield d
