@@ -1,7 +1,8 @@
 #
-# Author: Facundo Batista <facundo@canonical.com>
+# Authors: Facundo Batista <facundo@canonical.com>
+#          Alejandro J. Cura <alecu@canonical.com>
 #
-# Copyright 2009 Canonical Ltd.
+# Copyright 2009-2011 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 3, as published
@@ -19,7 +20,6 @@
 from __future__ import with_statement
 
 import os
-import unittest
 import random
 import time
 import logging
@@ -28,28 +28,93 @@ import threading
 from StringIO import StringIO
 from twisted.trial.unittest import TestCase as TwistedTestCase
 from twisted.internet import defer, reactor
-
-from contrib.testing import testcase
 from ubuntuone.devtools.handlers import MementoHandler
+
+from contrib.testing.testcase import BaseTwistedTestCase
+from ubuntuone.platform import open_file, stat_path
 from ubuntuone.syncdaemon import hash_queue
-from ubuntuone.storageprotocol.content_hash import \
-            content_hash_factory, crc32
+from ubuntuone.syncdaemon.hash_queue import HASHQUEUE_DELAY
+from ubuntuone.storageprotocol.content_hash import content_hash_factory, crc32
+
+FAKE_TIMESTAMP = 1
+
+
+class FakeEventQueue(object):
+    """Helper class."""
+
+    def __init__(self, deferred, expected_events=1):
+        """Initialize this fake instance."""
+        self.deferred = deferred
+        self.received = []
+        self.expected_events = expected_events
+
+    def push(self, event, **kwargs):
+        """Callback."""
+        self.received.append((hash_queue.time.time(), event, kwargs))
+        if len(self.received) == self.expected_events:
+            self.deferred.callback(event)
+
+
+class FakeTimeModule(object):
+    """A fake time module."""
+
+    def __init__(self):
+        """Initialize this fake instance."""
+        self.timestamp = 1000
+        self.sleep_calls = []
+        self.time_calls = []
+
+    def time(self):
+        """Return the current timestamp."""
+        self.time_calls.append(self.timestamp)
+        return self.timestamp
+
+    def sleep(self, delay):
+        """A forced nap."""
+        self.sleep_calls.append(delay)
+        self.timestamp += delay
+
+
+class FixedOrderedDictTestCase(BaseTwistedTestCase):
+    """Tests for the custom OrderedDict based on t.p.u.OrderedDict."""
+
+    def test_popitem_first(self):
+        """The popitem method understands last=False like in 2.7."""
+        od = hash_queue.FixedOrderedDict()
+        for x in range(1, 10):
+            od[x] = x * 2
+        item = od.popitem(last=False)
+        self.assertEqual(item, (1, 2))
+        item = od.popitem(last=True)
+        self.assertEqual(item, (9, 18))
+
+    def test_clear(self):
+        """The clear method resets the order of items."""
+        range1 = range(9, -1, -1)
+        range2 = range(0, 10)
+
+        od = hash_queue.FixedOrderedDict()
+        for x in range1:
+            od[x] = x
+        self.assertEqual(od.keys(), range1)
+        od.clear()
+        for x in range2:
+            od[x] = x
+        self.assertEqual(od.keys(), range2)
 
 
 # this is a unit test we need to access protected members
 # pylint: disable-msg=W0212
-class HasherTests(testcase.BaseTwistedTestCase):
+class HasherTests(BaseTwistedTestCase):
     """Test the whole stuff to receive signals."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Setup the test."""
-        testcase.BaseTwistedTestCase.setUp(self)
+        yield super(HasherTests, self).setUp()
         self.test_dir = self.mktemp('test_dir')
         self.timeout = 2
-
-    def tearDown(self):
-        """Clean up the tests."""
-        testcase.BaseTwistedTestCase.tearDown(self)
+        self.patch(hash_queue, "HASHQUEUE_DELAY", 0.0)
 
     def test_live_process(self):
         """Check that the hasher lives and dies."""
@@ -89,7 +154,6 @@ class HasherTests(testcase.BaseTwistedTestCase):
                 d.callback(kwargs)
         receiver = Helper()
         hasher = hash_queue._Hasher(queue, mark, receiver)
-        hasher.start()
 
         # log config
         handler = MementoHandler()
@@ -98,9 +162,13 @@ class HasherTests(testcase.BaseTwistedTestCase):
 
         # send what to hash
         testfile = os.path.join(self.test_dir, "testfile")
-        with open(testfile, "w") as fh:
+        with open_file(testfile, "wb") as fh:
             fh.write("foobar")
-        queue.put((testfile, "mdid"))
+        item = ((testfile, "mdid"), FAKE_TIMESTAMP)
+        queue.put(item)
+
+        # start the hasher after putting the work items
+        hasher.start()
 
         # wait event and stop hasher
         yield d
@@ -132,13 +200,16 @@ class HasherTests(testcase.BaseTwistedTestCase):
                 d.callback((event, kwargs))
         receiver = Helper()
         hasher = hash_queue._Hasher(queue, mark, receiver)
-        hasher.start()
 
         # send what to hash
         testfile = os.path.join(self.test_dir, "testfile")
-        with open(testfile, "w") as fh:
+        with open_file(testfile, "wb") as fh:
             fh.write("foobar")
-        queue.put((testfile, "mdid"))
+        item = ((testfile, "mdid"), FAKE_TIMESTAMP)
+        queue.put(item)
+
+        # start the hasher after putting the work items
+        hasher.start()
 
         def check_info((event, kwargs)):
             """check the info pushed by the hasher"""
@@ -150,7 +221,7 @@ class HasherTests(testcase.BaseTwistedTestCase):
             realh = content_hash_factory()
             realh.hash_object.update("foobar")
             should_be = realh.content_hash()
-            curr_stat = os.stat(testfile)
+            curr_stat = stat_path(testfile)
             self.assertEquals(should_be, kwargs['hash'])
             for attr in ('st_mode', 'st_ino', 'st_dev', 'st_nlink', 'st_uid',
                          'st_gid', 'st_size', 'st_ctime', 'st_mtime'):
@@ -175,10 +246,13 @@ class HasherTests(testcase.BaseTwistedTestCase):
                 d.callback((event, kwargs))
         receiver = Helper()
         hasher = hash_queue._Hasher(queue, mark, receiver)
-        hasher.start()
 
         # send what to hash
-        queue.put(("not_to_be_found", "foo"))
+        item = (("not_to_be_found", "foo"), FAKE_TIMESTAMP)
+        queue.put(item)
+
+        # start the hasher after putting the work items
+        hasher.start()
 
         def check_info((event, kwargs)):
             """Check the info pushed by the hasher."""
@@ -201,10 +275,10 @@ class HasherTests(testcase.BaseTwistedTestCase):
             text = "supercalifragilistico"+str(i)
             hasher.hash_object.update(text)
             tfile = os.path.join(self.test_dir, "tfile"+str(i))
-            with open(tfile, "w") as fh:
+            with open_file(tfile, "wb") as fh:
                 fh.write("supercalifragilistico"+str(i))
             d = dict(path=tfile, hash=hasher.content_hash(),
-                     crc32=crc32(text), size=len(text), stat=os.stat(tfile))
+                     crc32=crc32(text), size=len(text), stat=stat_path(tfile))
             should_be.append(("HQ_HASH_NEW", d))
 
         # create the hasher
@@ -230,12 +304,16 @@ class HasherTests(testcase.BaseTwistedTestCase):
         receiver = Helper()
 
         hasher = hash_queue._Hasher(queue, mark, receiver)
-        hasher.start()
 
         # send what to hash
         for i in range(10):
             tfile = os.path.join(self.test_dir, "tfile"+str(i))
-            queue.put((tfile, "mdid"))
+            item = ((tfile, "mdid"), FAKE_TIMESTAMP)
+            queue.put(item)
+
+        # start the hasher after putting the work items
+        hasher.start()
+
         return d
 
     def test_large_content(self):
@@ -269,32 +347,156 @@ class HasherTests(testcase.BaseTwistedTestCase):
                     d.callback(True)
         receiver = Helper()
         hasher = hash_queue._Hasher(queue, mark, receiver)
-        hasher.start()
         # send what to hash
-        with open(testfile, "w") as fh:
+        with open_file(testfile, "wb") as fh:
             fh.write(testinfo)
-        queue.put((testfile, "mdid"))
+        item = ((testfile, "mdid"), FAKE_TIMESTAMP)
+        queue.put(item)
+
+        # start the hasher after putting the work items
+        hasher.start()
+
         return d
 
+    @defer.inlineCallbacks
+    def test_open_file_with_rb(self):
+        """Check that the file to hash is opened with 'rb' mode."""
+        called = []
 
-class HashQueueTests(testcase.BaseTwistedTestCase):
+        orig = hash_queue.open_file
+        def faked_open_file(*a):
+            called.append(a)
+            return orig(*a)
+
+        self.patch(hash_queue, 'open_file', faked_open_file)
+
+        queue = hash_queue.UniqueQueue()
+        testfile = os.path.join(self.test_dir, "testfile")
+        with open_file(testfile, "wb") as fh:
+            fh.write("foobar")
+        item = ((testfile, "mdid"), FAKE_TIMESTAMP)
+        queue.put(item)
+
+        d = defer.Deferred()
+        eq = FakeEventQueue(d)
+
+        hasher = hash_queue._Hasher(queue=queue, end_mark='end-mark',
+                                    event_queue=eq)
+        # start the hasher after putting the work items
+        hasher.start()
+
+        yield d
+        hasher.stop()
+
+        self.assertEqual(called, [(testfile, 'rb')])
+
+
+class HasherSleepTests(BaseTwistedTestCase):
+    """The hasher thread sleeps sometimes too."""
+
+    timeout = 3
+
+    def setUpWithCount(self, expected_events=1):
+        """Initialize this test instance."""
+        self.test_dir = self.mktemp('test_dir')
+        self.fake_time = FakeTimeModule()
+        self.patch(hash_queue, "time", self.fake_time)
+        self.queue = hash_queue.UniqueQueue()
+
+        self.testfile = os.path.join(self.test_dir, "testfile")
+        with open_file(self.testfile, "wb") as fh:
+            fh.write("foobar")
+
+        self.event_d = defer.Deferred()
+        self.eq = FakeEventQueue(self.event_d, expected_events)
+        self.hasher = hash_queue._Hasher(queue=self.queue, end_mark='end-mark',
+                                    event_queue=self.eq)
+
+        def stop_hasher():
+            """Safely stop the hasher."""
+            self.hasher.stop()
+            self.hasher.join(timeout=5)
+            self.assertFalse(self.hasher.isAlive())
+
+        self.addCleanup(stop_hasher)
+
+    @defer.inlineCallbacks
+    def test_hasher_sleeps(self):
+        """The hasher thread sleeps while it waits."""
+        self.setUpWithCount(expected_events=1)
+        item_time = self.fake_time.timestamp + HASHQUEUE_DELAY
+        item = ((self.testfile, "fake_mdid"), item_time)
+        self.queue.put(item)
+        # start the hasher after putting the work items
+        self.hasher.start()
+        yield self.event_d
+        self.assertEqual(self.fake_time.sleep_calls, [HASHQUEUE_DELAY])
+
+    @defer.inlineCallbacks
+    def test_hasher_does_not_sleep_if_not_needed(self):
+        """The hasher thread does not sleep if not needed."""
+        self.setUpWithCount(expected_events=1)
+        item_time = self.fake_time.timestamp - 1
+        item = ((self.testfile, "fake_mdid"), item_time)
+        self.queue.put(item)
+        # start the hasher after putting the work items
+        self.hasher.start()
+        yield self.event_d
+        self.assertEqual(self.fake_time.sleep_calls, [])
+
+    @defer.inlineCallbacks
+    def test_hasher_sleeping_complex(self):
+        """A complex test for the sleeping beauty."""
+        self.setUpWithCount(expected_events=3)
+
+        initial_time = self.fake_time.timestamp
+
+        item_time1 = initial_time + HASHQUEUE_DELAY
+        item1 = ((self.testfile, "fake_mdid1"), item_time1)
+        self.queue.put(item1)
+
+        item_time2 = initial_time + HASHQUEUE_DELAY * 1.1
+        item2 = ((self.testfile, "fake_mdid2"), item_time2)
+        self.queue.put(item2)
+
+        item_time3 = initial_time + HASHQUEUE_DELAY * 1.2
+        item3 = ((self.testfile, "fake_mdid2"), item_time3)
+        self.queue.put(item3)
+
+        item_time4 = initial_time + HASHQUEUE_DELAY * 1.3
+        item4 = ((self.testfile, "fake_mdid3"), item_time4)
+        self.queue.put(item4)
+
+        item_time5 = initial_time + HASHQUEUE_DELAY * 1.4
+        item5 = ((self.testfile, "fake_mdid1"), item_time5)
+        self.queue.put(item5)
+        # start the hasher after putting the work items
+        self.hasher.start()
+
+        yield self.event_d
+        expected_naps = [
+            item_time3 - initial_time,
+            item_time4 - item_time3,
+            item_time5 - item_time4,
+        ]
+        self.assertEqual(self.fake_time.sleep_calls, expected_naps)
+
+
+
+class HashQueueTests(BaseTwistedTestCase):
     """Test the whole stuff to receive signals."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Setup the test."""
-        testcase.BaseTwistedTestCase.setUp(self)
+        yield super(HashQueueTests, self).setUp()
         self.test_dir = self.mktemp('test_dir')
         self.timeout = 2
+        self.patch(hash_queue, "HASHQUEUE_DELAY", 0.0)
         self.log = logging.getLogger("ubuntuone.SyncDaemon.TEST")
         self.log.info("starting test %s.%s", self.__class__.__name__,
                       self._testMethodName)
 
-    def tearDown(self):
-        """Clean up the tests."""
-        self.log.info("finished test %s.%s", self.__class__.__name__,
-                      self._testMethodName)
-        testcase.BaseTwistedTestCase.tearDown(self)
-
     def test_called_back_ok(self):
         """Test that the hasher produces correct info."""
         # create the hasher
@@ -310,7 +512,7 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
 
         # send what to hash
         testfile = os.path.join(self.test_dir, "testfile")
-        with open(testfile, "w") as fh:
+        with open_file(testfile, "wb") as fh:
             fh.write("foobar")
         hq.insert(testfile, "mdid")
 
@@ -322,7 +524,7 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
             realh = content_hash_factory()
             realh.hash_object.update("foobar")
             should_be = realh.content_hash()
-            curr_stat = os.stat(testfile)
+            curr_stat = stat_path(testfile)
             self.assertEquals(should_be, kwargs['hash'])
             for attr in ('st_mode', 'st_ino', 'st_dev', 'st_nlink', 'st_uid',
                          'st_gid', 'st_size', 'st_ctime', 'st_mtime'):
@@ -371,9 +573,9 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
     def test_being_hashed(self):
         """Tell if something is being hashed."""
         tfile1 = os.path.join(self.test_dir, "tfile1")
-        open(tfile1, "w").close()
+        open_file(tfile1, "wb").close()
         tfile2 = os.path.join(self.test_dir, "tfile2")
-        open(tfile2, "w").close()
+        open_file(tfile2, "wb").close()
 
         class C(object):
             """Bogus."""
@@ -417,10 +619,10 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
             text = "supercalifragilistico"+str(i)
             hasher.hash_object.update(text)
             tfile = os.path.join(self.test_dir, "tfile"+str(i))
-            with open(tfile, "w") as fh:
+            with open_file(tfile, "wb") as fh:
                 fh.write("supercalifragilistico"+str(i))
             d = dict(path=tfile, hash=hasher.content_hash(),
-                     crc32=crc32(text), size=len(text), stat=os.stat(tfile))
+                     crc32=crc32(text), size=len(text), stat=stat_path(tfile))
             should_be.append(("HQ_HASH_NEW", d))
 
         d = defer.Deferred()
@@ -457,10 +659,10 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
             text = "supercalifragilistico"+str(i)
             hasher.hash_object.update(text)
             tfile = os.path.join(self.test_dir, "tfile"+str(i))
-            with open(tfile, "w") as fh:
+            with open_file(tfile, "wb") as fh:
                 fh.write("supercalifragilistico"+str(i))
             d = dict(path=tfile, hash=hasher.content_hash(),
-                     crc32=crc32(text), size=len(text), stat=os.stat(tfile))
+                     crc32=crc32(text), size=len(text), stat=stat_path(tfile))
             should_be.append(("HQ_HASH_NEW", d))
 
         d = defer.Deferred()
@@ -499,7 +701,7 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
         # start the hasher
         self.log.debug('Hasher started (forced)')
         hq.hasher.start()
-        # insert the last item to check the unqieness in the queue while
+        # insert the last item to check the uniqueness in the queue while
         # the hasher is running
         for i in range(9, 10):
             tfile = os.path.join(self.test_dir, "tfile"+str(i))
@@ -507,7 +709,7 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
         return d
 
     def test_interrupt_current(self):
-        """ Test that the hasher correctly interrupts a inprogress task."""
+        """Test that the hasher correctly interrupts a inprogress task."""
         # calculate what we should receive
         testinfo = os.urandom(1000)
         hasher = content_hash_factory()
@@ -515,44 +717,53 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
         testfile = os.path.join(self.test_dir, "testfile")
         testhash = hasher.content_hash()
         # send what to hash
-        with open(testfile, "w") as fh:
+        with open_file(testfile, "wb") as fh:
             fh.write(testinfo)
 
         d = defer.Deferred()
 
         class Helper(object):
-            """Helper class."""
+            """Helper class, acts as a event_queue."""
+
             def push(self, event, path, hash, crc32, size, stat):
                 """Callback."""
+                msg = None
                 if event != "HQ_HASH_NEW":
-                    d.errback(Exception("envent is not HQ_HASH_NEW"))
+                    msg = "envent is not HQ_HASH_NEW (got %r instead)." % event
                 elif path != testfile:
-                    d.errback(Exception("path is not the original one"))
+                    msg = "path is not the original %r (got %r instead)." % \
+                          (testfile, path)
                 elif hash != testhash:
-                    d.errback(Exception("the hashes are different!"))
+                    msg = "hash is not the original %r (got %r instead)." % \
+                          (testhash, hash)
+                if msg:
+                    d.errback(Exception(msg))
                 else:
                     d.callback(True)
 
         class FakeFile(StringIO):
-            """An endless file"""
+            """An endless file."""
+
             def read(self, size=10):
-                """return random bytes"""
+                """Return random bytes."""
                 return os.urandom(size)
+
             # context manager API
             def __enter__(self):
                 return self
+
             def __exit__(self, *args):
                 pass
 
-        # patch __builtin__.open so we don't have to try to interrupt a hashing
-        bi = __import__('__builtin__')
-        old_open = bi.open
+        old_open = hash_queue.open_file
 
         class OpenFaker(object):
             """A class to fake open for specific paths"""
+
             def __init__(self, paths):
                 self.paths = paths
                 self.done = False
+
             def __call__(self, path, mode='r'):
                 """the custom open implementation"""
                 if self.done or path not in self.paths:
@@ -560,14 +771,15 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
                 else:
                     self.done = True
                     return FakeFile()
+
         open_faker = OpenFaker([testfile])
-        bi.open = open_faker
+        hash_queue.open_file = open_faker
 
         receiver = Helper()
         hq = hash_queue.HashQueue(receiver)
         def cleanup():
-            """cleanup the mess"""
-            bi.open = old_open
+            """Cleanup the mess."""
+            hash_queue.open_file = old_open
             hq.shutdown()
         self.addCleanup(cleanup)
         hq.insert(testfile, "mdid")
@@ -595,7 +807,7 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
         hasher.hash_object.update(testinfo)
         testfile = os.path.join(self.test_dir, "testfile")
         # send what to hash
-        with open(testfile, "w") as fh:
+        with open_file(testfile, "wb") as fh:
             fh.write(testinfo)
 
         class Helper(object):
@@ -604,7 +816,6 @@ class HashQueueTests(testcase.BaseTwistedTestCase):
                 """Callback."""
         receiver = Helper()
         hq = hash_queue.HashQueue(receiver)
-        self.addCleanup(hq.shutdown)
         # read in small chunks, so we have more iterations
         hq.hasher.chunk_size = 2**10
         hq.insert(testfile, "mdid")
@@ -640,35 +851,46 @@ class UniqueQueueTests(TwistedTestCase):
         queue.put(('item1', "mdid"))
         queue.put(('item1', "mdid"))
         self.assertEquals(1, queue.qsize())
-        self.assertEquals(1, len(queue._set))
         queue.get()
         self.assertEquals(0, queue.qsize())
-        self.assertEquals(0, len(queue._set))
         queue.put(('item1', "mdid"))
         queue.put(('item2', "mdid"))
         queue.put(('item1', "mdid"))
         queue.put(('item2', "mdid"))
         self.assertEquals(2, queue.qsize())
-        self.assertEquals(2, len(queue._set))
         queue.get()
         queue.get()
         self.assertEquals(0, queue.qsize())
-        self.assertEquals(0, len(queue._set))
+
+    def test_previous_item_discarded(self):
+        """It's the previous instance of an item the one that's discarded."""
+        item1 = ('item1', "mdid")
+        item2 = ('item2', "mdid")
+        queue = hash_queue.UniqueQueue()
+        queue.put(item1)
+        queue.put(item2)
+        queue.put(item1)
+        result = queue.get()
+        self.assertEqual(result, item2)
 
     def test_contains(self):
         """test contains functionality"""
         queue = hash_queue.UniqueQueue()
+        fake_timestamp_1 = 1
+        fake_timestamp_2 = 2
 
         # nothing in it
         self.assertFalse(("item1", "mdid") in queue)
 
         # put one and check
-        queue.put(('item1', "mdid"))
+        item1 = (('item1', "mdid"), fake_timestamp_1)
+        queue.put(item1)
         self.assertTrue(("item1", "mdid") in queue)
         self.assertFalse(("item2", "mdid") in queue)
 
         # put second and check
-        queue.put(('item2', "mdid"))
+        item2 = (('item2', "mdid"), fake_timestamp_2)
+        queue.put(item2)
         self.assertTrue(("item1", "mdid") in queue)
         self.assertTrue(("item2", "mdid") in queue)
 
@@ -712,10 +934,3 @@ class UniqueQueueTests(TwistedTestCase):
         t.start()
         reactor.callLater(0.1, queue.clear)
         return d
-
-def test_suite():
-    # pylint: disable-msg=C0111
-    return unittest.TestLoader().loadTestsFromName(__name__)
-
-if __name__ == "__main__":
-    unittest.main()
