@@ -20,6 +20,7 @@ from __future__ import with_statement
 
 import logging
 
+from twisted.internet import defer
 from twisted.trial import unittest
 
 from ubuntuone.devtools.handlers import MementoHandler
@@ -39,15 +40,19 @@ from ubuntuone.syncdaemon.logger import (
 class DebugCaptureTest(unittest.TestCase):
     """Tests for DebugCapture context manager."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Setup the logger and the handler"""
+        yield super(DebugCaptureTest, self).setUp()
         self.handler = MementoHandler()
         self.logger = logging.getLogger(self.__class__.__name__)
         self.logger.addHandler(self.handler)
         self.logger.setLevel(logging.DEBUG)
 
+    @defer.inlineCallbacks
     def tearDown(self):
         """close the handler and restore the logger (Logger's are global)"""
+        yield super(DebugCaptureTest, self).tearDown()
         self.handler.close()
         self.logger.removeHandler(self.handler)
         self.logger.setLevel(logging.DEBUG)
@@ -222,8 +227,10 @@ class DebugCaptureTest(unittest.TestCase):
 class FilterTests(unittest.TestCase):
     """Tests log filters"""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Setup the logger and the handler"""
+        yield super(FilterTests, self).setUp()
         self.handler = MementoHandler()
         self.handler.setLevel(logging.DEBUG)
         root_logger.addHandler(self.handler)
@@ -253,15 +260,19 @@ class FilterTests(unittest.TestCase):
 class MultiFilterTest(unittest.TestCase):
     """Tests for logger.MultiFilter"""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Setup the logger and the handler"""
+        yield super(MultiFilterTest, self).setUp()
         self.handler = MementoHandler()
         self.logger = logging.getLogger(self.__class__.__name__)
         self.logger.addHandler(self.handler)
         self.logger.setLevel(logging.DEBUG)
 
+    @defer.inlineCallbacks
     def tearDown(self):
         """close the handler and restore the logger (Logger's are global)"""
+        yield super(MultiFilterTest, self).tearDown()
         self.handler.close()
         self.logger.removeHandler(self.handler)
         self.logger.setLevel(logging.DEBUG)
