@@ -29,8 +29,10 @@ from ubuntuone.syncdaemon.action_queue import PathLockingTree
 class InternalDeferredTests(TwistedTestCase):
     """Test the internal deferreds handling functionality."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Set up."""
+        yield super(InternalDeferredTests, self).setUp()
         self.plt = PathLockingTree()
 
     def test_single_element_old(self):
@@ -217,8 +219,10 @@ class InternalDeferredTests(TwistedTestCase):
 class LockingTests(TwistedTestCase):
     """Test the locking between elements."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Set up."""
+        yield super(LockingTests, self).setUp()
         self.plt = PathLockingTree()
 
     @defer.inlineCallbacks
@@ -417,8 +421,10 @@ class LockingTests(TwistedTestCase):
 class CleaningTests(TwistedTestCase):
     """Test that the releases clean the tree."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Set up."""
+        yield super(CleaningTests, self).setUp()
         self.plt = PathLockingTree()
 
     @defer.inlineCallbacks
@@ -547,8 +553,10 @@ class CleaningTests(TwistedTestCase):
 class LoggingTests(TwistedTestCase):
     """Test the logging."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Set up."""
+        yield super(LoggingTests, self).setUp()
         self.plt = PathLockingTree()
 
         self.handler = MementoHandler()
