@@ -107,7 +107,9 @@ class FakeMain(object):
 class Base(TwistedTestCase):
     """Base class for state tests."""
 
+    @defer.inlineCallbacks
     def setUp(self):
+        yield super(Base, self).setUp()
         # create fake classes
         self.eq = FakeEventQueue()
         self.aq = FakeActionQueue()
@@ -157,15 +159,18 @@ class Base(TwistedTestCase):
         reactor.callLater(.1, check, 0)
         return d
 
+    @defer.inlineCallbacks
     def tearDown(self):
+        yield super(Base, self).tearDown()
         self.sm.connection.shutdown()
 
 
 class QueueBase(Base):
     """Basic setup for QueueManager."""
 
+    @defer.inlineCallbacks
     def setUp(self):
-        Base.setUp(self)
+        yield super(QueueBase, self).setUp()
         self.sm.state = StateManager.QUEUE_MANAGER
 
 
@@ -223,8 +228,9 @@ class TestQueueManagerFromOutside(QueueBase):
 class TestConnectionManager(Base):
     """Test the "internal network" transitions."""
 
+    @defer.inlineCallbacks
     def setUp(self):
-        Base.setUp(self)
+        yield super(TestConnectionManager, self).setUp()
 
         # put SM on some state that does not generate further
         # transition-related efforts for this CM
@@ -341,8 +347,9 @@ class TestConnectionManager(Base):
 class TestConnectionManagerTimings(Base):
     """Times handled by ConnectionManager."""
 
+    @defer.inlineCallbacks
     def setUp(self):
-        Base.setUp(self)
+        yield super(TestConnectionManagerTimings, self).setUp()
 
         # set timeout values to really low, to make tests run quicker
         self.sm.connection.handshake_timeout = 0
@@ -843,8 +850,10 @@ class TestStateManagerEnterExit(Base):
 class TestStateManagerPassToNetworkManager(Base):
     """All network events should go to NetworkManager."""
 
+    @defer.inlineCallbacks
     def setUp(self):
-        Base.setUp(self)
+        yield super(TestStateManagerPassToNetworkManager,
+                    self).setUp()
 
         # put a function in the middle to log calls
         self.called_events = []
@@ -893,8 +902,10 @@ class TestStateManagerPassToNetworkManager(Base):
 class TestStateManagerPassToQueueManager(Base):
     """All queue events should go to QueueManager."""
 
+    @defer.inlineCallbacks
     def setUp(self):
-        Base.setUp(self)
+        yield super(TestStateManagerPassToQueueManager,
+                    self).setUp()
 
         # put a function in the middle to log calls
         self.called_events = []
