@@ -96,9 +96,10 @@ class FakeStatusFrontend(object):
 class StatusListenerTestCase(FakeMainTestCase):
     """Tests for StatusListener."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Initialize this instance."""
-        super(StatusListenerTestCase, self).setUp()
+        yield super(StatusListenerTestCase, self).setUp()
         self.status_frontend = FakeStatusFrontend()
         self.listener = status_listener.StatusListener(self.fs, self.vm,
                                                        self.status_frontend)
