@@ -25,12 +25,12 @@ import copy
 from inspect import getmembers, getargspec, ismethod
 import logging
 import os
-import shutil
 import unittest
 import uuid
 
 from twisted.internet import defer
 from twisted.python.failure import Failure
+from ubuntuone.devtools.testcase import skipIfOS
 
 from contrib.testing.testcase import (
     FakeMain,
@@ -40,6 +40,12 @@ from contrib.testing.testcase import (
 )
 
 from ubuntuone.devtools.handlers import MementoHandler
+from ubuntuone.platform import (
+    make_dir,
+    open_file,
+    remove_file,
+    stat_path,
+)
 from ubuntuone.syncdaemon.filesystem_manager import FileSystemManager
 from ubuntuone.syncdaemon.tritcask import Tritcask
 from ubuntuone.syncdaemon.fsm import fsm as fsm_module
@@ -78,36 +84,28 @@ class FSKeyTestCase(BaseTwistedTestCase):
     @defer.inlineCallbacks
     def setUp(self):
         """ Setup the test """
-        unittest.TestCase.setUp(self)
-        self.test_dir = self.mktemp("test_dir")
-        self.shares_dir = os.path.join(self.test_dir, 'shares')
-        os.makedirs(self.shares_dir)
-        self.root_dir = os.path.join(self.test_dir, 'root')
-        os.makedirs(self.root_dir)
-        self.fsmdir = os.path.join(self.test_dir, "fsmdir")
-        self.partials_dir = os.path.join(self.test_dir, "partials")
-        os.makedirs(self.partials_dir)
+        yield super(FSKeyTestCase, self).setUp()
+        self.shares_dir = self.mktemp('shares')
+        self.root_dir = self.mktemp('root')
+        self.fsmdir = self.mktemp( "fsmdir")
+        self.partials_dir = self.mktemp("partials")
         self.tritcask_dir = self.mktemp("tritcask_dir")
         self.db = Tritcask(self.tritcask_dir)
+        self.addCleanup(self.db.shutdown)
         self.fsm = FileSystemManager(self.fsmdir, self.partials_dir,
                                      FakeVolumeManager(self.root_dir),
                                      self.db)
         self.eq = EventQueue(self.fsm)
+        self.addCleanup(self.eq.shutdown)
         self.fsm.register_eq(self.eq)
         self.share = yield self.create_share('share', 'share_name')
         self.share_path = self.share.path
 
-    def tearDown(self):
-        """ Clean up the test dir"""
-        self.eq.shutdown()
-        self.db.shutdown()
-        shutil.rmtree(self.test_dir)
-
     @defer.inlineCallbacks
     def create_share(self, share_id, share_name, access_level='Modify'):
         """Create a share."""
         share_path = os.path.join(self.shares_dir, share_name)
-        os.makedirs(share_path)
+        make_dir(share_path, recursive=True)
         share = Share(path=share_path, volume_id=share_id,
                       access_level=access_level)
         yield self.fsm.vm.add_share(share)
@@ -166,6 +164,7 @@ class FSKeyTests(FSKeyTestCase):
         """Test for mdid reset after a deletion"""
         # create a node
         path = os.path.join(self.share.path, 'path')
+        open_file(path, 'w').close()
         self.fsm.create(path, "share", node_id='uuid1')
         key = FSKey(self.fsm, path=path)
         # fake a conflict and delete the metadata
@@ -212,13 +211,48 @@ class FSKeyTests(FSKeyTestCase):
         self.assertNotEquals('server_hash1',
                              self.fsm.get_by_path(path).server_hash)
 
+    def test_subscribed_yes_nodecreated(self):
+        """Ask if subscribed (yes) for a node that exists."""
+        path = os.path.join(self.share.path, 'path')
+        self.share.subscribed = True
+        self.fsm.create(path, self.share.id, node_id='uuid1')
+        key = FSKey(self.fsm, path=path)
+        self.assertTrue(key.is_subscribed())
+
+    def test_subscribed_no_nodecreated(self):
+        """Ask if subscribed (no) for a node that exists."""
+        path = os.path.join(self.share.path, 'path')
+        assert not self.share.subscribed
+        self.fsm.create(path, self.share.id, node_id='uuid1')
+        key = FSKey(self.fsm, path=path)
+        self.assertFalse(key.is_subscribed())
+
+    def test_subscribed_yes_newnode(self):
+        """Ask if subscribed (yes) for a node that still has no metadata."""
+        path = os.path.join(self.share.path, 'path')
+        self.share.subscribed = True
+        self.fsm.create(path, self.share.id, node_id='uuid1')
+        child = os.path.join(path, 'child')
+        key = FSKey(self.fsm, path=child)
+        self.assertTrue(key.is_subscribed())
+
+    def test_subscribed_no_newnode(self):
+        """Ask if subscribed (no) for a node that still has no metadata."""
+        path = os.path.join(self.share.path, 'path')
+        assert not self.share.subscribed
+        self.fsm.create(path, self.share.id, node_id='uuid1')
+        child = os.path.join(path, 'child')
+        key = FSKey(self.fsm, path=child)
+        self.assertFalse(key.is_subscribed())
+
 
 class BaseSync(BaseTwistedTestCase):
     """Base test infrastructure for Sync."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Init."""
-        BaseTwistedTestCase.setUp(self)
+        yield super(BaseSync, self).setUp()
         self.root = self.mktemp('root')
         self.shares = self.mktemp('shares')
         self.data = self.mktemp('data')
@@ -229,58 +263,58 @@ class BaseSync(BaseTwistedTestCase):
         self.main = FakeMain(root_dir=self.root, shares_dir=self.shares,
                              data_dir=self.data,
                              partials_dir=self.partials_dir)
+        self.addCleanup(self.main.shutdown)
         self._logger = logging.getLogger('ubuntuone.SyncDaemon')
         self._logger.addHandler(self.handler)
+        self.addCleanup(self._logger.removeHandler, self.handler)
 
+    @defer.inlineCallbacks
     def tearDown(self):
         """Clean up."""
-        self._logger.removeHandler(self.handler)
-        self.main.shutdown()
         FakeMain._sync_class = None
-        shutil.rmtree(self.root)
-        shutil.rmtree(self.shares)
-        shutil.rmtree(self.data)
         for record in self.handler.records:
             exc_info = getattr(record, 'exc_info', None)
             if exc_info is not None:
                 raise exc_info[0], exc_info[1], exc_info[2]
-        BaseTwistedTestCase.tearDown(self)
+        yield super(BaseSync, self).tearDown()
+
+    @defer.inlineCallbacks
+    def create_share(self, share_id, share_name, access_level='Modify'):
+        """Create a share."""
+        share_path = os.path.join(self.shares, share_name)
+        share = Share(path=share_path, volume_id=share_id, node_id='shrootid',
+                      access_level=access_level, accepted=True)
+        yield self.fsm.vm.add_share(share)
+        defer.returnValue(share)
 
 
 class TestSync(BaseSync):
     """Test for Sync."""
 
+    timeout = 3
+
+    @defer.inlineCallbacks
     def setUp(self):
         """Set up."""
-        BaseSync.setUp(self)
+        yield super(TestSync, self).setUp()
         self.sync = Sync(main=self.main)
         self.fsm = self.main.fs
         self.handler.setLevel(logging.DEBUG)
 
+    @skipIfOS('win32', 'In windows we can not unlink opened files.')
     def test_deleting_open_files_is_no_cause_for_despair(self):
         """test_deleting_open_files_is_no_cause_for_despair."""
         def cb(_):
             d0 = self.main.wait_for('HQ_HASH_NEW')
-            f = file(self.root + '/a_file', 'w')
-            print >> f, 'hola'
-            os.unlink(self.root + '/a_file')
+            fname = os.path.join(self.root, 'a_file')
+            f = open_file(fname, 'w')
+            f.write('hola')
+            remove_file(fname)
             f.close()
-            print >> file(self.root + '/b_file', 'w'), 'chau'
-            return d0
-        d = self.main.wait_for('SYS_LOCAL_RESCAN_DONE')
-        self.main.start()
-        d.addCallback(cb)
-        return d
 
-    def test_stomp_deleted_file_is_no_cause_for_despair_either(self):
-        """test_stomp_deleted_file_is_no_cause_for_despair_either."""
-        def cb(_):
-            d0 = self.main.wait_for('HQ_HASH_NEW')
-            f = file(self.root + '/a_file', 'w')
-            print >> f, 'hola'
-            os.unlink(self.root + '/a_file')
-            f.close()
-            print >> file(self.root + '/b_file', 'w'), 'chau'
+            fname = os.path.join(self.root, 'b_file')
+            f = open_file(fname, 'w')
+            f.write('chau')
             return d0
         d = self.main.wait_for('SYS_LOCAL_RESCAN_DONE')
         self.main.start()
@@ -490,6 +524,7 @@ class TestSync(BaseSync):
 
         # create the node
         somepath = os.path.join(self.root, 'somepath')
+        open_file(somepath, 'w').close()
         mdid = self.fsm.create(somepath, '', node_id='node_id')
 
         # send the event and check args after the ssmr instance
@@ -545,6 +580,7 @@ class TestSync(BaseSync):
 
         # create the node
         somepath = os.path.join(self.root, 'somepath')
+        make_dir(somepath)
         mdid = self.fsm.create(somepath, '', node_id='node_id', is_dir=True)
 
         # send the event and check args after the ssmr instance
@@ -640,7 +676,7 @@ class TestSync(BaseSync):
                                       'parent_id', 'name'])
         self.assertEqual(r.share_id, '')
         self.assertEqual(r.node_id, 'node_id')
-        self.assertEqual(r.path, 'somepath/name')
+        self.assertEqual(r.path, os.path.join('somepath', 'name'))
 
     def test_SV_FILE_NEW_node_no_id(self):
         """Handle SV_FILE_NEW having a node without node_id."""
@@ -688,17 +724,148 @@ class TestSync(BaseSync):
         # call and check
         self.sync._handle_SV_FILE_NEW('', 'node_id', 'parent_id', 'name')
         self.assertEqual(called, [childpath])
-        self.handler.debug  = True
         self.assertTrue(self.handler.check_debug("Wanted to apply SV_FILE_NEW",
                                                  "found it with other id"))
 
+    @defer.inlineCallbacks
+    def test_handle_FILE_CREATE_unsubscribed(self):
+        """The event is received in a volume that is no longer subscribed."""
+        called = []
+        self.patch(SyncStateMachineRunner, 'on_event',
+                   lambda *a: called.append(True))
+
+        # create the share and a path for the node
+        share = yield self.create_share('share', 'someshare')
+        somepath = os.path.join(share.path, 'somepath')
+
+        # send the event, and check that is not processed by SSMR
+        self.sync.handle_FS_FILE_CREATE(somepath)
+        self.assertFalse(called)
+        should_logged = ("FS_FILE_CREATE", "discarded",
+                         "volume not subscribed", repr(somepath))
+        self.assertTrue(self.handler.check_debug(*should_logged))
+
+    @defer.inlineCallbacks
+    def test_handle_DIR_CREATE_unsubscribed(self):
+        """The event is received in a volume that is no longer subscribed."""
+        called = []
+        self.patch(SyncStateMachineRunner, 'on_event',
+                   lambda *a: called.append(True))
+
+        # create the share and a path for the node
+        share = yield self.create_share('share', 'someshare')
+        somepath = os.path.join(share.path, 'somepath')
+
+        # send the event, and check that is not processed by SSMR
+        self.sync.handle_FS_DIR_CREATE(somepath)
+        self.assertFalse(called)
+        should_logged = ("FS_DIR_CREATE", "discarded",
+                         "volume not subscribed", repr(somepath))
+        self.assertTrue(self.handler.check_debug(*should_logged))
+
+    @defer.inlineCallbacks
+    def test_handle_FILE_DELETE_unsubscribed(self):
+        """The event is received in a volume that is no longer subscribed."""
+        called = []
+        self.patch(SyncStateMachineRunner, 'on_event',
+                   lambda *a: called.append(True))
+
+        # create the share and the node to be worked on
+        share = yield self.create_share('share', 'someshare')
+        somepath = os.path.join(share.path, 'somepath')
+        self.fsm.create(somepath, share.volume_id)
+
+        # send the event, and check that is not processed by SSMR
+        self.sync.handle_FS_FILE_DELETE(somepath)
+        self.assertFalse(called)
+        should_logged = ("FS_FILE_DELETE", "discarded",
+                         "volume not subscribed", repr(somepath))
+        self.assertTrue(self.handler.check_debug(*should_logged))
+
+    @defer.inlineCallbacks
+    def test_handle_DIR_DELETE_unsubscribed(self):
+        """The event is received in a volume that is no longer subscribed."""
+        called = []
+        self.patch(SyncStateMachineRunner, 'on_event',
+                   lambda *a: called.append(True))
+
+        # create the share and the node to be worked on
+        share = yield self.create_share('share', 'someshare')
+        somepath = os.path.join(share.path, 'somepath')
+        self.fsm.create(somepath, share.volume_id)
+
+        # send the event, and check that is not processed by SSMR
+        self.sync.handle_FS_DIR_DELETE(somepath)
+        self.assertFalse(called)
+        should_logged = ("FS_DIR_DELETE", "discarded",
+                         "volume not subscribed", repr(somepath))
+        self.assertTrue(self.handler.check_debug(*should_logged))
+
+    @defer.inlineCallbacks
+    def test_handle_FILE_MOVE_unsubscribed(self):
+        """The event is received in a volume that is no longer subscribed."""
+        called = []
+        self.patch(SyncStateMachineRunner, 'on_event',
+                   lambda *a: called.append(True))
+
+        # create the share and the node to be worked on
+        share = yield self.create_share('share', 'someshare')
+        somepath = os.path.join(share.path, 'somepath')
+        self.fsm.create(somepath, share.volume_id)
+
+        # send the event, and check that is not processed by SSMR
+        self.sync.handle_FS_FILE_MOVE(somepath, 'otherpath')
+        self.assertFalse(called)
+        should_logged = ("FS_FILE_MOVE", "discarded",
+                         "volume not subscribed", repr(somepath))
+        self.assertTrue(self.handler.check_debug(*should_logged))
+
+    @defer.inlineCallbacks
+    def test_handle_DIR_MOVE_unsubscribed(self):
+        """The event is received in a volume that is no longer subscribed."""
+        called = []
+        self.patch(SyncStateMachineRunner, 'on_event',
+                   lambda *a: called.append(True))
+
+        # create the share and the node to be worked on
+        share = yield self.create_share('share', 'someshare')
+        somepath = os.path.join(share.path, 'somepath')
+        self.fsm.create(somepath, share.volume_id)
+
+        # send the event, and check that is not processed by SSMR
+        self.sync.handle_FS_DIR_MOVE(somepath, 'otherpath')
+        self.assertFalse(called)
+        should_logged = ("FS_DIR_MOVE", "discarded",
+                         "volume not subscribed", repr(somepath))
+        self.assertTrue(self.handler.check_debug(*should_logged))
+
+    @defer.inlineCallbacks
+    def test_handle_FILE_CLOSE_WRITE_unsubscribed(self):
+        """The event is received in a volume that is no longer subscribed."""
+        called = []
+        self.patch(SyncStateMachineRunner, 'on_event',
+                   lambda *a: called.append(True))
+
+        # create the share and the node to be worked on
+        share = yield self.create_share('share', 'someshare')
+        somepath = os.path.join(share.path, 'somepath')
+        self.fsm.create(somepath, share.volume_id)
+
+        # send the event, and check that is not processed by SSMR
+        self.sync.handle_FS_FILE_CLOSE_WRITE(somepath)
+        self.assertFalse(called)
+        should_logged = ("FS_FILE_CLOSE_WRITE", "discarded",
+                         "volume not subscribed", repr(somepath))
+        self.assertTrue(self.handler.check_debug(*should_logged))
+
 
 class SyncStateMachineRunnerTestCase(BaseSync):
     """Tests for the SyncStateMachineRunner."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Init."""
-        BaseSync.setUp(self)
+        yield super(SyncStateMachineRunnerTestCase, self).setUp()
         self.fsm = self.main.fs
         self.aq = self.main.action_q
 
@@ -714,12 +881,7 @@ class SyncStateMachineRunnerTestCase(BaseSync):
         self.handler = MementoHandler()
         self.handler.setLevel(logging.DEBUG)
         self.ssmr.log.addHandler(self.handler)
-
-    def tearDown(self):
-        """Clean up."""
-        self.ssmr.log.removeHandler(self.handler)
-        self.ssmr = None
-        BaseSync.tearDown(self)
+        self.addCleanup(self.ssmr.log.removeHandler, self.handler)
 
     def test_delete_file_without_hash(self):
         """Delete_file can be called without the server hash."""
@@ -734,9 +896,9 @@ class SyncStateMachineRunnerTestCase(BaseSync):
         """This method should log detailed info."""
         # create testing data
         somepath = os.path.join(self.root, 'somepath')
-        open(somepath, "w").close()
-        oldstat = os.stat(somepath)
-        f = open(somepath, "w")
+        open_file(somepath, "w").close()
+        oldstat = stat_path(somepath)
+        f = open_file(somepath, "w")
         f.write("new")
         f.close()
 
@@ -823,7 +985,7 @@ class SyncStateMachineRunnerTestCase(BaseSync):
         # create the partial correctly, and break it!
         self.fsm.create_partial('node_id', '')
         partial_path = self.fsm._get_partial_path(self.fsm.fs[self.mdid])
-        os.remove(partial_path)
+        remove_file(partial_path)
 
         # event!
         self.ssmr.commit_file('AQ_DOWNLOAD_COMMIT', None, 'hash')
@@ -1224,9 +1386,10 @@ class FakedState(object):
 class StateMachineRunnerTestCase(BaseTwistedTestCase):
     """Test suite for StateMachineRunner."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Init."""
-        BaseTwistedTestCase.setUp(self)
+        yield super(StateMachineRunnerTestCase, self).setUp()
 
         logger = logging.getLogger('ubuntuone.SyncDaemon.sync')
         self.smr = fsm_module.StateMachineRunner(None, logger)
@@ -1259,9 +1422,10 @@ class StateMachineRunnerTestCase(BaseTwistedTestCase):
 class TestNewGenerationOnOperations(BaseSync):
     """Test we handle ok the new generation in some operations."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Set up."""
-        super(TestNewGenerationOnOperations, self).setUp()
+        yield super(TestNewGenerationOnOperations, self).setUp()
         self.sync = Sync(main=self.main)
         self.handler.setLevel(logging.DEBUG)
 
@@ -1420,9 +1584,10 @@ class TestNewGenerationOnOperations(BaseSync):
 class TestSyncDelta(BaseSync):
     """Base class for testing sync stuff related to deltas."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Do the setUp."""
-        super(TestSyncDelta, self).setUp()
+        yield super(TestSyncDelta, self).setUp()
         self.sync = Sync(main=self.main)
         self.root_id = root_id = "roootid"
         self.main.vm._got_root(root_id)
@@ -1469,8 +1634,12 @@ class TestSyncDelta(BaseSync):
 
 
 class TestHandleAqDeltaOk(TestSyncDelta):
-    """Sync.handle_AQ_DELTA_OK handles the recepcion of a new delta and applies
-    all the changes that came from it."""
+    """Test case for Sync.handle_AQ_DELTA_OK.
+
+    Assert that handles the recepcion of a new delta and applies all the
+    changes that came from it.
+
+    """
 
     def test_not_full(self):
         """If we dont have a full delta, we need to ask for another one."""
@@ -1820,9 +1989,10 @@ class TestHandleAqDeltaOk(TestSyncDelta):
 class TestHandleAqRescanFromScratchOk(TestSyncDelta):
     """Sync.handle_AQ_RESCAN_FROM_SCRATCH_OK handles rescan from scratch."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Setup."""
-        super(TestHandleAqRescanFromScratchOk, self).setUp()
+        yield super(TestHandleAqRescanFromScratchOk, self).setUp()
 
         self.rootdt = delta.FileInfoDelta(
             generation=5, is_live=True, file_type=delta.DIRECTORY,
@@ -1911,9 +2081,10 @@ class TestHandleAqRescanFromScratchOk(TestSyncDelta):
 class TestSyncEvents(TestSyncDelta):
     """Testing sync stuff related to events."""
 
+    @defer.inlineCallbacks
     def setUp(self):
         """Do the setUp."""
-        super(TestSyncEvents, self).setUp()
+        yield super(TestSyncEvents, self).setUp()
         self.sync = Sync(main=self.main)
         self.handler.setLevel(logging.DEBUG)
 
