@@ -140,6 +140,16 @@ class DataFileTest(BaseTestCase):
         new_file.fd.flush()
         self.assertEqual(len('foo'), new_file.size)
 
+    def test_has_hint(self):
+        """Test that has_hint works as expetced."""
+        new_file = self.file_class(self.base_dir)
+        self.assertFalse(new_file.has_hint)
+
+    def test_hint_size(self):
+        """Test that hint_size work as expected."""
+        new_file = self.file_class(self.base_dir)
+        self.assertEqual(new_file.hint_size, 0)
+
     def test__open(self):
         """Test the _open private method."""
         new_file = self.file_class(self.base_dir)
@@ -523,6 +533,26 @@ class ImmutableDataFileTest(DataFileTest):
         self.assertEqual(len('bar')+len('foo')+header_size+crc32_size,
                          data_file.size)
 
+    def test_has_hint(self):
+        """Test that has_hint works as expetced."""
+        rw_file = DataFile(self.base_dir)
+        tstamp, value_pos, value_sz = rw_file.write(0, 'foo', 'bar')
+        data_file = rw_file.make_immutable()
+        self.assertFalse(data_file.has_hint)
+        data_file.get_hint_file().close()
+        self.assertTrue(data_file.has_hint)
+
+    def test_hint_size(self):
+        """Test that hint_size work as expected."""
+        rw_file = DataFile(self.base_dir)
+        tstamp, value_pos, value_sz = rw_file.write(0, 'foo', 'bar')
+        data_file = rw_file.make_immutable()
+        self.assertEqual(data_file.hint_size, 0)
+        hint_file = data_file.get_hint_file()
+        hint_file.fd.write("some data")
+        hint_file.close()
+        self.assertEqual(data_file.hint_size, len("some data"))
+
 
 class DeadDataFileTest(ImmutableDataFileTest):
     """Tests for DeadDataFile."""
@@ -617,12 +647,21 @@ class HintFileTest(BaseTestCase):
         self.assertEqual('w+b', hint_file.fd.mode)
 
     def test_init_existing(self):
-        """Test initialization."""
+        """Test initialization with existing file."""
+        path = os.path.join(self.base_dir, 'test_hint_existing')
+        hint_file = HintFile(path)
+        hint_file.fd.write("some data")
+        hint_file.close()
+        hint_file = HintFile(path)
+        self.assertEqual('rb', hint_file.fd.mode)
+
+    def test_init_existing_empty(self):
+        """Test initialization with existing file."""
         path = os.path.join(self.base_dir, 'test_hint_existing')
         hint_file = HintFile(path)
         hint_file.close()
         hint_file = HintFile(path)
-        self.assertEqual('rb', hint_file.fd.mode)
+        self.assertEqual('w+b', hint_file.fd.mode)
 
     def test_close(self):
         """Test for the close method."""
@@ -1032,6 +1071,27 @@ class InitTest(BaseTestCase):
             self.assertEqual(6, len(hint_entry))
             self.assertTrue(db.get(hint_entry[2], hint_entry[-1]) is not None)
 
+    def test_build_keydir_with_empty_hint(self):
+        """Test _build_keydir using a hint file."""
+        db = Tritcask(self.base_dir)
+        # create some stuff.
+        for i in range(10):
+            db.put(i, 'foo%d' % (i,), 'bar%s' % (i,))
+        # make the data file inactive and generate the hint.
+        db.live_file.make_immutable()
+        old_keydir = db._keydir
+        db.shutdown()
+        db = Tritcask(self.base_dir)
+        hint_filename = db._immutable.values()[0].hint_filename
+        # truncate the hint file
+        open(hint_filename, 'w').close()
+        db.shutdown()
+        # open the tritcask and make sure it regenerates the hint file
+        db = Tritcask(self.base_dir)
+        self.addCleanup(db.shutdown)
+        self.assertTrue(os.path.exists(hint_filename), os.listdir(db.base_path))
+        self.assertEqual(old_keydir, db._keydir)
+
     def test_build_keydir(self):
         """Test _build_keydir method."""
         db = Tritcask(self.base_dir)
