@@ -35,6 +35,3 @@ class TestParse(unittest.TestCase):
         'test parsing a simple machine'
         f = fsm.StateMachine(p("../u1fsfsm.py"))
         f.validate()
-
-if __name__ == "__main__":
-    unittest.main()
