@@ -25,7 +25,7 @@ def is_zeitgeist_installed():
         # use the above module in some way so pylint does not complain
         assert(zeitgeist is not None)
         return True
-    except ImportError:
+    except (ImportError, AttributeError):
         return False
 
 def get_listener(fsm, vm):
