@@ -1,8 +1,9 @@
 # ubuntuone.syncdaemon.event_queue - Event queuing
 #
-# Author: Facundo Batista <facundo@canonical.com>
+# Authors: Facundo Batista <facundo@canonical.com>
+#          Manuel de la Pena <manuel@canonical.com>
 #
-# Copyright 2009 Canonical Ltd.
+# Copyright 2009-2011 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 3, as published
@@ -15,6 +16,7 @@
 #
 # You should have received a copy of the GNU General Public License along
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 """Module that implements the Filesystem Monitor machinery."""
 
 import logging
@@ -22,10 +24,8 @@ import os
 
 import pyinotify
 from twisted.internet import abstract, reactor, error, defer
+from ubuntuone.platform.linux.os_helper import access
 
-from ubuntuone.syncdaemon.filesystem_notifications import (
-    GeneralINotifyProcessor
-)
 
 # translates quickly the event and it's is_dir state to our standard events
 NAME_TRANSLATIONS = {
@@ -151,6 +151,10 @@ class _GeneralINotifyProcessor(pyinotify.ProcessEvent):
     FS_(DIR|FILE)_MOVE event when possible.
     """
     def __init__(self, monitor, ignore_config=None):
+        # XXX: avoid circular imports
+        from ubuntuone.syncdaemon.filesystem_notifications import (
+            GeneralINotifyProcessor
+        )
         self.general_processor = GeneralINotifyProcessor(monitor,
             self.handle_dir_delete, NAME_TRANSLATIONS,
             self.platform_is_ignored, pyinotify.IN_IGNORED,
@@ -158,6 +162,11 @@ class _GeneralINotifyProcessor(pyinotify.ProcessEvent):
         self.held_event = None
         self.timer = None
 
+    def shutdown(self):
+        """Shut down the processor."""
+        if self.timer is not None and self.timer.active():
+            self.timer.cancel()
+
     def rm_from_mute_filter(self, event, paths):
         """Remove an event and path(s) from the mute filter."""
         self.general_processor.rm_from_mute_filter(event, paths)
@@ -422,6 +431,7 @@ class FilesystemMonitor(object):
         self._inotify_notifier_antr.stop()
         reactor.removeReader(self._inotify_reader_gral)
         reactor.removeReader(self._inotify_reader_antr)
+        self._processor.shutdown()
 
     def rm_watch(self, dirpath):
         """Remove watch from a dir."""
@@ -453,15 +463,42 @@ class FilesystemMonitor(object):
             w_dict = self._general_watchs
             events = INOTIFY_EVENTS_GENERAL
 
+        # not add it if already there
+        if dirpath in w_dict:
+            self.log.debug("Watch already there for %r", dirpath)
+            return defer.succeed(False)
+
         # add the watch!
         self.log.debug("Adding %s inotify watch to %r", w_type, dirpath)
         result = w_manager.add_watch(dirpath, events)
         w_dict[dirpath] = result[dirpath]
+        return defer.succeed(True)
 
-    def has_watch(self, dirpath):
-        """Check if a dirpath is watched."""
-        return (dirpath in self._general_watchs or
-                dirpath in self._ancestors_watchs)
+    @defer.inlineCallbacks
+    def add_watches_to_udf_ancestors(self, volume):
+        """Add a inotify watch to volume's ancestors if it's an UDF."""
+        added_watches = []
+
+        def revert_watches():
+            """Restore the just added watches and unsubscribe volume."""
+            for path in added_watches:
+                self.eq.rm_watch(path)
+
+        for ancestor in volume.ancestors:
+            # check that ancestor is still there
+            if not access(ancestor):
+                self.log.info("Tree broken at path: %r", volume.path)
+                revert_watches()
+                defer.returnValue(False)
+
+            self.log.debug("Adding watch to UDF's ancestor %r", ancestor)
+            really_added = yield self.eq.add_watch(ancestor)
+            # only note it for the revert if the watch was not there before
+            if really_added:
+                added_watches.append(ancestor)
+
+        # all is ok
+        defer.returnValue(True)
 
     def inotify_watch_fix(self, pathfrom, pathto):
         """Fix the path in inotify structures."""
