@@ -15,12 +15,15 @@
 #
 # You should have received a copy of the GNU General Public License along
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
-""" SyncDaemon Tools """
-import dbus
+
+"""SyncDaemon Tools."""
+
 import logging
 import time
 import sys
 
+import dbus
+
 from ubuntuone.platform.linux.dbus_interface import (
     DBUS_IFACE_NAME,
     DBUS_IFACE_STATUS_NAME,
@@ -39,29 +42,32 @@ from twisted.python.failure import Failure
 
 
 def is_running(bus=None):
-    """Check if there is a syncdaemon instance running or if
-    the name is registered in the bus.
+    """Check if there is a syncdaemon instance running.
+
+    Running means the name is registered in the given bus.
+
     """
     if bus is None:
         bus = dbus.SessionBus()
     return DBUS_IFACE_NAME in bus.list_names()
 
+
 class ErrorSignal(Exception):
     pass
 
 
 class DBusClient(object):
-    """ Low level dbus client. To help testing the DBus interface. """
+    """Low level dbus client. To help testing the DBus interface."""
 
     def __init__(self, bus, path, interface, destination=DBUS_IFACE_NAME):
-        """ create the instance """
+        """Create the instance."""
         self.bus = bus
         self.path = path
         self.interface = interface
         self.destination = destination
 
     def send_signal(self, signal, *args):
-        """ Send method with *args """
+        """Send method with *args."""
         msg = SignalMessage(self.path, self.interface,
                             signal)
         msg.set_no_reply(True)
@@ -69,7 +75,7 @@ class DBusClient(object):
         self.bus.send_message(msg)
 
     def call_method(self, method, *args, **kwargs):
-        """ Call method with *args and **kwargs over dbus"""
+        """Call method with *args and **kwargs over dbus."""
         msg = MethodCallMessage(self.destination, self.path, self.interface,
                                 method)
         msg.set_no_reply(True)
@@ -85,7 +91,7 @@ class DBusClient(object):
         assert error_handler != None
 
         def parse_reply(message):
-            """ handle the reply message"""
+            """Handle the reply message."""
             if isinstance(message, ErrorMessage):
                 return error_handler(DBusException(
                                     name=message.get_error_name(),
@@ -104,27 +110,29 @@ class DBusClient(object):
 
 
 class SyncDaemonTool(object):
-    """ Various utility methods to test/play with the SyncDaemon. """
+    """Various utility methods to test/play with the SyncDaemon."""
 
-    def __init__(self, bus):
+    def __init__(self, bus=None):
+        if bus is None:
+            bus = dbus.SessionBus()
         self.bus = bus
         self.last_event = 0
         self.delayed_call = None
         self.log = logging.getLogger('ubuntuone.SyncDaemon.SDTool')
 
     def _get_dict(self, a_dict):
-        """ Converts a dict returned by dbus to a dict of strings. """
+        """Converts a dict returned by dbus to a dict of strings."""
         str_dict = {}
         for key in a_dict:
             str_dict[key] = unicode(a_dict[key])
         return str_dict
 
     def wait_connected(self):
-        """ Wait until syncdaemon is connected to the server. """
+        """Wait until syncdaemon is connected to the server."""
         self.log.debug('wait_connected')
         d = defer.Deferred()
         def check_connection_status():
-            """ Check if the daemon is up and running. """
+            """Check if the daemon is up and running."""
             # check if the syncdaemon is running
             # catch all errors, pylint: disable-msg=W0703
             try:
@@ -140,12 +148,10 @@ class SyncDaemonTool(object):
         return d
 
     def get_current_downloads(self):
-        """ Return a deferred that 'll be called with the list
-        of current downloads
-        """
+        """Return a deferred that will be fired with the current downloads."""
         d = defer.Deferred()
         def current_downloads():
-            """ Call Status.current_downloads """
+            """Call Status.current_downloads."""
             status_client = DBusClient(self.bus, '/status',
                                        DBUS_IFACE_STATUS_NAME)
             status_client.call_method('current_downloads',
@@ -153,7 +159,7 @@ class SyncDaemonTool(object):
                                       error_handler=d.errback)
 
         def reply_handler(downloads):
-            """ current downloads callback """
+            """Current downloads callback."""
             downloads_str = []
             for download in downloads:
                 downloads_str.append(self._get_dict(download))
@@ -163,12 +169,14 @@ class SyncDaemonTool(object):
         return d
 
     def wait_all_downloads(self, verbose=False):
-        """ Wait until there is no more pending downloads """
+        """Wait until there is no more pending downloads."""
         self.log.debug('wait_all_downloads')
         d = self.get_current_downloads()
         def reply_handler(downloads):
-            """ Check if the are downloads in progress, and reschelude a
-            new check if there is at least one.
+            """Check if the are downloads in progress.
+
+            If so, reschelude a new check if there is at least one.
+
             """
             if verbose:
                 sys.stdout.write(', %s' % str(len(downloads)))
@@ -187,12 +195,10 @@ class SyncDaemonTool(object):
         return d
 
     def get_current_uploads(self):
-        """ Return a deferred that 'll be called with the list
-        of current uploads
-        """
+        """Return a deferred that will be called with the current uploads."""
         d = defer.Deferred()
         def current_uploads():
-            """ Call Status.current_uploads """
+            """Call Status.current_uploads."""
             status_client = DBusClient(self.bus, '/status',
                                        DBUS_IFACE_STATUS_NAME)
             status_client.call_method('current_uploads',
@@ -200,7 +206,7 @@ class SyncDaemonTool(object):
                                       error_handler=d.errback)
 
         def reply_handler(uploads):
-            """ reply handler """
+            """Reply handler."""
             uploads_str = []
             for upload in uploads:
                 uploads_str.append(self._get_dict(upload))
@@ -210,13 +216,15 @@ class SyncDaemonTool(object):
         return d
 
     def wait_all_uploads(self, verbose=False):
-        """ Wait until there is no more pending uploads """
+        """Wait until there is no more pending uploads."""
         self.log.debug('wait_all_uploads')
         d = self.get_current_uploads()
 
         def reply_handler(uploads):
-            """ Check if the are downloads in progress, and reschelude a
-            new check if there is at least one.
+            """Check if the are downloads in progress.
+
+            If so, reschelude a new check if there is at least one.
+
             """
             if verbose:
                 sys.stdout.write(', %s' % str(len(uploads)))
@@ -289,7 +297,9 @@ class SyncDaemonTool(object):
         return d
 
     def wait_for_nirvana(self, last_event_interval=5, verbose=False):
-        """ Wait until the syncdaemon reachs nirvana. This is when there are:
+        """Wait until the syncdaemon reachs nirvana.
+
+        This is when there are:
             - the syncdaemon is connected
             - 0 transfers inprogress
             - no more events are fired in the event queue
@@ -305,7 +315,7 @@ class SyncDaemonTool(object):
         return d
 
     def accept_share(self, share_id):
-        """ Accept the share with id: share_id. """
+        """Accept the share with id: share_id."""
         self.log.debug('accept_share(%s)', share_id)
         shares_client = DBusClient(self.bus, '/shares', DBUS_IFACE_SHARES_NAME)
         d = self.wait_for_signal('ShareAnswerResponse',
@@ -316,7 +326,7 @@ class SyncDaemonTool(object):
         return d
 
     def reject_share(self, share_id):
-        """ Reject the share with id: share_id. """
+        """Reject the share with id: share_id."""
         self.log.debug('reject_share(%s)', share_id)
         shares_client = DBusClient(self.bus, '/shares', DBUS_IFACE_SHARES_NAME)
         d = self.wait_for_signal('ShareAnswerResponse',
@@ -359,12 +369,12 @@ class SyncDaemonTool(object):
         yield d
 
     def get_shares(self):
-        """ Get the list of shares (accepted or not) """
+        """Get the list of shares (accepted or not)."""
         self.log.debug('get_shares')
         shares_client = DBusClient(self.bus, '/shares', DBUS_IFACE_SHARES_NAME)
         d = defer.Deferred()
         def reply_handler(results):
-            """ get_shares reply handler. """
+            """Get_shares reply handler."""
             shares = []
             for result in results:
                 shares.append(self._get_dict(result))
@@ -377,8 +387,10 @@ class SyncDaemonTool(object):
         return d
 
     def refresh_shares(self):
-        """ Call refresh_shares method via DBus.
+        """Call refresh_shares method via DBus.
+
         Request a refresh of share list to the server.
+
         """
         self.log.debug('refresh_shares')
         shares_client = DBusClient(self.bus, '/shares', DBUS_IFACE_SHARES_NAME)
@@ -389,7 +401,7 @@ class SyncDaemonTool(object):
         return d
 
     def offer_share(self, path, username, name, access_level):
-        """ Offer a share at the specified path to user with id: username. """
+        """Offer a share at the specified path to user with id: username."""
         self.log.debug('offer_share(%s, %s, %s, %s)',
                    path, username, name, access_level)
         shares_client = DBusClient(self.bus, '/shares', DBUS_IFACE_SHARES_NAME)
@@ -401,12 +413,12 @@ class SyncDaemonTool(object):
         return d
 
     def list_shared(self):
-        """ get the list of the shares "shared"/created/offered. """
+        """Get the list of the shares "shared"/created/offered."""
         self.log.debug('list_shared')
         shares_client = DBusClient(self.bus, '/shares', DBUS_IFACE_SHARES_NAME)
         d = defer.Deferred()
         def reply_handler(results):
-            """ get_shares reply handler. """
+            """Get_shares reply handler."""
             shares = []
             for result in results:
                 shares.append(self._get_dict(result))
@@ -419,8 +431,7 @@ class SyncDaemonTool(object):
 
     def wait_for_signals(self, signal_ok, signal_error,
                          dbus_iface=DBUS_IFACE_FOLDERS_NAME):
-        """wait for one of the specified DBus signals
-        this returns a deferred
+        """Wait for one of the specified DBus signals, return a deferred.
 
         @param signal_ok: this will fire the deferred's callback
         @param signal_error: the will fire the deferred's errback
@@ -450,19 +461,19 @@ class SyncDaemonTool(object):
     def create_folder(self, path):
         """Create a user defined folder in the specified path."""
         self.log.debug('create_folder')
-        folders_client = DBusClient(self.bus, '/folders', DBUS_IFACE_FOLDERS_NAME)
-
+        folders_client = DBusClient(self.bus, '/folders',
+                                    DBUS_IFACE_FOLDERS_NAME)
         d = self.wait_for_signals('FolderCreated', 'FolderCreateError')
         folders_client.call_method('create', path,
-                                  reply_handler=lambda _: None,
-                                  error_handler=d.errback)
+                                   reply_handler=lambda _: None,
+                                   error_handler=d.errback)
         return d
 
     def delete_folder(self, folder_id):
         """Delete a user defined folder given its id."""
         self.log.debug('delete_folder')
-        folders_client = DBusClient(self.bus, '/folders', DBUS_IFACE_FOLDERS_NAME)
-
+        folders_client = DBusClient(self.bus, '/folders',
+                                    DBUS_IFACE_FOLDERS_NAME)
         d = self.wait_for_signals('FolderDeleted', 'FolderDeleteError')
         folders_client.call_method('delete', folder_id,
                                   reply_handler=lambda _: None,
@@ -472,8 +483,8 @@ class SyncDaemonTool(object):
     def subscribe_folder(self, folder_id):
         """Subscribe to a user defined folder given its id."""
         self.log.debug('subscribe_folder')
-        folders_client = DBusClient(self.bus, '/folders', DBUS_IFACE_FOLDERS_NAME)
-
+        folders_client = DBusClient(self.bus, '/folders',
+                                    DBUS_IFACE_FOLDERS_NAME)
         d = self.wait_for_signals('FolderSubscribed', 'FolderSubscribeError')
         folders_client.call_method('subscribe', folder_id,
                                   reply_handler=lambda _: None,
@@ -483,9 +494,10 @@ class SyncDaemonTool(object):
     def unsubscribe_folder(self, folder_id):
         """Unsubscribe from a user defined folder given its id."""
         self.log.debug('unsubscribe_folder')
-        folders_client = DBusClient(self.bus, '/folders', DBUS_IFACE_FOLDERS_NAME)
-
-        d = self.wait_for_signals('FolderUnSubscribed', 'FolderUnSubscribeError')
+        folders_client = DBusClient(self.bus, '/folders',
+                                    DBUS_IFACE_FOLDERS_NAME)
+        d = self.wait_for_signals('FolderUnSubscribed',
+                                  'FolderUnSubscribeError')
         folders_client.call_method('unsubscribe', folder_id,
                                   reply_handler=lambda _: None,
                                   error_handler=d.errback)
@@ -494,18 +506,19 @@ class SyncDaemonTool(object):
     def get_folders(self):
         """Return the list of folders (a list of dicts)."""
         self.log.debug('get_folders')
-        folders_client = DBusClient(self.bus, '/folders', DBUS_IFACE_FOLDERS_NAME)
+        folders_client = DBusClient(self.bus, '/folders',
+                                    DBUS_IFACE_FOLDERS_NAME)
         d = defer.Deferred()
         def reply_handler(results):
-            """ get_folders reply handler. """
+            """Get_folders reply handler."""
             folders = []
             for result in results:
                 folders.append(self._get_dict(result))
             self.log.debug('folders: %r', folders)
             d.callback(folders)
         folders_client.call_method('get_folders',
-                                  reply_handler=reply_handler,
-                                  error_handler=d.errback)
+                                   reply_handler=reply_handler,
+                                   error_handler=d.errback)
         return d
 
     def get_folder_info(self, path):
@@ -519,7 +532,7 @@ class SyncDaemonTool(object):
         return d
 
     def get_metadata(self, path):
-        """ calls the exposed mtehod FileSystem.get_metadata using DBus. """
+        """Call the exposed mtehod FileSystem.get_metadata using DBus."""
         self.log.debug('get_metadata(%s)', path)
         fs_client = DBusClient(self.bus, '/filesystem', DBUS_IFACE_FS_NAME)
         d = defer.Deferred()
@@ -552,7 +565,7 @@ class SyncDaemonTool(object):
         defer.returnValue(file_info)
 
     def quit(self):
-        """quit the syncdaemon"""
+        """Quit the syncdaemon."""
         self.log.debug('quit')
         # avoid triggering dbus activation while calling quit
         if not is_running(self.bus):
@@ -563,7 +576,7 @@ class SyncDaemonTool(object):
                            reply_handler=d.callback,
                            error_handler=d.errback)
         def check(r):
-            """wait 0.5 sec to return, to allow syncdaemon to shutdown"""
+            """Wait 0.5 sec to return, to allow syncdaemon to shutdown."""
             d1 = defer.Deferred()
             reactor.callLater(0.5, d1.callback, r)
             return d1
@@ -571,15 +584,16 @@ class SyncDaemonTool(object):
         return d
 
     def wait_for_signal(self, signal_name, filter):
-        """wait for the specified DBus signal (the first received).
-        'filter' is used to fire the deferred callback.
+        """Wait for the specified DBus signal (the first received).
 
         @param signal_name: the signal name
-        @param filter: a callable to filter signal, must return True
+        @param filter: a callable to filter signal, must return True, and is
+        used to fire the deferred callback.
+
         """
         d = defer.Deferred()
         def signal_handler(result):
-            """handle the signals and fires the call/errback"""
+            """Handle the signals and fires the call/errback"""
             try:
                 if filter(result) and not d.called:
                     d.callback(result)
@@ -590,14 +604,14 @@ class SyncDaemonTool(object):
         match = self.bus.add_signal_receiver(signal_handler,
                                              signal_name=signal_name)
         def cleanup(result):
-            """remove the signal receiver from the bus."""
+            """Remove the signal receiver from the bus."""
             self.bus.remove_signal_receiver(match)
             return result
         d.addCallback(cleanup)
         return d
 
     def connect(self):
-        """Connect syncdaemon"""
+        """Connect syncdaemon."""
         sd_client = DBusClient(self.bus, '/', DBUS_IFACE_SYNC_NAME)
         d = defer.Deferred()
         sd_client.call_method('connect',
@@ -606,7 +620,7 @@ class SyncDaemonTool(object):
         return d
 
     def disconnect(self):
-        """Disconnect syncdaemon"""
+        """Disconnect syncdaemon."""
         sd_client = DBusClient(self.bus, '/', DBUS_IFACE_SYNC_NAME)
         d = defer.Deferred()
         sd_client.call_method('disconnect',
@@ -615,7 +629,7 @@ class SyncDaemonTool(object):
         return d
 
     def get_status(self):
-        """Get the current_status dict"""
+        """Get the current_status dict."""
         d = defer.Deferred()
         status_client = DBusClient(self.bus, '/status',
                                    DBUS_IFACE_STATUS_NAME)
@@ -631,6 +645,16 @@ class SyncDaemonTool(object):
                                   error_handler=d.errback)
         return d
 
+    def free_space(self, vol_id):
+        """Return the free space of the given volume."""
+        d = defer.Deferred()
+        status_client = DBusClient(self.bus, '/status',
+                                   DBUS_IFACE_STATUS_NAME)
+        status_client.call_method('free_space', vol_id,
+                                  reply_handler=d.callback,
+                                  error_handler=d.errback)
+        return d
+
     def waiting(self):
         """Return a description of the waiting queue elements."""
         d = defer.Deferred()
@@ -652,7 +676,7 @@ class SyncDaemonTool(object):
         return d
 
     def waiting_content(self):
-        """Returns the waiting content queue elements."""
+        """Return the waiting content queue elements."""
         d = defer.Deferred()
         status_client = DBusClient(self.bus, '/status',
                                    DBUS_IFACE_STATUS_NAME)
@@ -662,9 +686,7 @@ class SyncDaemonTool(object):
         return d
 
     def start(self):
-        """Start syncdaemon using the StartServiceByName method
-        if it's not running.
-        """
+        """Start syncdaemon if it's not running."""
         if not is_running(self.bus):
             wait_d = self.wait_for_signal('StatusChanged', lambda x: x)
             d = defer.Deferred()
@@ -702,30 +724,40 @@ class SyncDaemonTool(object):
                                   error_handler=d.errback)
         return d
 
-    def is_throttling_enabled(self):
-        """Check if throttling is enabled."""
+
+    def _is_setting_enabled(self, setting_name):
+        """Return whether 'setting_name' is enabled."""
         d = defer.Deferred()
         config_client = DBusClient(self.bus, '/config',
                                    DBUS_IFACE_CONFIG_NAME)
-        config_client.call_method('bandwidth_throttling_enabled',
+        config_client.call_method('%s_enabled' % setting_name,
                                   reply_handler=d.callback,
                                   error_handler=d.errback)
         return d
 
+    def _enable_setting(self, setting_name, enabled, dbus_method=None):
+        """Enable/disable 'setting_name'."""
+        d = defer.Deferred()
+        config_client = DBusClient(self.bus, '/config',
+                                   DBUS_IFACE_CONFIG_NAME)
+        if dbus_method is not None:
+            meth = dbus_method
+        elif enabled:
+            meth = 'enable_%s'
+        else:
+            meth = 'disable_%s'
+        config_client.call_method(meth % setting_name,
+                                  reply_handler=d.callback,
+                                  error_handler=d.errback)
+        return d
+
+    def is_throttling_enabled(self):
+        """Check if throttling is enabled."""
+        return self._is_setting_enabled('bandwidth_throttling')
 
     def enable_throttling(self, enabled):
         """Enable/disable throttling."""
-        d = defer.Deferred()
-        config_client = DBusClient(self.bus, '/config',
-                                   DBUS_IFACE_CONFIG_NAME)
-        if enabled:
-            meth = 'enable_bandwidth_throttling'
-        else:
-            meth = 'disable_bandwidth_throttling'
-        config_client.call_method(meth,
-                                  reply_handler=d.callback,
-                                  error_handler=d.errback)
-        return d
+        return self._enable_setting('bandwidth_throttling', enabled)
 
     def is_files_sync_enabled(self):
         """Check if files sync is enabled."""
@@ -758,6 +790,38 @@ class SyncDaemonTool(object):
                 config.save()
                 self.start()
 
+    def is_autoconnect_enabled(self):
+        """Check if autoconnect is enabled."""
+        return self._is_setting_enabled('autoconnect')
+
+    def enable_autoconnect(self, enabled):
+        """Enable/disable autoconnect."""
+        return self._enable_setting('autoconnect', enabled)
+
+    def is_show_all_notifications_enabled(self):
+        """Check if show_all_notifications is enabled."""
+        return self._is_setting_enabled('show_all_notifications')
+
+    def enable_show_all_notifications(self, enabled):
+        """Enable/disable show_all_notifications."""
+        return self._enable_setting('show_all_notifications', enabled)
+
+    def is_share_autosubscribe_enabled(self):
+        """Check if share_autosubscribe is enabled."""
+        return self._is_setting_enabled('share_autosubscribe')
+
+    def enable_share_autosubscribe(self, enabled):
+        """Enable/disable share_autosubscribe."""
+        return self._enable_setting('share_autosubscribe', enabled)
+
+    def is_udf_autosubscribe_enabled(self):
+        """Check if udf_autosubscribe is enabled."""
+        return self._is_setting_enabled('udf_autosubscribe')
+
+    def enable_udf_autosubscribe(self, enabled):
+        """Enable/disable udf_autosubscribe."""
+        return self._enable_setting('udf_autosubscribe', enabled)
+
     def refresh_volumes(self):
         """Call refresh_volumes method via DBus.
 
@@ -797,11 +861,38 @@ class SyncDaemonTool(object):
                                   error_handler=d.errback)
         return d
 
+    def get_root_dir(self):
+        """Return the root directory."""
+        d = defer.Deferred()
+        client = DBusClient(self.bus, '/', DBUS_IFACE_SYNC_NAME)
+        client.call_method('get_rootdir',
+                           reply_handler=d.callback,
+                           error_handler=d.errback)
+        return d
+
+    def get_shares_dir(self):
+        """Return the shares directory."""
+        d = defer.Deferred()
+        client = DBusClient(self.bus, '/', DBUS_IFACE_SYNC_NAME)
+        client.call_method('get_sharesdir',
+                           reply_handler=d.callback,
+                           error_handler=d.errback)
+        return d
+
+    def get_shares_dir_link(self):
+        """Return the shares link directory."""
+        d = defer.Deferred()
+        client = DBusClient(self.bus, '/', DBUS_IFACE_SYNC_NAME)
+        client.call_method('get_sharesdir_link',
+                           reply_handler=d.callback,
+                           error_handler=d.errback)
+        return d
+
 
 # callbacks used by u1sdtool script
 
 def show_shared(shares, out):
-    """ Callback that prints the list of shared shares"""
+    """Callback that prints the list of shared shares."""
     if len(shares) == 0:
         out.write("No shared\n")
     else:
@@ -816,7 +907,7 @@ def show_shared(shares, out):
 
 
 def show_folders(folders, out):
-    """ Callback that prints the list of user defined folders"""
+    """Callback that prints the list of user defined folders."""
     if len(folders) == 0:
         out.write("No folders\n")
     else:
@@ -840,7 +931,7 @@ def show_error(error, out):
 
 
 def show_shares(shares, out):
-    """ Callback that print the list of shares"""
+    """Callback that prints the list of shares."""
     if len(shares) == 0:
         out.write("No shares\n")
     else:
@@ -854,7 +945,7 @@ def show_shares(shares, out):
 
 
 def show_path_info(result, path, out):
-    """ print the path info to stdout"""
+    """Print the path info to stdout."""
     out_encoding = out.encoding
     if out_encoding is None:
         out_encoding = 'utf-8'
@@ -866,7 +957,7 @@ def show_path_info(result, path, out):
 
 
 def show_uploads(uploads, out):
-    """ print the uploads to stdout"""
+    """Print the uploads to stdout."""
     if uploads:
         out.write("Current uploads:\n")
     else:
@@ -879,7 +970,7 @@ def show_uploads(uploads, out):
 
 
 def show_downloads(downloads, out):
-    """ print the downloas to stdout"""
+    """Print the downloads to stdout."""
     if downloads:
         out.write("Current downloads:\n")
     else:
@@ -898,6 +989,9 @@ def show_state(state_dict, out):
         out.write("    %s: %s\n" % (k, v))
     out.write("\n")
 
+def show_free_space(free_space, out):
+    """Print the free_space result."""
+    out.write("Free space: %d bytes\n" % (free_space,))
 
 def show_waiting(waiting_ops, out):
     """Print the waiting result.
@@ -906,7 +1000,7 @@ def show_waiting(waiting_ops, out):
     name, if it's running or not, the share_id, then the node_id, then the
     path, and the rest in alphabetical order.
     """
-    for op_name, op_data in waiting_ops:
+    for op_name, op_id, op_data in waiting_ops:
         # running
         attributes = []
         running = op_data.pop('running', None)
@@ -934,11 +1028,11 @@ def show_waiting_metadata(waiting_ops, out):
     alphabetical order.
     """
     out.write("Warning: this option is deprecated! Use '--waiting' instead\n")
-    return show_waiting(waiting_ops, out)
+    return show_waiting(((x[0], None, x[1]) for x in waiting_ops), out)
 
 
 def show_waiting_content(waiting_ops, out):
-    """Print the waiting_content result"""
+    """Print the waiting_content result."""
     out.write("Warning: this option is deprecated! Use '--waiting' instead\n")
     value_tpl = "operation='%(operation)s' node_id='%(node)s' " + \
             "share_id='%(share)s' path='%(path)s'"
