@@ -647,12 +647,14 @@ class StatusAggregator(object):
         lines = []
         files_uploading = len(self.files_uploading)
         if files_uploading > 0:
+            self.uploading_filename = os.path.basename(
+                self.files_uploading[0].path)
             lines.append(files_being_uploaded(
                 self.uploading_filename, files_uploading))
         files_downloading = len(self.files_downloading)
         if files_downloading > 0:
-            self.downloading_filename = self.files_downloading[0].path.split(
-                os.path.sep)[-1]
+            self.downloading_filename = os.path.basename(
+                self.files_downloading[0].path)
             lines.append(files_being_downloaded(
                 self.downloading_filename, files_downloading))
         return "\n".join(lines)
@@ -713,8 +715,8 @@ class StatusAggregator(object):
                 (command.share_id, command.node_id)] = command.deflated_size
         # pylint: disable=W0201
         if not self.downloading_filename:
-            self.downloading_filename = self.files_downloading[0].path.split(
-                os.path.sep)[-1]
+            self.downloading_filename = os.path.basename(
+                self.files_downloading[0].path)
         # pylint: enable=W0201
         self.update_progressbar()
         logger.debug(
@@ -744,8 +746,8 @@ class StatusAggregator(object):
                 (command.share_id, command.node_id)] = command.deflated_size
         # pylint: disable=W0201
         if not self.uploading_filename:
-            self.uploading_filename = self.files_uploading[0].path.split(
-                os.path.sep)[-1]
+            self.uploading_filename = os.path.basename(
+                self.files_uploading[0].path)
         # pylint: enable=W0201
         self.update_progressbar()
         logger.debug(
