@@ -13,7 +13,7 @@
 #
 # You should have received a copy of the GNU General Public License along
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
-""" client module """
+"""Client module."""
 
 # required capabilities
 REQUIRED_CAPS = frozenset(["no-content",
