@@ -37,7 +37,7 @@ import uuid
 import zlib
 
 from collections import deque, defaultdict
-from functools import wraps, partial
+from functools import partial
 from urllib import urlencode
 from urllib2 import urlopen, Request, HTTPError
 from urlparse import urljoin
@@ -51,6 +51,7 @@ from twisted.names import client as dns_client
 from twisted.python.failure import Failure, DefaultException
 
 from oauth import oauth
+from ubuntu_sso.utils import timestamp_checker
 from ubuntuone import clientdefs
 from ubuntuone.platform import platform, remove_file
 from ubuntuone.storageprotocol import protocol_pb2, content_hash
@@ -72,18 +73,7 @@ UNKNOWN = type('', (), {'__repr__': lambda _: '---'})()
 EREGEX = "^.+\\@(\\[?)[a-zA-Z0-9\\-\\.]+\\.([a-zA-Z]{2,3}|[0-9]{1,3})(\\]?)$"
 
 # progress threshold to emit a download/upload progress event: 64Kb
-TRANSFER_PROGRESS_THRESHOLD = 64*1024*1024
-
-def passit(func):
-    """Pass the value on for the next deferred, while calling func with it."""
-
-    @wraps(func)
-    def wrapper(a):
-        """Do it."""
-        func(a)
-        return a
-
-    return wrapper
+TRANSFER_PROGRESS_THRESHOLD = 64 * 1024 * 1024
 
 
 class DeferredInterrupted(Exception):
@@ -116,7 +106,6 @@ class InterruptibleDeferred(defer.Deferred):
             self.errback(DeferredInterrupted())
 
 
-
 class PathLockingTree(object):
     """Tree that stores deferreds in the nodes."""
 
@@ -214,7 +203,6 @@ class PathLockingTree(object):
             node, element = branch.pop()
             del node['children_nodes'][element]
 
-
         # finally, log and release the deferred
         logger.debug("pathlock releasing %s; remaining: %d", elements,
                                                              self.count)
@@ -250,8 +238,8 @@ def sanitize_message(message):
         return ('start - processMessage: id: %s, type: %s',
                 message.id, message.type)
     elif message.type == protocol_pb2.Message.PUT_CONTENT:
-        lines = [ line for line in str(message).split("\n")
-                if not line.strip().startswith("magic_hash:") ]
+        lines = [line for line in str(message).split("\n")
+                if not line.strip().startswith("magic_hash:")]
         return ('start - processMessage: %s',
                 " ".join(lines))
     else:
@@ -461,12 +449,8 @@ class ZipQueue(object):
                 magic_hasher.update(data)
             upload.deflated_size = f.tell()
 
-            # keep the file open, but reset its position
-            # to zero, ready to be read later
-            f.seek(0)
-
             upload.magic_hash = magic_hasher.content_hash()
-        except Exception, e: # pylint: disable-msg=W0703
+        except Exception, e:  # pylint: disable-msg=W0703
             failed = True
             reactor.callFromThread(deferred.errback, e)
         finally:
@@ -705,10 +689,10 @@ class ActionQueue(ThrottlingStorageClientFactory, object):
         self.token = None
         self.consumer = None
 
-        self.client = None # an instance of self.protocol
+        self.client = None  # an instance of self.protocol
 
         # is a twisted.internet.tcp/ssl.Connector instance
-        self.connector = None # created on reactor.connectTCP/SSL
+        self.connector = None  # created on reactor.connectTCP/SSL
         # we need to track down if a connection is in progress
         # to avoid double connections
         self.connect_in_progress = False
@@ -1029,8 +1013,8 @@ class ActionQueue(ThrottlingStorageClientFactory, object):
     @defer.inlineCallbacks
     def authenticate(self):
         """Authenticate against the server using stored credentials."""
-        metadata = {'version':clientdefs.VERSION,
-                    'platform':platform}
+        metadata = {'version': clientdefs.VERSION,
+                    'platform': platform}
         authenticate_d = self._send_request_and_handle_errors(
             request=self.client.oauth_authenticate,
             request_error=protocol_errors.AuthenticationFailedError,
@@ -1752,14 +1736,17 @@ class CreateShare(ActionQueueCommand):
         if share_to and re.match(EREGEX, share_to):
             self.use_http = True
 
-    def _create_share_http(self, node_id, user, name, read_only, deferred):
+    def _create_share_http(self, node_id, user, name, read_only):
         """Create a share using the HTTP Web API method."""
 
         url = "https://one.ubuntu.com/files/api/offer_share/"
         method = oauth.OAuthSignatureMethod_PLAINTEXT()
+        timestamp = timestamp_checker.get_faithful_time()
+        parameters = {"oauth_timestamp": timestamp}
         request = oauth.OAuthRequest.from_consumer_and_token(
             http_url=url,
             http_method="POST",
+            parameters=parameters,
             oauth_consumer=self.action_queue.consumer,
             token=self.action_queue.token)
         request.sign_request(method, self.action_queue.consumer,
@@ -1771,24 +1758,15 @@ class CreateShare(ActionQueueCommand):
         pdata = urlencode(data)
         headers = request.to_header()
         req = Request(url, pdata, headers)
-        try:
-            urlopen(req)
-        except HTTPError, e:
-            reactor.callFromThread(deferred.errback, Failure(e))
-
-        reactor.callFromThread(deferred.callback, None)
+        urlopen(req)
 
     def _run(self):
         """Do the actual running."""
         if self.use_http:
             # External user, do the HTTP REST method
-            deferred = defer.Deferred()
-            d = threads.deferToThread(self._create_share_http,
+            return threads.deferToThread(self._create_share_http,
                                       self.node_id, self.share_to,
-                                      self.name, self.access_level != 'Modify',
-                                      deferred)
-            d.addErrback(deferred.errback)
-            return deferred
+                                      self.name, self.access_level != 'Modify')
         else:
             return self.action_queue.client.create_share(self.node_id,
                                                          self.share_to,
@@ -2078,7 +2056,6 @@ class GetDeltaFromScratch(ActionQueueCommand):
         self.log = mklog(logger, 'GetDeltaFromScratch', self.volume_id, None)
 
 
-
 class ChangePublicAccess(ActionQueueCommand):
     """Change the public access of a file."""
 
@@ -2103,9 +2080,12 @@ class ChangePublicAccess(ActionQueueCommand):
 
         url = "https://one.ubuntu.com/files/api/set_public/%s" % (node_key,)
         method = oauth.OAuthSignatureMethod_PLAINTEXT()
+        timestamp = timestamp_checker.get_faithful_time()
+        parameters = {"oauth_timestamp": timestamp}
         request = oauth.OAuthRequest.from_consumer_and_token(
             http_url=url,
             http_method="POST",
+            parameters=parameters,
             oauth_consumer=self.action_queue.consumer,
             token=self.action_queue.token)
         request.sign_request(method, self.action_queue.consumer,
@@ -2155,9 +2135,12 @@ class GetPublicFiles(ActionQueueCommand):
         """Get public files list using the HTTP Web API method."""
 
         method = oauth.OAuthSignatureMethod_PLAINTEXT()
+        timestamp = timestamp_checker.get_faithful_time()
+        parameters = {"oauth_timestamp": timestamp}
         request = oauth.OAuthRequest.from_consumer_and_token(
             http_url=self._url,
             http_method="GET",
+            parameters=parameters,
             oauth_consumer=self.action_queue.consumer,
             token=self.action_queue.token)
         request.sign_request(method, self.action_queue.consumer,
@@ -2498,6 +2481,11 @@ class Upload(ActionQueueCommand):
 
     def finish(self):
         """Release the semaphore if already acquired."""
+        if self.tempfile is not None:
+            # clean the temporary file
+            self.tempfile.close()
+            remove_file(self.tempfile.name)
+
         if self.tx_semaphore is not None:
             self.tx_semaphore = self.tx_semaphore.release()
             self.log.debug('semaphore released')
@@ -2509,7 +2497,7 @@ class Upload(ActionQueueCommand):
                                            share_id=self.share_id,
                                            node_id=self.node_id,
                                            hash=self.hash)
-
+        self.tempfile.seek(0)
         f = UploadProgressWrapper(self.tempfile, self)
 
         # access here the magic hash value, don't log anywhere, and
@@ -2521,9 +2509,7 @@ class Upload(ActionQueueCommand):
             upload_id=self.upload_id, upload_id_cb=self._upload_id_cb,
             magic_hash=magic_hash)
         self.upload_req = req
-        d = req.deferred
-        d.addBoth(passit(lambda _: self.tempfile.close()))
-        return d
+        return req.deferred
 
     def _upload_id_cb(self, upload_id):
         """Handle the received upload_id, save it in the metadata."""
@@ -2545,9 +2531,6 @@ class Upload(ActionQueueCommand):
 
     def handle_success(self, request):
         """It worked! Push the event."""
-        # remove the temporary file
-        remove_file(self.tempfile.name)
-
         # send the event
         d = dict(share_id=self.share_id, node_id=self.node_id, hash=self.hash,
                  new_generation=request.new_generation)
@@ -2563,7 +2546,6 @@ class Upload(ActionQueueCommand):
 
     def handle_failure(self, failure):
         """It didn't work! Push the event."""
-        remove_file(self.tempfile.name)
         self.action_queue.event_queue.push('AQ_UPLOAD_ERROR',
                                            error=failure.getErrorMessage(),
                                            share_id=self.share_id,
