@@ -25,7 +25,8 @@ import logging
 
 from ConfigParser import NoOptionError, NoSectionError
 from optparse import OptionParser
-from xdg.BaseDirectory import (
+from ubuntuone.platform import path_exists, rename
+from ubuntu_sso.xdg_base_directory import (
     load_config_paths,
     save_config_path,
     xdg_data_home,
@@ -75,26 +76,40 @@ _user_config_path = os.path.join(save_config_path('ubuntuone'), CONFIG_FILE)
 # this object is the shared config
 _user_config = None
 
+path_from_unix = lambda path: path.replace('/', os.path.sep)
+
 
 def home_dir_parser(value):
-    """ Parser for the root_dir and shares_dir options.
-    returns the path using user home + value.
+    """Parser for the root_dir and shares_dir options.
+
+    Return the path using user home + value.
+
     """
-    return os.path.expanduser(value)
+    result = os.path.expanduser(path_from_unix(value))
+    assert isinstance(result, str)
+    return result
 
 
 def xdg_cache_dir_parser(value):
-    """ Parser for the data_dir option.
-    returns the path using xdg_cache_home + value.
+    """Parser for the data_dir option.
+
+    Return the path using xdg_cache_home + value.
+
     """
-    return os.path.join(xdg_cache_home, value)
+    result = os.path.join(xdg_cache_home, path_from_unix(value))
+    assert isinstance(result, str)
+    return result
 
 
 def xdg_data_dir_parser(value):
-    """ Parser for the data_dir option.
-    returns the path using xdg_data_home + value.
+    """Parser for the data_dir option.
+
+    Return the path using xdg_data_home + value.
+
     """
-    return os.path.join(xdg_data_home, value)
+    result = os.path.join(xdg_data_home, path_from_unix(value))
+    assert isinstance(result, str)
+    return result
 
 
 def log_level_parser(value):
@@ -137,11 +152,11 @@ def get_config_files():
     config_files = []
     for xdg_config_dir in load_config_paths('ubuntuone'):
         config_file = os.path.join(xdg_config_dir, CONFIG_FILE)
-        if os.path.exists(config_file):
+        if path_exists(config_file):
             config_files.append(config_file)
 
         config_logs = os.path.join(xdg_config_dir, CONFIG_LOGS)
-        if os.path.exists(config_logs):
+        if path_exists(config_logs):
             config_files.append(config_logs)
 
     # reverse the list as load_config_paths returns the user dir first
@@ -149,11 +164,11 @@ def get_config_files():
     # if we are running from a branch, get the config files from it too
     config_file = os.path.join(os.path.dirname(__file__), os.path.pardir,
                                os.path.pardir, 'data', CONFIG_FILE)
-    if os.path.exists(config_file):
+    if path_exists(config_file):
         config_files.append(config_file)
 
     config_logs = os.path.join('data', CONFIG_LOGS)
-    if os.path.exists(config_logs):
+    if path_exists(config_logs):
         config_files.append(config_logs)
 
     return config_files
@@ -276,11 +291,11 @@ class _Config(SyncDaemonConfigParser):
         for section in [MAIN, THROTTLING, NOTIFICATIONS]:
             if self.has_section(section) and not self.options(section):
                 self.remove_section(section)
-        with open(self.config_file+'.new', 'w') as fp:
+        with open(self.config_file + '.new', 'w') as fp:
             self.write(fp)
-        if os.path.exists(self.config_file):
-            os.rename(self.config_file, self.config_file+'.old')
-        os.rename(self.config_file+'.new', self.config_file)
+        if path_exists(self.config_file):
+            rename(self.config_file, self.config_file + '.old')
+        rename(self.config_file + '.new', self.config_file)
 
     def get_parsed(self, section, option):
         """get that fallbacks to our custom defaults"""
