@@ -1,6 +1,7 @@
 # ubuntuone.syncdaemon.event_queue - Event queuing
 #
-# Author: Facundo Batista <facundo@canonical.com>
+# Authors: Facundo Batista <facundo@canonical.com>
+#          Manuel de la Pena <manuel@canonical.com>
 #
 # Copyright 2009-2011 Canonical Ltd.
 #
@@ -21,7 +22,9 @@ import collections
 import functools
 import logging
 
-from ubuntuone.platform import FilesystemMonitor
+from twisted.internet import defer
+
+from ubuntuone.platform import access, FilesystemMonitor
 
 # these are our internal events, what is inserted into the whole system
 EVENTS = {
@@ -202,9 +205,15 @@ class EventQueue(object):
         if not self.empty_event_queue_callbacks:
             self._have_empty_eq_cback = False
 
+    @defer.inlineCallbacks
     def shutdown(self):
         """Make the monitor shutdown."""
-        self.monitor.shutdown()
+        yield self.monitor.shutdown()
+        # clean up all registered listeners
+        if len(self.listener_map.items()) > 0:
+            for k, v in self.listener_map.items():
+                v.clear()
+                del self.listener_map[k]
 
     def rm_watch(self, dirpath):
         """Remove watch from a dir."""
@@ -212,11 +221,19 @@ class EventQueue(object):
 
     def add_watch(self, dirpath):
         """Add watch to a dir."""
-        self.monitor.add_watch(dirpath)
+        return self.monitor.add_watch(dirpath)
 
-    def has_watch(self, dirpath):
-        """Check if a dirpath is watched."""
-        return self.monitor.has_watch(dirpath)
+    def add_watches_to_udf_ancestors(self, volume):
+        """Add a inotify watch to volume's ancestors if it's an UDF."""
+        # This is a platform dependent operation since there are cases in
+        # which the watches do not have to be added (On windows we do not
+        # have to add them since we have an opened handle.)
+        # finally, check that UDF is ok in disk
+        if not access(volume.path):
+            # if we cannot access the UDF lets return false and do
+            # nothing
+            return defer.succeed(False)
+        return self.monitor.add_watches_to_udf_ancestors(volume)
 
     def unsubscribe(self, obj):
         """Remove the callback object from the listener queue.
