@@ -18,7 +18,6 @@
 """Module that implements the Events Nanny machinery."""
 
 import logging
-import functools
 
 class DownloadFinishedNanny(object):
     """Supervises the download finished signals.
@@ -33,7 +32,6 @@ class DownloadFinishedNanny(object):
         self.eq = eq
         self.hq = hq
         eq.subscribe(self)
-        self.eq_push = functools.partial(eq.push, "AQ_DOWNLOAD_FINISHED")
 
         # need to use a dict, to keep how many simultaneous opens we receive
         self._opened = {}
@@ -46,6 +44,11 @@ class DownloadFinishedNanny(object):
         # keep here those events we blocked
         self._blocked = {}
 
+    def eq_push(self, share_id, node_id, server_hash):
+        """Push the event to EQ."""
+        self.eq.push("AQ_DOWNLOAD_FINISHED", share_id=share_id,
+                     node_id=node_id, server_hash=server_hash)
+
     def handle_FS_DIR_MOVE(self, path_from, path_to):
         """Receives DIR_MOVE to change paths."""
         # add a "/" to path_from, to be nice to partial names
