@@ -715,16 +715,19 @@ class FileSystemManager(object):
                   path, mdobj['mdid'], share_id, node_id)
 
     def get_mdobjs_by_share_id(self, share_id, base_path=None):
-        """Returns all the mdobj that belongs to a share and it path
-        startswith base_path.
+        """Get all the mdobjs from a share.
+
+        If base_path is present, only return those who start with (or
+        are equal to) that path.
         """
         if base_path is None:
             base_path = self._get_share(share_id).path
+        compare_path = base_path + os.path.sep
+
         all_mdobjs = []
         # filter by path first, so we don't touch disk
-        for path in self._idx_path:
-            if path.startswith(base_path):
-                mdid = self._idx_path[path]
+        for path, mdid in self._idx_path.iteritems():
+            if path == base_path or path.startswith(compare_path):
                 mdobj = self.fs[mdid]
                 if mdobj["share_id"] == share_id:
                     all_mdobjs.append(_MDObject(**mdobj))
