@@ -1,9 +1,10 @@
 # ubuntuone.syncdaemon.hash_queue - hash queues
 #
-# Author: Facundo Batista <facundo@canonical.com>
-#         Guillermo Gonzalez <guillermo.gonzalez@canonical.com>
+# Authors: Facundo Batista <facundo@canonical.com>
+#          Guillermo Gonzalez <guillermo.gonzalez@canonical.com>
+#          Alejandro J. Cura <alecu@canonical.com>
 #
-# Copyright 2009 Canonical Ltd.
+# Copyright 2009-2011 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 3, as published
@@ -23,8 +24,10 @@ from __future__ import with_statement
 import logging
 import threading
 import Queue
+import time
 
 from twisted.internet import reactor
+from twisted.python.util import OrderedDict as TxOrderedDict
 
 from ubuntuone.storageprotocol.content_hash import \
     content_hash_factory, crc32
@@ -33,6 +36,37 @@ from ubuntuone.platform import (
     open_file,
     stat_path,
 )
+from ubuntuone.platform.constants import HASHQUEUE_DELAY
+
+
+NO_TIMESTAMP = None
+
+
+class FixedOrderedDict(TxOrderedDict):
+    """A t.p.u.OrderedDict that behaves like Python 2.7's OrderedDict."""
+
+    def popitem(self, last=False):
+        """Take out the first or last item, and return it."""
+        index = -1 if last else 0
+        key = self._order[index]
+        value = self[key]
+        del self[key]
+        return (key, value)
+
+    def clear(self):
+        """Remove every item from the dict."""
+        self._order = []
+        TxOrderedDict.clear(self)
+
+
+try:
+    # try to use the OrderedDict from stdlib >= 2.7
+    from collections import OrderedDict as StdlibOrderedDict
+    OrderedDict = StdlibOrderedDict
+except ImportError:
+    # if not available, use the patched one based on twisted
+    OrderedDict = FixedOrderedDict
+
 
 class StopHashing(Exception):
     """The current hash was cancelled."""
@@ -60,7 +94,7 @@ class _Hasher(threading.Thread):
         while True:
             if self._stopped:
                 break
-            info = self.queue.get()
+            info, timestamp = self.queue.get()
             if info is self.end_mark:
                 self._stopped = True
                 self.queue.task_done()
@@ -71,6 +105,13 @@ class _Hasher(threading.Thread):
                 self.hashing = path
             m = "Hasher: got file to hash: path %r  mdid %s"
             self.logger.debug(m, path, mdid)
+
+            now = time.time()
+            delta = timestamp - now
+            if delta > 0:
+                self.logger.trace("Waiting %f before starting hash", delta)
+                time.sleep(delta)
+
             try:
                 result = self._hash(path)
             except (IOError, OSError), e:
@@ -105,7 +146,8 @@ class _Hasher(threading.Thread):
         # for a new item
         self.queue.clear()
         # set the end_mark in case we are waiting a path
-        self.queue.put(self.end_mark)
+        item = (self.end_mark, NO_TIMESTAMP)
+        self.queue.put(item)
         self._stopped = True
 
     def _hash(self, path):
@@ -115,7 +157,7 @@ class _Hasher(threading.Thread):
         size = 0
         try:
             initial_stat = stat_path(path)
-            with open_file(path) as fh:
+            with open_file(path, 'rb') as fh:
                 while True:
                     # stop hashing if path_to_cancel == path or _stopped is True
                     with self.mutex:
@@ -159,6 +201,10 @@ class HashQueue(object):
         self.hasher.start()
         self.logger.info("HashQueue: _hasher started")
 
+    def _timestamp(self):
+        """A timestamp with a small delay into the future."""
+        return time.time() + HASHQUEUE_DELAY
+
     def insert(self, path, mdid):
         """Insert the path of a file to be hashed."""
         if self._stopped:
@@ -167,7 +213,8 @@ class HashQueue(object):
             return
         self.logger.debug("HashQueue: inserting path %r  mdid %s", path, mdid)
         self.hasher.cancel_if_running(path)
-        self._queue.put((path, mdid))
+        item = ((path, mdid), self._timestamp())
+        self._queue.put(item)
 
     def shutdown(self):
         """Shutdown all resources and clear the queue"""
@@ -196,28 +243,33 @@ class HashQueue(object):
 
 
 class UniqueQueue(Queue.Queue):
-    """Variant of Queue that only inserts unique items in the Queue
-    """
+    """Variant of Queue that only inserts unique items in the Queue."""
 
     def __init__(self, *args, **kwargs):
         """create the instance"""
         Queue.Queue.__init__(self, *args, **kwargs)
-        self._set = set() # to quickly check if a item is in the queue
         self.logger = logging.getLogger('ubuntuone.SyncDaemon.HQ.Queue')
 
+    def _init(self, maxsize):
+        """Override the underlaying data initialization."""
+        self.queue = OrderedDict()
+
+    def _qsize(self, len=len):
+        """The size of the queue."""
+        return len(self.queue)
+
     def _put(self, item):
-        """Custom _put that avoid duplicate items in the queue."""
-        if item not in self._set:
-            Queue.Queue._put(self, item)
-            self._set.add(item)
-        else:
-            self.logger.debug('Item already in the queue: %r', item)
+        """Custom _put that removes previous instances of this item."""
+        key, value = item
+        if key in self.queue:
+            # we must delete it first, so the new one is added to the end
+            del(self.queue[key])
+            self.logger.debug('Previous item removed from the queue: %r', key)
+        self.queue[key] = value
 
     def _get(self):
-        """Custom _get that removes the item from self._set."""
-        item = Queue.Queue._get(self)
-        self._set.discard(item)
-        return item
+        """Custom _get that returns the first (key, value) pair."""
+        return self.queue.popitem(last=False)
 
     def clear(self):
         """clear the internal queue and notify all blocked threads"""
@@ -226,6 +278,6 @@ class UniqueQueue(Queue.Queue):
             self.unfinished_tasks = 0
             self.all_tasks_done.notifyAll()
 
-    def __contains__(self, item):
-        """Tell if item in the queue."""
-        return item in self._set
+    def __contains__(self, key):
+        """Tell if a key is in the queue."""
+        return key in self.queue
