@@ -54,7 +54,7 @@ class IContentQueue(Interface):
         """
 
     def upload(share_id, node_id, previous_hash, hash, crc32,
-               size, path, fileobj_factory, tempfile_factory=None):
+               size, path, fileobj_factory):
         """
         Put the content of file-like object fileobj up in the given
         node.
