@@ -2,7 +2,7 @@
 #
 # Author: Facundo Batista <facundo@canonical.com>
 #
-# Copyright 2009 Canonical Ltd.
+# Copyright 2009-2011 Canonical Ltd.
 #
 # This program is free software: you can redistribute it and/or modify it
 # under the terms of the GNU General Public License version 3, as published
@@ -31,6 +31,7 @@ from twisted.internet import defer, reactor
 from ubuntuone.syncdaemon.interfaces import IMarker
 from ubuntuone.platform import (
     access,
+    is_link,
     listdir,
     path_exists,
     remove_file,
@@ -38,9 +39,11 @@ from ubuntuone.platform import (
     stat_path,
 )
 
+
 class ScanTransactionDirty(Exception):
     """The transaction was dirty."""
 
+
 class ScanNoDirectory(Exception):
     """The whole directory went away."""
 
@@ -52,6 +55,27 @@ log_debug = functools.partial(lr_logger.log, logging.DEBUG)
 log_error = functools.partial(lr_logger.log, logging.ERROR)
 log_warning = functools.partial(lr_logger.log, logging.WARNING)
 
+
+def is_valid_name(path):
+    """Tell if the name is valid.
+
+    This checks for the bytes in the path to be UTF8 valid before let them
+    get into Syncdaemon.
+
+    There is a similar check in platform/linux/filesystem_notification.py; we
+    don't use that because this is more platform independant, will all go away
+    when put everything in unicode, and most importantly because don't want to
+    send invalid notifications and don't write in the invalid log for *every*
+    local rescan.
+    """
+    try:
+        path.decode("utf8")
+    except UnicodeDecodeError:
+        return False
+    else:
+        return True
+
+
 class LocalRescan(object):
     """Local re-scanner.
 
@@ -205,7 +229,7 @@ class LocalRescan(object):
             log_error("The received path is not in any share!")
             raise ValueError("The received path is not in any share!")
 
-        # uglier than os.path.exists and isdir, but only hit the disk once
+        # uglier than path_exists and isdir, but only hit the disk once
         stat_result = get_stat(direct)
         if stat_result is None:
             m = "The received path is not in disk: path %r  mdid %s"
@@ -235,39 +259,6 @@ class LocalRescan(object):
             self._process_next_queue(None)
         return self._previous_deferred
 
-    def _add_watches_to_udf_ancestors(self, volume):
-        """Add a inotify watch to volume's ancestors if it's an UDF."""
-        added_watches = []
-
-        def revert_watches():
-            """Restore the just added watches and unsubscribe volume."""
-            for path in added_watches:
-                self.eq.rm_watch(path)
-            m = "Unsubscribing UDF %r because not in disk: %r"
-            log_info(m, volume.volume_id, volume.path)
-            self.vm.unsubscribe_udf(volume.volume_id)
-
-        for ancestor in volume.ancestors:
-            # check that ancestor is still there
-            if not access(ancestor):
-                log_info("Tree broken at path: %r", volume.path)
-                revert_watches()
-                return False
-
-            # add watch if needed
-            if not self.eq.has_watch(ancestor):
-                log_debug("Adding watch to UDF's ancestor %r", ancestor)
-                added_watches.append(ancestor)
-                self.eq.add_watch(ancestor)
-
-        # finally, check that UDF is ok in disk
-        if not access(volume.path):
-            revert_watches()
-            return False
-
-        # all is ok
-        return True
-
     def _process_next_queue(self, _):
         """Process the next item in the queue, if any."""
         log_debug("process next in queue (len %d)", len(self._queue))
@@ -280,14 +271,20 @@ class LocalRescan(object):
         # more to scan
         scan_info = self._queue.pop()
 
+        @defer.inlineCallbacks
         def safe_scan():
             """Scan safely"""
             try:
                 # add watches to UDF ancestors and check UDF is ok
                 volume = scan_info[0]
                 if isinstance(volume, volume_manager.UDF):
-                    udf_ok = self._add_watches_to_udf_ancestors(volume)
+                    udf_ok = yield self.eq.add_watches_to_udf_ancestors(volume)
                     if not udf_ok:
+                        # we need to ensure that the udf is not subscribed
+                        # when an error happens while adding the parent watches.
+                        m = "Unsubscribing UDF %r because not in disk: %r"
+                        log_info(m, volume.volume_id, volume.path)
+                        self.vm.unsubscribe_udf(volume.volume_id)
                         self._process_next_queue(None)
                         return
 
@@ -615,15 +612,13 @@ class LocalRescan(object):
                 filesdirs[fname] = is_dir, statinfo, changed
         return filesdirs
 
+    @defer.inlineCallbacks
     def _scan_one_dir(self, scan_info):
         """Gets one dir and compares with fsm."""
         share, dirpath, udf_mode = scan_info
 
-        if self.eq.has_watch(dirpath):
-            log_debug("Path already has watch: %r", dirpath)
-        else:
-            log_debug("Adding watch to %r", dirpath)
-            self.eq.add_watch(dirpath)
+        log_debug("Adding watch to %r", dirpath)
+        yield self.eq.add_watch(dirpath)
 
         to_later = []
         self.eq.freeze_begin(dirpath)
@@ -640,14 +635,22 @@ class LocalRescan(object):
             for something in dircontent:
                 fullname = os.path.join(dirpath, something)
                 stat_result = get_stat(fullname)
-                if stat_result is None or \
-                   stat.S_ISLNK(stat_result.st_mode):
+                if stat_result is None:
+                    # gone between the listdir and now
                     continue
-                elif not access(fullname):
+                if is_link(fullname):
+                    log_info("Ignoring path as it's a symlink: %r", fullname)
+                    continue
+                if not is_valid_name(fullname):
+                    m = "Ignoring path because it's invalid (non utf8): %r"
+                    log_info(m, fullname)
+                    continue
+                if not access(fullname):
                     log_warning("Ignoring path as we don't have enough "
                                 "permissions to track it: %r", fullname)
                     continue
-                elif stat.S_ISDIR(stat_result.st_mode):
+
+                if stat.S_ISDIR(stat_result.st_mode):
                     dnames.append(something)
                 elif stat.S_ISREG(stat_result.st_mode):
                     fnames.append(something)
@@ -701,4 +704,5 @@ class LocalRescan(object):
             d.addCallback(filter_delete_events)
         d.addCallback(self.eq.freeze_commit)
         d.addCallback(control)
-        return d
+        result = yield d
+        defer.returnValue(result)
