@@ -37,15 +37,14 @@ from ubuntuone.syncdaemon import (
     volume_manager,
 )
 from ubuntuone import syncdaemon, clientdefs
-import ubuntuone.platform
-
-from ubuntuone.platform import event_logging
+from ubuntuone.platform import event_logging, ExternalInterface
 from ubuntuone.syncdaemon import status_listener
 from ubuntuone.syncdaemon.states import StateManager, QueueManager
 
 
 class WaitingHelpingHandler(object):
     """An auxiliary class that helps wait for events."""
+
     def __init__(self, event_queue, waiting_events, waiting_kwargs,
                  result=None):
         self.deferred = defer.Deferred()
@@ -97,6 +96,9 @@ class Main(object):
 
         self.logger.info("Starting Ubuntu One client version %s",
                          clientdefs.VERSION)
+        self.logger.info("Using %r as root dir", self.root_dir)
+        self.logger.info("Using %r as data dir", self.data_dir)
+        self.logger.info("Using %r as shares root dir", self.shares_dir)
         self.db = tritcask.Tritcask(self.tritcask_dir)
         self.vm = volume_manager.VolumeManager(self)
         self.fs = filesystem_manager.FileSystemManager(
@@ -104,7 +106,7 @@ class Main(object):
         self.event_q = event_queue.EventQueue(self.fs, ignore_files)
         self.fs.register_eq(self.event_q)
 
-        # subscribe VM to EQ
+        # subscribe VM to EQ, to be unsubscribed in shutdown
         self.event_q.subscribe(self.vm)
         self.vm.init_root()
 
@@ -125,7 +127,7 @@ class Main(object):
         self.lr = local_rescan.LocalRescan(self.vm, self.fs,
                                            self.event_q, self.action_q)
 
-        self.external = ubuntuone.platform.ExternalInterface(
+        self.external = ExternalInterface(
             self, glib_loop, broadcast_events)
         self.external.oauth_credentials = oauth_credentials
         if user_config.get_autoconnect():
@@ -133,22 +135,24 @@ class Main(object):
 
         self.eventlog_listener = None
         self.start_event_logger()
+
+        self.status_listener = None
         self.start_status_listener()
-        self.logger.info("Using %r as root dir", self.root_dir)
-        self.logger.info("Using %r as data dir", self.data_dir)
-        self.logger.info("Using %r as shares root dir", self.shares_dir)
+
         self.mark = task.LoopingCall(self.log_mark)
         self.mark.start(mark_interval)
 
     def start_event_logger(self):
         """Start the event logger if it's available for this platform."""
         self.eventlog_listener = event_logging.get_listener(self.fs, self.vm)
+        # subscribe to EQ, to be unsubscribed in shutdown
         if self.eventlog_listener:
             self.event_q.subscribe(self.eventlog_listener)
 
     def start_status_listener(self):
         """Start the status listener if it is configured to start."""
         self.status_listener = status_listener.get_listener(self.fs, self.vm)
+        # subscribe to EQ, to be unsubscribed in shutdown
         if self.status_listener:
             self.event_q.subscribe(self.status_listener)
 
@@ -197,17 +201,26 @@ class Main(object):
         """Setup the daemon to be ready to run."""
         # hook the event queue to the root dir
         self.event_q.push('SYS_INIT_DONE')
-        self.event_q.add_watch(self.root_dir)
 
+    @defer.inlineCallbacks
     def shutdown(self, with_restart=False):
         """Shutdown the daemon; optionally restart it."""
         self.event_q.push('SYS_USER_DISCONNECT')
         self.event_q.push('SYS_QUIT')
-        self.event_q.shutdown()
-        self.hash_q.shutdown()
-        self.db.shutdown()
+
+        self.event_q.unsubscribe(self.vm)
+        if getattr(self, 'eventlog_listener', None) is not None:
+            self.event_q.unsubscribe(self.eventlog_listener)
+        if getattr(self, 'status_listener', None) is not None:
+            self.event_q.unsubscribe(self.status_listener)
+
+        # shutdown these before event_q so the listeners are unsubscribed
         self.state_manager.shutdown()
+        self.hash_q.shutdown()
         self.external.shutdown(with_restart)
+
+        yield self.event_q.shutdown()
+        self.db.shutdown()
         self.mark.stop()
 
     def restart(self):
