@@ -215,6 +215,24 @@ class FSKey(object):
         except Exception:
             return default
 
+    def is_subscribed(self):
+        """Tell if the node is in a subscribed volume or not."""
+        try:
+            volume_id = self.get('share_id')
+        except KeyError:
+            # node still not created, can get the parent only with path
+            if 'path' not in self.keys:
+                raise
+
+            # let's get the parent and its volume
+            parent_path = os.path.dirname(self.keys["path"])
+            parent = self.fs.get_by_path(parent_path)
+            volume_id = parent.share_id
+
+        # get the volume and return if subscribed
+        volume = self.fs.vm.get_volume(volume_id)
+        return volume.subscribed
+
 
 def loglevel(lvl):
     """Make a function that logs at lvl log level."""
@@ -924,6 +942,10 @@ class Sync(object):
         """on FS_FILE_CREATE"""
         key = FSKey(self.m.fs, path=path)
         log = FileLogger(self.logger, key)
+        if not key.is_subscribed():
+            log.debug("FS_FILE_CREATE on path %r discarded because of "
+                      "volume not subscribed", path)
+            return
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
         ssmr.on_event("FS_FILE_CREATE", {}, path)
 
@@ -931,6 +953,10 @@ class Sync(object):
         """on FS_DIR_CREATE"""
         key = FSKey(self.m.fs, path=path)
         log = FileLogger(self.logger, key)
+        if not key.is_subscribed():
+            log.debug("FS_DIR_CREATE on path %r discarded because of "
+                      "volume not subscribed", path)
+            return
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
         ssmr.on_event("FS_DIR_CREATE", {}, path)
 
@@ -938,6 +964,10 @@ class Sync(object):
         """on FS_FILE_DELETE"""
         key = FSKey(self.m.fs, path=path)
         log = FileLogger(self.logger, key)
+        if not key.is_subscribed():
+            log.debug("FS_FILE_DELETE on path %r discarded because of "
+                      "volume not subscribed", path)
+            return
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
         ssmr.on_event("FS_FILE_DELETE", {}, path)
 
@@ -945,6 +975,10 @@ class Sync(object):
         """on FS_DIR_DELETE"""
         key = FSKey(self.m.fs, path=path)
         log = FileLogger(self.logger, key)
+        if not key.is_subscribed():
+            log.debug("FS_DIR_DELETE on path %r discarded because of "
+                      "volume not subscribed", path)
+            return
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
         ssmr.on_event("FS_DIR_DELETE", {}, path)
 
@@ -952,6 +986,10 @@ class Sync(object):
         """on FS_FILE_MOVE"""
         key = FSKey(self.m.fs, path=path_from)
         log = FileLogger(self.logger, key)
+        if not key.is_subscribed():
+            log.debug("FS_FILE_MOVE on path %r discarded because of "
+                      "volume not subscribed", path_from)
+            return
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
         ssmr.on_event("FS_FILE_MOVE", {}, path_from, path_to)
 
@@ -959,6 +997,10 @@ class Sync(object):
         """on FS_DIR_MOVE"""
         key = FSKey(self.m.fs, path=path_from)
         log = FileLogger(self.logger, key)
+        if not key.is_subscribed():
+            log.debug("FS_DIR_MOVE on path %r discarded because of "
+                      "volume not subscribed", path_from)
+            return
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
         ssmr.on_event("FS_DIR_MOVE", {}, path_from, path_to)
 
@@ -996,6 +1038,10 @@ class Sync(object):
         """on FS_FILE_CLOSE_WRITE"""
         key = FSKey(self.m.fs, path=path)
         log = FileLogger(self.logger, key)
+        if not key.is_subscribed():
+            log.debug("FS_FILE_CLOSE_WRITE on path %r discarded because of "
+                      "volume not subscribed", path)
+            return
         ssmr = SyncStateMachineRunner(self.fsm, self.m, key, log)
         ssmr.on_event('FS_FILE_CLOSE_WRITE', {})
 
@@ -1151,10 +1197,10 @@ class Sync(object):
                     continue
 
                 # if the path changed, we have a move, notify it
-                node_parent_id = self.m.fs.get_by_path(
-                    self.m.fs.get_abspath(node.share_id,
-                                          os.path.dirname(node.path))
-                    ).node_id
+                path_parent_name = os.path.dirname(node.path)
+                abs_parent_path = self.m.fs.get_abspath(node.share_id,
+                                                        path_parent_name)
+                node_parent_id = self.m.fs.get_by_path(abs_parent_path).node_id
                 node_name = os.path.basename(node.path)
                 if dt.parent_id != node_parent_id or \
                         dt_name != node_name:
