@@ -202,6 +202,13 @@ class DataFile(object):
         """Return true if there is a hint file on-disk."""
         return os.path.exists(self.hint_filename)
 
+    @property
+    def hint_size(self):
+        """Return the hint file size."""
+        if self.has_hint:
+            return os.stat(self.hint_filename).st_size
+        return 0
+
     def exists(self):
         return os.path.exists(self.filename)
 
@@ -433,8 +440,8 @@ class HintFile(object):
         """Create the instance."""
         self.path = path
         self.tempfile = None
-        if os.path.exists(self.path):
-            # if it's there open only for read
+        if os.path.exists(self.path) and os.stat(self.path).st_size > 0:
+            # if it's there and size > 0, open only for read
             self.fd = open(self.path, 'rb')
         else:
             # this is a new hint file, lets create it as a tempfile.
@@ -755,8 +762,8 @@ class Tritcask(object):
         # sort the files by name, in order to load from older -> newest
         for data_file in sorted(self._immutable.values(),
                                 key=attrgetter('filename')):
-            if data_file.has_hint:
-                # load directly from the hint file.
+            if data_file.has_hint and data_file.hint_size > 0:
+                # load directly from the hint file, only if the size > 0
                 self._load_from_hint(data_file)
             elif data_file.exists() and data_file.size > 0:
                 self._load_from_data(data_file)
@@ -808,7 +815,8 @@ class Tritcask(object):
                 if build_hint:
                     hint_entry = HintEntry.from_tritcask_entry(entry)
                     hint_idx[hint_entry.key] = hint_entry
-        if build_hint:
+        if build_hint and hint_idx:
+            # only build the hint file if hint_idx contains data
             with data_file.get_hint_file() as hint_file:
                 for key, hint_entry in hint_idx.iteritems():
                     hint_file.write(hint_entry)
