@@ -117,6 +117,9 @@ class Volume(object):
 
     # default generation for all volumes without a value in the metadata
     generation = None
+    # transient value used in local rescan during resubscribe. Set it always to
+    # False no matter what is passed to the constructor
+    local_rescanning = False
 
     def __init__(self, volume_id, node_id, generation=None, subscribed=False):
         """Create the volume."""
@@ -229,7 +232,7 @@ class Share(Volume):
     @property
     def active(self):
         """Return True if this Share is accepted."""
-        return self.accepted and self.subscribed
+        return self.accepted and self.subscribed and not self.local_rescanning
 
     def __eq__(self, other):
         result = (super(Share, self).__eq__(other) and
@@ -342,7 +345,7 @@ class UDF(Volume):
     @property
     def active(self):
         """Returns True if the UDF is subscribed."""
-        return self.subscribed
+        return self.subscribed and not self.local_rescanning
 
     @classmethod
     def from_udf_volume(cls, udf_volume, path):
@@ -853,13 +856,13 @@ class VolumeManager(object):
             self._create_share_dir(share)
             self.log.debug('add_share: volume active, temporarly unsubscribe '
                            'to rescan (scan_local? %r).', share.can_write())
-            share.subscribed = False
+            share.local_rescanning = True
             self.shares[share.volume_id] = share
 
             def subscribe(result):
                 """Subscribe the share after the local rescan."""
                 volume = self.get_volume(share.volume_id)
-                volume.subscribed = True
+                volume.local_rescanning = False
                 self.store_volume(volume)
                 return result
 
@@ -1081,7 +1084,7 @@ class VolumeManager(object):
             if udf.subscribed:
                 self.log.debug('add_udf: volume subscribed, '
                                'temporarly unsubscribe to do local rescan.')
-                udf.subscribed = False
+                udf.local_rescanning = True
                 self.udfs[udf.volume_id] = udf
                 if not path_exists(udf.path):
                     make_dir(udf.path, recursive=True)
@@ -1094,7 +1097,7 @@ class VolumeManager(object):
 
                     """
                     volume = self.get_volume(udf.volume_id)
-                    volume.subscribed = True
+                    volume.local_rescanning = False
                     self.store_volume(volume)
                     return result
 
@@ -1268,11 +1271,14 @@ class VolumeManager(object):
 
             """
             volume = self.get_volume(volume_id)
-            volume.subscribed = True
+            volume.local_rescanning = False
             self.store_volume(volume)
             return result
 
         try:
+            volume.subscribed = True
+            volume.local_rescanning = True
+            self.store_volume(volume)
             d = self._scan_volume(volume, scan_local=volume.can_write())
         except KeyError, e:
             push_error(error="METADATA_DOES_NOT_EXIST")
